#
# Table structure for table `mob_skill_db_re`
#

DROP TABLE IF EXISTS `mob_skill_db_re`;
CREATE TABLE IF NOT EXISTS `mob_skill_db_re` (
  `MOB_ID` smallint(6) NOT NULL,
  `INFO` text NOT NULL,
  `STATE` text NOT NULL,
  `SKILL_ID` smallint(6) NOT NULL,
  `SKILL_LV` tinyint(4) NOT NULL,
  `RATE` smallint(4) NOT NULL,
  `CASTTIME` mediumint(9) NOT NULL,
  `DELAY` int(9) NOT NULL,
  `CANCELABLE` text NOT NULL,
  `TARGET` text NOT NULL,
  `CONDITION` text NOT NULL,
  `CONDITION_VALUE` text,
  `VAL1` mediumint(9) DEFAULT NULL,
  `VAL2` mediumint(9) DEFAULT NULL,
  `VAL3` mediumint(9) DEFAULT NULL,
  `VAL4` mediumint(9) DEFAULT NULL,
  `VAL5` mediumint(9) DEFAULT NULL,
  `EMOTION` text,
  `CHAT` text
) ENGINE=MyISAM;

# Mob Skill Database
# Based on Aegis Episode 11.3
#
# Structure of Database:
#REPLACE INTO `mob_skill_db_re` VALUES ( MobID,'Dummy value (info only)','State',SkillID,SkillLv,Rate,CastTime,Delay,'Cancelable','Target','Condition type','Condition value',val1,val2,val3,val4,val5,'Emotion','Chat');
#
# RATE: the chance of the skill being casted when the condition is fulfilled (10000 = 100%).
# DELAY: the time (in milliseconds) before attempting to recast the same skill.
#
# STATE:
#	any (except dead) / idle (in standby) / walk (in movement) / dead (on killed) /
#	loot /attack / angry (like attack, except player has not attacked mob yet) /
#	chase (following target, after being attacked) / follow (following target,
#	without being attacked) / anytarget (attack+angry+chase+follow)
#
# TARGET:
#	target (current target) / self / friend / master / randomtarget (any enemy within skill's range)
#
#	The following are for ground-skills, a random target tile is selected from the specified area:
#	    around1 (3x3 area around self) / around2 (5x5 area around self) /
#	    around3 (7x7 area around self) / around4 (9x9 area around self) /
#	    around5 (3x3 area around target) / around6 (5x5 area around target) /
#	    around7 (7x7 area around target) / around8 (9x9 area around target) /
#	    around = around4
#
# CONDITION:
#	always			Unconditional (no condition value).
#	onspawn			When mob spawns/respawns (no condition value).
#	myhpltmaxrate		When mob's HP drops to the specified %.
#	myhpinrate		When mob's HP is in a certain % range (condition value = lower bound, val1 = upper bound).
#	mystatuson		If mob has the specified abnormality in status.
#	mystatusoff		If mob has ended the specified abnormality in status.
#	friendhpltmaxrate	When mob's friend's HP drops to the specified %.
#	friendhpinrate		When mob's friend's HP is in a certain % range (condition value = lower bound, val1 = upper bound).
#	friendstatuson		If friend has the specified abnormality in status.
#	friendstatusoff		If friend has ended the specified abnormality in status.
#	attackpcgt		When attack PCs become greater than specified number.
#	attackpcge		When attack PCs become greater than or equal to the specified number.
#	slavelt			When number of slaves is less than the original specified number.
#	slavele			When number of slaves is less than or equal to the original specified number.
#	closedattacked		When close range melee attacked (no condition value).
#	longrangeattacked	When long range attacked, ex. bows, guns, ranged skills (no condition value).
#	skillused		When the specified skill is used on the mob.
#	afterskill		After mob casts the specified skill.
#	casttargeted		When a target is in cast range (no condition value).
#	rudeattacked		When mob is rude attacked (no condition value).
#
#	Status abnormalities specified through the statuson/statusoff system:
#	    anybad (any type of state change) / stone / freeze / stun / sleep /
#	    poison / curse / silence / confusion / blind / hiding / sight (unhidden)
#
# Note: if a negative MobID is provided, the skill will be treated as 'global':
#	-1: added for all boss types.
#	-2: added for all normal types.
#	-3: added for all mobs.

REPLACE INTO `mob_skill_db_re` VALUES (1001,'Scorpion@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1001,'Scorpion@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1002,'Poring@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1002,'Poring@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1004,'Hornet@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1004,'Hornet@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1005,'Familiar@NPC_BLINDATTACK','attack',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1005,'Familiar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1005,'Familiar@NPC_BLINDATTACK','angry',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1005,'Familiar@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1007,'Fabre@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1007,'Fabre@NPC_METAMORPHOSIS','idle',193,1,10,2000,5000,'no','self','always','0',1008,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1008,'Pupa@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1008,'Pupa@NPC_METAMORPHOSIS','idle',193,1,10,2000,5000,'no','self','always','0',1018,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1009,'Condor@BS_ADRENALINE','attack',111,1,500,1500,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1009,'Condor@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',5,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1009,'Condor@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1010,'Willow@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1010,'Willow@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1010,'Willow@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1011,'Chonchon@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1011,'Chonchon@NPC_RUN','attack',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1011,'Chonchon@NPC_RUN','chase',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1012,'Roda Frog@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1013,'Wolf@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1013,'Wolf@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1014,'Spore@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1014,'Spore@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1014,'Spore@NPC_SLEEPATTACK','attack',182,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1014,'Spore@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1015,'Zombie@NPC_POISON','attack',176,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1015,'Zombie@NPC_POISON','angry',176,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1015,'Zombie@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1015,'Zombie@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1016,'Archer Skeleton@AC_DOUBLE','attack',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1016,'Archer Skeleton@NPC_UNDEADATTACK','attack',347,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1016,'Archer Skeleton@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1018,'Creamy@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1018,'Creamy@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1018,'Creamy@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1018,'Creamy@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1019,'Peco Peco@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1019,'Peco Peco@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1019,'Peco Peco@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1019,'Peco Peco@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1020,'Mandragora@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_SMOKING','idle',195,1,500,0,36000000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always','0',27,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1023,'Orc Warrior@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1024,'Worm Tail@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1025,'Snake@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1025,'Snake@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1026,'Munak@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1026,'Munak@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1026,'Munak@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1026,'Munak@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1026,'Munak@NPC_STUNATTACK','angry',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1028,'Skeleton Soldier@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1029,'Isis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1029,'Isis@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1029,'Isis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1029,'Isis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1030,'Anacondaq@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1030,'Anacondaq@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1031,'Poporing@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1031,'Poporing@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1031,'Poporing@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1032,'Verit@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1032,'Verit@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1032,'Verit@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1033,'Elder Willow@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1033,'Elder Willow@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1033,'Elder Willow@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1033,'Elder Willow@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1033,'Elder Willow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1034,'Thara Frog@NPC_WATERATTACK','attack',184,3,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1035,'Hunter Fly@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1036,'Ghoul@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1036,'Ghoul@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1037,'Side Winder@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1037,'Side Winder@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1037,'Side Winder@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@AS_VENOMDUST','attack',140,5,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@AS_VENOMDUST','attack',140,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@MG_STONECURSE','attack',16,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@ASC_METEORASSAULT','attack',406,10,10000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_CURSEATTACK','attack',181,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_CURSEATTACK','chase',181,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_DARKBREATH','attack',202,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_DARKBREATH','chase',202,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_POISONATTACK','attack',188,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1522,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1522,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_SUMMONSLAVE','idle',196,7,10000,0,0,'no','self','onspawn','0',1029,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@SM_BASH','attack',5,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_DARKTHUNDER','chase',341,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@NPC_DARKTHUNDER','chase',341,10,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1038,'Osiris@WZ_QUAGMIRE','chase',92,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@MO_BODYRELOCATION','chase',264,1,2000,200,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_DARKSTRIKE','chase',340,10,2000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_EARTHQUAKE','chase',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_EARTHQUAKE','attack',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_HELLJUDGEMENT','chase',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_HELLJUDGEMENT','attack',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_DARKBREATH','attack',202,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',1101,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@WZ_VERMILION','chase',85,21,5000,500,2000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1039,'Baphomet@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1040,'Golem@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1040,'Golem@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1040,'Golem@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1040,'Golem@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1041,'Mummy@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1042,'Steel Chonchon@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1042,'Steel Chonchon@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1044,'Obeaune@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1044,'Obeaune@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1044,'Obeaune@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1044,'Obeaune@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1044,'Obeaune@PR_LEXDIVINA','chase',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_STUNATTACK','angry',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1045,'Marc@NPC_WATERATTACK','angry',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_HEAL','attack',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_HEAL','chase',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@LK_SPIRALPIERCE','attack',397,5,10000,1000,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@CR_AUTOGUARD','attack',249,10,2000,500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@CR_AUTOGUARD','chase',249,10,2000,500,300000,'no','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_DECAGI','attack',30,48,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@AL_DECAGI','chase',30,48,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',1427,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@NPC_SUMMONSLAVE','idle',196,1,10000,700,10000,'no','self','slavele','3',1427,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1046,'Doppelganger@BS_HAMMERFALL','chase',110,10,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1047,'PecoPeco Egg@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1047,'PecoPeco Egg@NPC_METAMORPHOSIS','idle',193,1,10,2000,5000,'no','self','always','0',1049,1050,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1048,'Thief Bug Egg@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1048,'Thief Bug Egg@NPC_METAMORPHOSIS','idle',193,4,10,2000,5000,'no','self','always','0',1051,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1049,'Picky@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1049,'Picky@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1050,'Shell Picky@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1050,'Shell Picky@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1051,'Thief Bug Larva@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1051,'Thief Bug Larva@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1052,'Rocker@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1052,'Rocker@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1053,'Thief Bug Female@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1053,'Thief Bug Female@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1054,'Thief Bug Male@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1054,'Thief Bug Male@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1055,'Muka@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1055,'Muka@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1055,'Muka@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1056,'Smokie@AL_HEAL','idle',28,5,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1056,'Smokie@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1056,'Smokie@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1056,'Smokie@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1056,'Smokie@TF_HIDING','attack',51,1,500,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@AM_POTIONPITCHER','idle',231,1,500,1500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@SM_PROVOKE','chase',6,10,500,600,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1057,'Yoyo@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1058,'Metaller@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1058,'Metaller@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1058,'Metaller@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1058,'Metaller@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_HEAL','attack',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_HEAL','chase',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_PNEUMA','attack',25,1,2000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_PNEUMA','chase',25,1,2000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SILENCEATTACK','chase',178,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1604,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1604,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',1156,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@NPC_STOP','attack',342,1,2000,0,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@WZ_JUPITEL','attack',84,28,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1059,'Mistress@WZ_JUPITEL','chase',84,28,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1060,'Bigfoot@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1060,'Bigfoot@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1060,'Bigfoot@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1060,'Bigfoot@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1060,'Bigfoot@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1061,'Nightmare@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1061,'Nightmare@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1061,'Nightmare@NPC_MENTALBREAKER','attack',159,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1061,'Nightmare@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1062,'Santa Poring@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1062,'Santa Poring@NPC_HOLYATTACK','attack',189,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1063,'Lunatic@NPC_RUN','attack',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1063,'Lunatic@NPC_RUN','chase',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1064,'Megalodon@NPC_CHANGETELEKINESIS','attack',169,1,500,2000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1064,'Megalodon@NPC_UNDEADATTACK','attack',347,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_FROSTDIVER','attack',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_FROSTDIVER','chase',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_LIGHTNINGBOLT','attack',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_LIGHTNINGBOLT','chase',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@SA_DISPELL','attack',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@SA_DISPELL','chase',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_FROSTDIVER','angry',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_FROSTDIVER','follow',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_LIGHTNINGBOLT','angry',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@MG_LIGHTNINGBOLT','follow',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@SA_DISPELL','angry',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@SA_DISPELL','follow',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1065,'Strouf@WZ_WATERBALL','angry',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1066,'Vadon@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1066,'Vadon@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1066,'Vadon@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1066,'Vadon@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1067,'Cornutus@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1067,'Cornutus@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1067,'Cornutus@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1067,'Cornutus@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1068,'Hydra@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@WZ_WATERBALL','angry',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1069,'Swordfish@WZ_WATERBALL','follow',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1070,'Kukre@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1070,'Kukre@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1071,'Pirate Skeleton@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1071,'Pirate Skeleton@SM_BASH','angry',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1071,'Pirate Skeleton@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@MG_FIREBOLT','angry',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@MG_FIREBOLT','follow',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1072,'Kaho@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1073,'Crab@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1074,'Shellfish@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1076,'Skeleton@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1076,'Skeleton@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_POISONATTACK','angry',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1077,'Poison Spore@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@HP_ASSUMPTIO','idle',361,5,10000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@HP_ASSUMPTIO','attack',361,5,10000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@MC_MAMMONITE','attack',42,22,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@SM_MAGNUM','attack',7,25,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@MG_FIREBALL','chase',17,44,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@NPC_SUMMONSLAVE','attack',196,10,10000,700,10000,'no','self','slavele','5',1608,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@NPC_SUMMONSLAVE','idle',196,10,10000,700,10000,'no','self','slavele','5',1608,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@NPC_SUMMONSLAVE','idle',196,10,10000,0,0,'no','self','onspawn','0',1054,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@TF_HIDING','attack',51,1,1000,0,5000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1086,'Golden Thief Bug@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@AL_DECAGI','chase',30,48,2000,0,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_ARMORBRAKE','attack',344,10,2000,500,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_ARMORBRAKE','chase',344,10,2000,500,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@LK_SPIRALPIERCE','attack',397,5,1500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@MG_THUNDERSTORM','attack',21,20,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@MG_THUNDERSTORM','chase',21,20,10000,1500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@CR_AUTOGUARD','attack',249,10,2000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_SUMMONSLAVE','attack',196,8,10000,700,10000,'no','self','slavele','3',1439,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@NPC_SUMMONSLAVE','idle',196,8,10000,700,10000,'no','self','slavele','3',1439,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1087,'Orc Hero@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1052,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1052,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1088,'Vocal@NPC_SUMMONSLAVE','idle',196,8,10000,0,0,'no','self','onspawn','0',1052,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1089,'Toad@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1089,'Toad@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1012,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1089,'Toad@NPC_SUMMONSLAVE','idle',196,6,10000,0,0,'no','self','onspawn','0',1012,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1089,'Toad@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1090,'Mastering@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1090,'Mastering@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1002,1113,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1090,'Mastering@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1002,1113,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1090,'Mastering@NPC_SUMMONSLAVE','idle',196,10,10000,0,0,'no','self','onspawn','0',1002,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1090,'Mastering@NPC_WATERATTACK','attack',184,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','0',1011,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','0',1011,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_SUMMONSLAVE','idle',196,8,10000,0,0,'no','self','onspawn','0',1011,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1091,'Dragon Fly@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',27,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1013,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1013,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1092,'Vagabond Wolf@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1013,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1093,'Eclipse@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1093,'Eclipse@NPC_LICK','attack',206,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1093,'Eclipse@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1063,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1093,'Eclipse@NPC_SUMMONSLAVE','idle',196,8,10000,0,0,'no','self','onspawn','0',1063,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1094,'Ambernite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1094,'Ambernite@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1094,'Ambernite@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1095,'Andre@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1095,'Andre@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@MG_SAFETYWALL','attack',12,5,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_HOLYATTACK','attack',189,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_HOLYATTACK','chase',189,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1031,1062,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1031,1062,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@NPC_SUMMONSLAVE','idle',196,3,10000,0,0,'no','self','onspawn','0',1031,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1096,'Angeling@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1097,'Ant Egg@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1097,'Ant Egg@NPC_METAMORPHOSIS','idle',193,1,10,2000,5000,'no','self','always','0',1095,1105,1160,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_DARKBREATH','attack',202,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@MG_THUNDERSTORM','attack',21,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@MG_THUNDERSTORM','chase',21,5,1000,1500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@AL_HEAL','attack',28,5,5000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1098,'Anubis@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1099,'Argiope@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1099,'Argiope@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1099,'Argiope@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1099,'Argiope@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1099,'Argiope@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1100,'Argos@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1100,'Argos@PF_SPIDERWEB','idle',405,1,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1100,'Argos@PF_SPIDERWEB','attack',405,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1100,'Argos@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@SA_DISPELL','attack',289,1,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1101,'Baphomet Jr.@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',30,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1102,'Bathory@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1103,'Caramel@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1103,'Caramel@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1104,'Coco@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1104,'Coco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1104,'Coco@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1105,'Deniro@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1105,'Deniro@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1106,'Desert Wolf@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1106,'Desert Wolf@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1106,'Desert Wolf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1106,'Desert Wolf@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1107,'Baby Desert Wolf@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1107,'Baby Desert Wolf@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1108,'Deviace@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1108,'Deviace@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1108,'Deviace@WZ_WATERBALL','chase',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1108,'Deviace@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1108,'Deviace@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_DARKTHUNDER','attack',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1109,'Deviruchi@NPC_DARKTHUNDER','chase',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1110,'Dokebi@BS_ADRENALINE','attack',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1110,'Dokebi@BS_ADRENALINE','chase',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1110,'Dokebi@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1110,'Dokebi@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1110,'Dokebi@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1111,'Drainliar@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1111,'Drainliar@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1111,'Drainliar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_ARMORBRAKE','attack',344,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_AGIUP','attack',350,10,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1566,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1566,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',1192,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@WZ_WATERBALL','attack',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1112,'Drake@WZ_WATERBALL','chase',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1113,'Drops@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1113,'Drops@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1114,'Dustiness@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1114,'Dustiness@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1114,'Dustiness@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1114,'Dustiness@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@MG_FIREBALL','chase',17,44,4000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@MG_FIREBALL','chase',17,44,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_BLEEDING','attack',660,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AM_DEMONSTRATION','attack',229,5,2000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AM_DEMONSTRATION','chase',229,5,2000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_SUMMONSLAVE','attack',196,8,10000,700,10000,'no','self','slavele','3',1603,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_SUMMONSLAVE','idle',196,8,10000,700,10000,'no','self','slavele','3',1603,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1060,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@SM_MAGNUM','attack',7,25,3000,0,2000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1115,'Eddga@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1116,'Eggyra@NPC_BLINDATTACK','attack',177,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1116,'Eggyra@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1116,'Eggyra@CR_AUTOGUARD','attack',249,2,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1116,'Eggyra@CR_AUTOGUARD','chase',249,2,2000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@NPC_UNDEADATTACK','chase',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1117,'Evil Druid@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1118,'Flora@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1118,'Flora@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1118,'Flora@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1119,'Frilldora@NPC_FIREATTACK','angry',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@AL_TELEPORT','attack',26,1,5000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_INVISIBLE','attack',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_INVISIBLE','idle',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_INVISIBLE','chase',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@MG_SOULSTRIKE','chase',13,9,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_SUMMONSLAVE','attack',196,5,10000,2000,10000,'no','self','slavele','3',1599,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_SUMMONSLAVE','idle',196,5,10000,2000,10000,'no','self','slavele','3',1599,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1186,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1120,'Ghostring@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1121,'Giearth@HT_SANDMAN','idle',119,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1121,'Giearth@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',8,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1121,'Giearth@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1121,'Giearth@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1122,'Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',18,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1122,'Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1122,'Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1122,'Goblin@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1123,'Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1123,'Goblin@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1123,'Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1123,'Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1124,'Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1124,'Goblin@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1124,'Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1124,'Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1125,'Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1125,'Goblin@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1125,'Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1125,'Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1126,'Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1126,'Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1126,'Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1126,'Goblin@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1127,'Hode@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1127,'Hode@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1127,'Hode@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1128,'Horn@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1128,'Horn@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1129,'Horong@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1129,'Horong@MG_SIGHT','chase',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1129,'Horong@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1129,'Horong@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1129,'Horong@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1130,'Jakk@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@MG_FIREBALL','chase',17,5,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'10',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@MG_FROSTDIVER','chase',15,9,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'10',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@MG_SOULSTRIKE','chase',13,9,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_ATTRICHANGE','idle',161,1,500,2000,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'10',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_ENERGYDRAIN','chase',200,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_WINDATTACK','chase',187,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_DARKTHUNDER','chase',341,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_EMOTION_ON','idle',474,1,100,0,60000,'yes','self','always','0',20,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_EMOTION','idle',197,1,100,0,60000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_EMOTION','attack',197,1,10000,0,60000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1131,'Joker@NPC_EMOTION_ON','idle',474,1,100,0,60000,'yes','self','always','0',22,0x83,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1132,'Khalitzburg@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1133,'Kobold 1@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1133,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1133,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1133,'Kobold@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1134,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1134,'Kobold@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1134,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1134,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1135,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1135,'Kobold@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1135,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1135,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1138,'Magnolia@HT_FLASHER','idle',120,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1138,'Magnolia@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1138,'Magnolia@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1138,'Magnolia@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1139,'Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1139,'Mantis@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1139,'Mantis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1139,'Mantis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1140,'Marduk@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1141,'Marina@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1141,'Marina@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1141,'Marina@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_RUN','idle',354,7,10000,0,30000,'no','master','alchemist',NULL,NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_SELFDESTRUCTION','any',173,1,10000,3000,0,'no','self','afterskill','354',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_SELFDESTRUCTION','idle',173,1,500,2000,5000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_SELFDESTRUCTION','idle',173,1,10000,2000,5000,'no','self','skillused','173',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_SELFDESTRUCTION','idle',173,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1142,'Marine Sphere@NPC_SPEEDUP','any',332,1,10000,0,700,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1143,'Marionette@HT_FREEZINGTRAP','idle',121,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1143,'Marionette@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1143,'Marionette@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1144,'Marse@NPC_BLINDATTACK','attack',177,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1144,'Marse@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1144,'Marse@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1144,'Marse@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1145,'Martin@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1145,'Martin@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1145,'Martin@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1146,'Matyr@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1146,'Matyr@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1146,'Matyr@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1146,'Matyr@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_HEAL','attack',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_HEAL','attack',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_HEAL','chase',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_HEAL','chase',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@AL_TELEPORT','walk',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@CR_AUTOGUARD','attack',249,10,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1477,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1477,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@NPC_SUMMONSLAVE','idle',196,10,10000,0,0,'no','self','onspawn','0',1194,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@WZ_HEAVENDRIVE','attack',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@WZ_HEAVENDRIVE','chase',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1147,'Maya@WZ_HEAVENDRIVE','chase',91,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1148,'Medusa@MG_STONECURSE','attack',16,10,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1148,'Medusa@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1148,'Medusa@NPC_PETRIFYATTACK','chase',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1148,'Medusa@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1148,'Medusa@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1149,'Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_HEAL','attack',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_HEAL','attack',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_HEAL','chase',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_HEAL','chase',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@SA_LANDPROTECTOR','chase',288,1,10000,1000,5000,'no','target','skillused','89',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@SA_LANDPROTECTOR','attack',288,1,10000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MC_MAMMONITE','attack',42,22,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_COLDBOLT','chase',14,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_COLDBOLT','chase',14,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_FIREBOLT','chase',19,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_FIREBOLT','chase',19,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_LIGHTNINGBOLT','chase',20,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@MG_LIGHTNINGBOLT','chase',20,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@SA_DISPELL','attack',289,5,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@SA_DISPELL','chase',289,5,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1471,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1471,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@NPC_SUMMONSLAVE','idle',196,3,10000,0,0,'no','self','onspawn','0',1180,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@BS_HAMMERFALL','chase',110,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1150,'Moonlight Flower@BS_HAMMERFALL','attack',110,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1151,'Myst@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1151,'Myst@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1151,'Myst@NPC_MENTALBREAKER','attack',159,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1151,'Myst@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1152,'Orc Skeleton@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1152,'Orc Skeleton@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1152,'Orc Skeleton@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1152,'Orc Skeleton@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1153,'Orc Zombie@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1153,'Orc Zombie@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1153,'Orc Zombie@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1153,'Orc Zombie@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1154,'Pasana@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1155,'Petite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1155,'Petite@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1155,'Petite@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1155,'Petite@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1155,'Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1156,'Petite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1156,'Petite@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1156,'Petite@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1156,'Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1156,'Petite@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@AC_CHARGEARROW','attack',148,1,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@MG_THUNDERSTORM','chase',21,9,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_DARKSTRIKE','attack',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_DARKSTRIKE','chase',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_DARKBLESSING','attack',203,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_ENERGYDRAIN','attack',200,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_ENERGYDRAIN','chase',200,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_ENERGYDRAIN','chase',200,1,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1458,1464,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1157,'Pharaoh@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1458,1464,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1158,'Phen@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1158,'Phen@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_WIDESTONE','chase',666,5,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_WIDESTONE','attack',666,5,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_HELMBRAKE','attack',345,10,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_LICK','attack',206,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_PETRIFYATTACK','chase',180,5,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1558,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1558,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@NPC_SUMMONSLAVE','idle',196,10,10000,0,0,'no','self','onspawn','0',1127,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@WZ_HEAVENDRIVE','chase',91,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1159,'Phreeoni@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1160,'Piere@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1160,'Piere@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1161,'Plankton@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1163,'Raydric@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1164,'Requiem@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1164,'Requiem@NPC_EMOTION','walk',197,1,200,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1164,'Requiem@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@NPC_GROUNDATTACK','angry',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1165,'Sandman@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1166,'Savage@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1166,'Savage@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1166,'Savage@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1167,'Savage Babe@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1167,'Savage Babe@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1169,'Skeleton Worker@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1169,'Skeleton Worker@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1169,'Skeleton Worker@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1170,'Sohee@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1170,'Sohee@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1170,'Sohee@NPC_SUICIDE','attack',175,1,500,0,5000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1170,'Sohee@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1174,'Stainer@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1174,'Stainer@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1174,'Stainer@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1175,'Tarou@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1175,'Tarou@NPC_POISON','attack',176,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1176,'Vitata@AM_POTIONPITCHER','attack',231,2,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1176,'Vitata@AM_POTIONPITCHER','attack',231,2,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1176,'Vitata@AM_POTIONPITCHER','chase',231,2,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1176,'Vitata@AM_POTIONPITCHER','chase',231,2,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1176,'Vitata@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1177,'Zenorc@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1177,'Zenorc@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1177,'Zenorc@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1178,'Zerom@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',28,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1179,'Whisper@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1180,'Nine-Tail@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1180,'Nine-Tail@NPC_ATTRICHANGE','attack',161,1,500,2000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1180,'Nine-Tail@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1180,'Nine-Tail@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1180,'Nine-Tail@NPC_RANDOMATTACK','attack',183,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1188,'Bon Gun@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1188,'Bon Gun@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1189,'Orc Archer@AC_SHOWER','attack',47,5,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1189,'Orc Archer@HT_ANKLESNARE','idle',117,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1189,'Orc Archer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',5,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1189,'Orc Archer@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1189,'Orc Archer@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@AL_INCAGI','chase',29,10,2000,700,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@CR_SHIELDCHARGE','attack',250,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_EARTHQUAKE','chase',653,5,10000,1500,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_EARTHQUAKE','attack',653,5,10000,1500,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@MG_COLDBOLT','chase',14,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@MG_FIREBOLT','chase',19,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1473,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1473,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1189,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@PR_LEXDIVINA','chase',76,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@PR_LEXDIVINA','chase',76,10,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@SM_PROVOKE','chase',6,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1190,'Orc Lord@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1191,'Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1191,'Mimic@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1191,'Mimic@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1191,'Mimic@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1192,'Wraith@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1192,'Wraith@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1192,'Wraith@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1192,'Wraith@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1193,'Alarm@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1193,'Alarm@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1193,'Alarm@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1193,'Alarm@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1193,'Alarm@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1194,'Arclouse@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1194,'Arclouse@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1194,'Arclouse@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1194,'Arclouse@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1195,'Rideword@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1195,'Rideword@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1195,'Rideword@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1196,'Skeleton Prisoner@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1196,'Skeleton Prisoner@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1197,'Zombie Prisoner@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1197,'Zombie Prisoner@NPC_UNDEADATTACK','attack',347,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1198,'Dark Priest@NPC_DARKCROSS','attack',338,10,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1198,'Dark Priest@NPC_CHANGEUNDEAD','attack',348,5,1000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1198,'Dark Priest@MO_BODYRELOCATION','chase',264,1,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1198,'Dark Priest@NPC_DARKBREATH','chase',202,5,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1198,'Dark Priest@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1199,'Punk@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1199,'Punk@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1199,'Punk@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1199,'Punk@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',30,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@NPC_LICK','attack',206,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1257,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1200,'Zherlthsh@NPC_STOP','attack',342,1,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1201,'Rybio@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1201,'Rybio@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1201,'Rybio@SM_PROVOKE','chase',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1202,'Phendark@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1202,'Phendark@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1202,'Phendark@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@NPC_SHIELDBRAKE','attack',346,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1203,'Mysteltainn@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@NPC_HELMBRAKE','attack',345,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1204,'Tyrfing@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1205,'Executioner@NPC_ARMORBRAKE','attack',344,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1206,'Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1206,'Anolian@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1206,'Anolian@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1206,'Anolian@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1206,'Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@AM_POTIONPITCHER','idle',231,4,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',20,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'10',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1207,'Sting@WZ_QUAGMIRE','chase',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1208,'Wanderer@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1209,'Cramp@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1209,'Cramp@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1211,'Brilight@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1211,'Brilight@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1211,'Brilight@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1211,'Brilight@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@CR_SHIELDCHARGE','attack',250,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1213,'High Orc@NPC_SMOKING','idle',195,1,500,0,36000000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@AM_POTIONPITCHER','idle',231,1,500,1500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@HT_SKIDTRAP','idle',115,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@NPC_SUMMONSLAVE','attack',196,3,10000,2000,60000,'no','self','slavele','0',1057,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1057,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@SM_PROVOKE','chase',6,10,500,600,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1214,'Choco@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1215,'Stem Worm@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1215,'Stem Worm@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1216,'Penomena@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1216,'Penomena@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1216,'Penomena@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@AC_CHARGEARROW','chase',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@KN_BRANDISHSPEAR','attack',57,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@NPC_DARKNESSATTACK','attack',190,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@NPC_SUMMONSLAVE','idle',196,2,10000,2000,600000,'no','self','slavele','0',1132,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@NPC_ARMORBRAKE','attack',344,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1219,'Knight of Abyss@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1242,'Marin@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1242,'Marin@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1242,'Marin@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1242,'Marin@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1243,'Sasquatch@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1243,'Sasquatch@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1243,'Sasquatch@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1243,'Sasquatch@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1243,'Sasquatch@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1244,'Christmas Jakk@NPC_EMOTION','idle',197,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1245,'Christmas Goblin@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1245,'Christmas Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',18,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1245,'Christmas Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1245,'Christmas Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3695,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1245,'Christmas Goblin@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@NPC_HOLYATTACK','attack',189,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1246,'Christmas Cookie@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1247,'Antonio@AL_TELEPORT','attack',26,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1247,'Antonio@AL_TELEPORT','idle',26,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1247,'Antonio@AL_TELEPORT','chase',26,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1247,'Antonio@NPC_RUN','attack',354,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1247,'Antonio@NPC_RUN','chase',354,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1248,'Cruiser@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1248,'Cruiser@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',27,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1248,'Cruiser@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1249,'Myst Case@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AL_HEAL','attack',28,9,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AL_HEAL','chase',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AL_HEAL','chase',28,9,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@NPC_EMOTION','attack',197,1,2000,0,5000,'yes','self','always','0',17,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AM_DEMONSTRATION','attack',229,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'17',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@AM_DEMONSTRATION','chase',229,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'17',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@WZ_FIREPILLAR','idle',80,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@NPC_SUMMONSLAVE','attack',196,3,10000,2000,60000,'no','self','slavele','0',1743,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1743,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@PR_STRECOVERY','attack',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1250,'Chepet@PR_STRECOVERY','chase',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_WIDEFREEZE','chase',664,5,10000,1000,10000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_WIDEFREEZE','attack',664,5,10000,1000,10000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_WINDATTACK','chase',187,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_WINDATTACK','attack',187,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@CR_SHIELDCHARGE','attack',250,5,2000,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_HALLUCINATION','chase',207,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1607,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1607,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1515,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@WZ_STORMGUST','attack',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@WZ_STORMGUST','chase',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@WZ_STORMGUST','chase',89,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1251,'Stormy Knight@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@AL_DECAGI','chase',30,48,2000,0,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_WIDEFREEZE','chase',664,5,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_WIDEFREEZE','attack',664,5,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_WEAPONBRAKER','attack',343,10,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@WZ_STORMGUST','chase',89,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@WZ_STORMGUST','attack',89,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1606,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1606,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1252,'Garm@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@AC_CHARGEARROW','attack',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1253,'Gargoyle@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1254,'Raggler@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1254,'Raggler@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1254,'Raggler@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1254,'Raggler@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1255,'Neraid@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1255,'Neraid@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1255,'Neraid@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1256,'Pest@NPC_POISON','attack',176,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1257,'Injustice@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1257,'Injustice@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1258,'Goblin Archer@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',16,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1258,'Goblin Archer@NPC_POISON','attack',176,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1258,'Goblin Archer@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1258,'Goblin Archer@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1259,'Gryphon@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1259,'Gryphon@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1259,'Gryphon@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@CR_REFLECTSHIELD','attack',252,2,500,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1260,'Dark Frame@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1261,'Wild Rose@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1261,'Wild Rose@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1261,'Wild Rose@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1262,'Mutant Dragon@MG_FIREBALL','chase',17,44,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1262,'Mutant Dragon@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1262,'Mutant Dragon@SM_MAGNUM','attack',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@HT_SHOCKWAVE','idle',118,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@MG_LIGHTNINGBOLT','attack',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@MG_LIGHTNINGBOLT','chase',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1263,'Wind Ghost@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1264,'Merman@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1264,'Merman@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1264,'Merman@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1264,'Merman@NPC_RANGEATTACK','chase',160,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1264,'Merman@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','attack',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','idle',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','chase',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1265,'Cookie@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1266,'Aster@NPC_RUN','attack',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1266,'Aster@NPC_RUN','chase',354,1,10000,0,3000,'no','self','always','0',NULL,0x81,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1267,'Carat@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@KN_BRANDISHSPEAR','attack',57,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@NPC_CHANGEFIRE','attack',164,1,10000,2000,600000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@CR_SHIELDCHARGE','attack',250,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@WZ_METEOR','attack',83,5,2000,1500,5000,'no','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@WZ_METEOR','chase',83,5,2000,1500,5000,'no','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1268,'Bloody Knight@NPC_WEAPONBRAKER','attack',343,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1269,'Clock@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1269,'Clock@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1269,'Clock@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1269,'Clock@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1269,'Clock@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@WZ_FIREPILLAR','idle',80,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@SA_DISPELL','attack',289,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1270,'Tower Keeper@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1271,'Alligator@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1271,'Alligator@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@MG_FIREWALL','chase',18,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_HELLJUDGEMENT','chase',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_HELLJUDGEMENT','attack',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_DARKBLESSING','attack',203,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_HELMBRAKE','attack',345,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_SUMMONSLAVE','attack',196,3,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@NPC_SUMMONSLAVE','idle',196,3,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@WZ_METEOR','attack',83,11,5000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@WZ_METEOR','chase',83,11,5000,500,2000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1272,'Dark Lord@WZ_METEOR','chase',83,11,5000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1273,'Orc Lady@AL_INCAGI','chase',29,1,500,700,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1273,'Orc Lady@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1273,'Orc Lady@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1274,'Megalith@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1274,'Megalith@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@CR_SHIELDCHARGE','attack',250,3,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@SA_DISPELL','chase',289,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@SA_DISPELL','attack',289,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1275,'Alice@NPC_SUMMONSLAVE','idle',196,1,500,2000,60000,'no','self','slavele','0',1744,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1276,'Raydric Archer@AC_CHARGEARROW','attack',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1276,'Raydric Archer@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1276,'Raydric Archer@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1276,'Raydric Archer@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1276,'Raydric Archer@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1277,'Greatest General@NPC_BLINDATTACK','idle',177,3,10000,1500,5000,'no','randomtarget','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1277,'Greatest General@NPC_BLINDATTACK','idle',177,3,10000,1500,5000,'no','randomtarget','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1277,'Greatest General@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1277,'Greatest General@WZ_EARTHSPIKE','idle',90,3,5000,1500,5000,'no','randomtarget','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1277,'Greatest General@WZ_EARTHSPIKE','idle',90,3,5000,1500,5000,'no','randomtarget','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1278,'Stalactite Golem@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1279,'Tri-Joint@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1280,'Steam Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',27,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1280,'Steam Goblin@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1280,'Steam Goblin@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1280,'Steam Goblin@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@MG_LIGHTNINGBOLT','chase',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@MG_SAFETYWALL','attack',12,5,2000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1281,'Sage Worm@SA_DISPELL','attack',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1282,'Kobold Archer@AC_DOUBLE','attack',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1282,'Kobold Archer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1282,'Kobold Archer@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1282,'Kobold Archer@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@NPC_SUMMONSLAVE','idle',196,4,10000,2000,60000,'no','self','slavele','0',1253,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@NPC_SHIELDBRAKE','attack',346,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1283,'Chimera@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1285,'Guardian Archer@AC_CHARGEARROW','attack',148,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1285,'Guardian Archer@AC_DOUBLE','attack',46,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1287,'Guardian Soldier@NPC_STUNATTACK','attack',179,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1287,'Guardian Soldier@SM_BASH','attack',5,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@KN_BRANDISHSPEAR','attack',57,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1194,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1289,'Maya Purple@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1290,'Skeleton General@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1290,'Skeleton General@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1290,'Skeleton General@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1290,'Skeleton General@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1290,'Skeleton General@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1028,1016,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@AL_DECAGI','chase',30,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1291,'Wraith Dead@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',18,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_SUMMONSLAVE','attack',196,2,10000,2000,60000,'no','self','slavele','0',1109,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1109,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_DARKTHUNDER','attack',341,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@NPC_DARKTHUNDER','chase',341,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1292,'Mini Demon@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1018,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1293,'Creamy Fear@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1294,'Killer Mantis@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1139,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@MG_LIGHTNINGBOLT','attack',20,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@MG_LIGHTNINGBOLT','chase',20,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@SA_DISPELL','attack',289,5,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1295,'Owl Baron@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1320,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@CR_SHIELDCHARGE','attack',250,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','2',1133,1134,1135,1282,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1296,'Kobold Leader@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','2',1133,1134,1135,1282,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1297,'Ancient Mummy@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1297,'Ancient Mummy@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1297,'Ancient Mummy@NPC_DARKBREATH','attack',202,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1298,'Zombie Master@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1298,'Zombie Master@KN_BRANDISHSPEAR','attack',57,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1298,'Zombie Master@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1298,'Zombie Master@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1036,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@CR_SHIELDCHARGE','attack',250,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','2',1122,1123,1124,1125,1126,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1299,'Goblin Leader@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','2',1122,1123,1124,1125,1126,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1300,'Caterpillar@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1300,'Caterpillar@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1300,'Caterpillar@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@MC_MAMMONITE','attack',42,9,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@AM_ACIDTERROR','chase',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1301,'Am Mut@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1110,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1302,'Dark Illusion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1302,'Dark Illusion@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1302,'Dark Illusion@WZ_METEOR','attack',83,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1302,'Dark Illusion@WZ_METEOR','chase',83,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1303,'Giant Hornet@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1304,'Giant Spider@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1100,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1305,'Ancient Worm@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1305,'Ancient Worm@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1305,'Ancient Worm@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1305,'Ancient Worm@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1305,'Ancient Worm@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1099,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1243,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1306,'Leib Olmai@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1307,'Cat\'o\'Nine Tails@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1307,'Cat\'o\'Nine Tails@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1307,'Cat\'o\'Nine Tails@MC_MAMMONITE','attack',42,9,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1307,'Cat\'o\'Nine Tails@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1307,'Cat\'o\'Nine Tails@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1180,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1308,'Panzer Goblin@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1308,'Panzer Goblin@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1308,'Panzer Goblin@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@MG_SIGHT','chase',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@NPC_BLINDATTACK','chase',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@WZ_SIGHTRASHER','attack',81,7,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1129,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1309,'Gajomart@RG_STRIPWEAPON','attack',215,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1149,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1310,'Majoruros@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@NPC_STUNATTACK','attack',179,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@NPC_SUMMONSLAVE','idle',196,4,10000,2000,60000,'no','self','slavele','0',1166,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1311,'Gullinbursti@RG_STRIPSHIELD','attack',216,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@AS_SONICBLOW','attack',136,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@BS_MAXIMIZE','attack',114,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@BS_MAXIMIZE','chase',114,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@WZ_WATERBALL','chase',86,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_WEAPONBRAKER','attack',343,10,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_STUNATTACK','attack',179,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1364,1594,1600,1601,1602,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1364,1594,1600,1601,1602,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1312,'Turtle General@WZ_WATERBALL','attack',86,5,2000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@NPC_SUMMONSLAVE','idle',196,1,500,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@TF_SPRINKLESAND','attack',149,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1313,'Mobster@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','attack',231,4,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','attack',231,4,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','idle',231,4,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','idle',231,4,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','chase',231,4,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@AM_POTIONPITCHER','chase',231,4,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1314,'Perimeter@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@NPC_EMOTION','attack',197,1,2000,0,600000,'yes','self','myhpltmaxrate','20',23,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@TF_BACKSLIDING','attack',150,1,10000,500,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1315,'Assaulter@NPC_SUMMONSLAVE','chase',196,3,10000,0,600000,'yes','self','myhpltmaxrate','10',1364,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1316,'Solider@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1317,'Seal@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1317,'Seal@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1317,'Seal@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1317,'Seal@NPC_WATERATTACK','attack',184,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@MG_FIREBALL','chase',17,9,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1318,'Heater@WZ_FIREPILLAR','idle',80,3,500,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@MG_FROSTDIVER','attack',15,9,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1319,'Freezer@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@MG_LIGHTNINGBOLT','attack',20,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@MG_LIGHTNINGBOLT','chase',20,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1320,'Owl Duke@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1321,'Dragon Tail@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1321,'Dragon Tail@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1321,'Dragon Tail@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1322,'Spring Rabbit@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1322,'Spring Rabbit@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1322,'Spring Rabbit@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x83,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1322,'Spring Rabbit@SM_BASH','attack',5,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1322,'Spring Rabbit@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1323,'Sea Otter@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1365,'Apocalypse@KN_BRANDISHSPEAR','attack',57,5,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1365,'Apocalypse@NPC_DARKTHUNDER','attack',341,4,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1365,'Apocalypse@NPC_SELFDESTRUCTION','attack',173,1,1000,2000,5000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1365,'Apocalypse@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1365,'Apocalypse@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1366,'Lava Golem@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1366,'Lava Golem@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1366,'Lava Golem@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1367,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1366,'Lava Golem@WZ_FIREPILLAR','idle',80,3,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1367,'Blazer@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1367,'Blazer@MG_FIREBALL','chase',17,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1367,'Blazer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1367,'Blazer@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1367,'Blazer@WZ_SIGHTRASHER','attack',81,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1368,'Geographer@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1368,'Geographer@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1368,'Geographer@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1369,'Grand Peco@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1369,'Grand Peco@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1369,'Grand Peco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1369,'Grand Peco@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1369,'Grand Peco@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x1089,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1370,'Succubus@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_DARKSTRIKE','attack',340,7,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_DARKSTRIKE','chase',340,7,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_BLINDATTACK','angry',177,7,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_BLINDATTACK','follow',177,7,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_CURSEATTACK','angry',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_CURSEATTACK','follow',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_HALLUCINATION','angry',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_HALLUCINATION','follow',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_SILENCEATTACK','angry',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1371,'Fake Angel@NPC_SILENCEATTACK','follow',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1372,'Goat@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1372,'Goat@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1372,'Goat@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x1089,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1372,'Goat@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1372,'Goat@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_HELLJUDGEMENT','chase',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_HELLJUDGEMENT','attack',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_DARKSTRIKE','chase',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_CHANGEUNDEAD','attack',348,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_GRANDDARKNESS','attack',339,10,1000,700,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_STUNATTACK','attack',179,5,2000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1490,1490,1509,1508,1179,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1490,1490,1509,1508,1179,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1373,'Lord of Death@NPC_DARKSTRIKE','chase',340,10,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_DARKSTRIKE','attack',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_DARKSTRIKE','chase',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1374,'Incubus@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_DEFENDER','attack',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_COMBOATTACK','angry',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_DEFENDER','angry',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1375,'The Paper@NPC_DEFENDER','follow',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_SILENCEATTACK','angry',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_SILENCEATTACK','follow',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1376,'Harpy@NPC_WINDATTACK','angry',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_FIREBOLT','attack',19,7,10000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_FROSTDIVER','chase',15,9,10000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_LIGHTNINGBOLT','chase',20,7,10000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_STONECURSE','attack',16,10,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@SA_DISPELL','attack',289,1,0,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@NPC_RUN','attack',354,1,2000,500,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_FIREBOLT','idle',19,7,10000,1500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_FROSTDIVER','idle',15,9,10000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_LIGHTNINGBOLT','idle',20,7,10000,1500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1377,'Elder@MG_STONECURSE','idle',16,10,10000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_BLINDATTACK','chase',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_PETRIFYATTACK','chase',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_BLINDATTACK','angry',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_BLINDATTACK','follow',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_PETRIFYATTACK','angry',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1378,'Demon Pungus@NPC_PETRIFYATTACK','follow',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_DARKSTRIKE','attack',340,7,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_MENTALBREAKER','attack',159,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_DARKSTRIKE','angry',340,7,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1379,'Nightmare Terror@NPC_MENTALBREAKER','angry',159,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@AS_CLOAKING','idle',135,1,100,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@NPC_COMBOATTACK','angry',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1380,'Driller@NPC_GROUNDATTACK','angry',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@KN_SPEARSTAB','angry',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@NPC_STUNATTACK','angry',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1381,'Grizzly@SM_ENDURE','follow',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@WZ_METEOR','attack',83,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@WZ_METEOR','chase',83,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@MO_BODYRELOCATION','follow',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@NPC_FIREATTACK','angry',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@WZ_METEOR','angry',83,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1382,'Diabolic@WZ_METEOR','follow',83,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@AM_DEMONSTRATION','attack',229,1,100,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@NPC_FIREATTACK','angry',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@AM_DEMONSTRATION','angry',229,1,100,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1383,'Explosion@SM_MAGNUM','angry',7,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@MG_FIREBALL','chase',17,9,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@NPC_PIERCINGATT','attack',158,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1384,'Deleter@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@MG_FIREWALL','attack',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@NPC_FIREATTACK','chase',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1385,'Deleter@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@AS_CLOAKING','idle',135,1,100,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_DEFENDER','attack',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_DEFENDER','angry',205,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_DEFENDER','follow',205,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@NPC_SLEEPATTACK','angry',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1386,'Sleeper@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@TF_POISON','attack',52,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@TF_HIDING','angry',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@TF_POISON','angry',52,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@CR_AUTOGUARD','angry',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1387,'Gig@CR_AUTOGUARD','follow',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@CR_GRANDCROSS','attack',254,1,2000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_HEAL','attack',28,11,10000,0,1000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_HEAL','attack',28,11,10000,0,0,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_HEAL','chase',28,11,10000,0,1000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_HEAL','chase',28,11,10000,0,0,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@PR_SANCTUARY','attack',70,10,10000,500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@PR_SANCTUARY','chase',70,10,10000,500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_BARRIER','attack',204,1,500,1000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_HOLYATTACK','attack',189,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_HOLYATTACK','chase',189,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_SUMMONSLAVE','attack',196,7,10000,700,60000,'no','self','slavele','3',1443,1246,1742,1743,1744,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@NPC_SUMMONSLAVE','idle',196,7,10000,700,60000,'no','self','slavele','3',1443,1246,1742,1743,1744,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1388,'Arc Angeling@RG_INTIMIDATE','attack',219,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@AL_DECAGI','chase',30,48,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_HELLJUDGEMENT','chase',662,10,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_HELLJUDGEMENT','attack',662,10,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_BLOODDRAIN','attack',199,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_ENERGYDRAIN','chase',200,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@MO_BODYRELOCATION','chase',264,1,2000,200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_CHANGEUNDEAD','attack',348,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_INVISIBLE','attack',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_INVISIBLE','idle',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_INVISIBLE','chase',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_SUMMONSLAVE','attack',196,16,10000,700,5000,'no','self','slavele','5',1419,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_SUMMONSLAVE','idle',196,16,10000,700,5000,'no','self','slavele','5',1419,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1389,'Dracula@NPC_ENERGYDRAIN','chase',200,1,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1390,'Violy@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1390,'Violy@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1390,'Violy@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1390,'Violy@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1390,'Violy@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@AM_POTIONPITCHER','idle',231,4,500,500,5000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,0x108B,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1391,'Galapago@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1392,'Rotar Zairo@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1392,'Rotar Zairo@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1393,'G Mummy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1393,'G Mummy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_EARTHSPIKE','chase',90,5,4000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_EARTHSPIKE','chase',90,5,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_EARTHSPIKE','attack',90,5,4000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_HEAVENDRIVE','chase',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_HEAVENDRIVE','chase',91,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@WZ_HEAVENDRIVE','attack',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@MG_STONECURSE','attack',16,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@SM_ENDURE','chase',8,7,10000,0,20000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@AL_INCAGI','chase',29,1,5000,3000,120000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@KN_TWOHANDQUICKEN','attack',60,30,5000,3000,300000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@AL_DECAGI','chase',30,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_CHANGEDARKNESS','chase',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_CHANGEDARKNESS','attack',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@AL_TELEPORT','walk',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1395,'Wind Crystal@NPC_HELMBRAKE','attack',345,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@MG_LIGHTNINGBOLT','chase',20,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@MG_LIGHTNINGBOLT','chase',20,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_JUPITEL','chase',84,28,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_JUPITEL','chase',84,28,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_JUPITEL','attack',84,28,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_VERMILION','chase',85,21,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_VERMILION','chase',85,21,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@WZ_VERMILION','attack',85,21,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@SM_ENDURE','chase',8,7,10000,0,20000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@AL_INCAGI','chase',29,1,5000,3000,120000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@KN_TWOHANDQUICKEN','attack',60,30,5000,3000,300000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@AL_DECAGI','chase',30,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_CHANGEDARKNESS','chase',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_CHANGEDARKNESS','attack',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@AL_TELEPORT','walk',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1396,'Earth Crystal@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@MG_FIREBOLT','chase',19,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@MG_FIREBOLT','chase',19,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@MG_FIREBALL','chase',17,44,4000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@MG_FIREBALL','chase',17,44,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@MG_FIREBALL','attack',17,44,4000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@WZ_METEOR','chase',83,11,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@WZ_METEOR','chase',83,11,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@WZ_METEOR','attack',83,11,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@WZ_SIGHTRASHER','attack',81,10,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@SM_ENDURE','chase',8,7,10000,0,20000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@AL_INCAGI','chase',29,1,5000,3000,120000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@KN_TWOHANDQUICKEN','attack',60,30,5000,3000,300000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@AL_DECAGI','chase',30,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_CHANGEDARKNESS','chase',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_CHANGEDARKNESS','attack',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@AL_TELEPORT','walk',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1397,'Fire Crystal@NPC_WEAPONBRAKER','attack',343,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@MG_COLDBOLT','chase',14,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@MG_COLDBOLT','chase',14,5,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_WATERBALL','chase',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_WATERBALL','chase',86,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_WATERBALL','attack',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_STORMGUST','chase',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_STORMGUST','chase',89,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@WZ_STORMGUST','attack',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@SM_ENDURE','chase',8,7,10000,0,20000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@AL_INCAGI','chase',29,1,5000,3000,120000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@KN_TWOHANDQUICKEN','attack',60,30,5000,3000,300000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@AL_DECAGI','chase',30,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_WATERATTACK','attack',184,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_CHANGEDARKNESS','chase',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_CHANGEDARKNESS','attack',168,1,10000,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@AL_TELEPORT','walk',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1398,'Water Crystal@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@AL_TELEPORT','idle',26,1,2000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@NPC_BARRIER','attack',204,1,2000,3000,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@NPC_GUIDEDATTACK','attack',172,5,2000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@NPC_SUMMONSLAVE','attack',196,10,10000,2000,20000,'no','self','slavele','3',1026,1170,1029,1044,1749,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@NPC_SUMMONSLAVE','idle',196,10,10000,2000,20000,'no','self','slavele','3',1026,1170,1029,1044,1749,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_METEOR','chase',83,11,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_METEOR','attack',83,11,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_STORMGUST','chase',89,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_STORMGUST','attack',89,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_VERMILION','chase',85,21,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1399,'Event Baphomet@WZ_VERMILION','attack',85,21,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1400,'Karakasa@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'31',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1400,'Karakasa@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'31',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1400,'Karakasa@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1400,'Karakasa@WZ_WATERBALL','chase',86,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1400,'Karakasa@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always','0',28,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@AS_CLOAKING','idle',135,1,500,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@HT_CLAYMORETRAP','idle',123,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@NPC_REBIRTH','dead',208,2,5000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1401,'Shinobi@TF_BACKSLIDING','attack',150,1,10000,500,3600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1402,'Poison Toad@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1402,'Poison Toad@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1402,'Poison Toad@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1403,'Antique Firelock@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1404,'Miyabi Ningyo@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@NPC_STUNATTACK','chase',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@SM_PROVOKE','attack',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@SM_PROVOKE','chase',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','attack',90,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','attack',90,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','attack',90,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@NPC_FIREATTACK','angry',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@NPC_STUNATTACK','follow',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@SM_PROVOKE','angry',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@SM_PROVOKE','follow',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','angry',90,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','angry',90,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1405,'Tengu@WZ_EARTHSPIKE','angry',90,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@NPC_WATERATTACK','attack',184,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'16',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@NPC_WATERATTACK','angry',184,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1406,'Kapha@WZ_WATERBALL','follow',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'16',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1408,'Bloody Butterfly@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1408,'Bloody Butterfly@NPC_DARKSTRIKE','chase',340,5,2000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1408,'Bloody Butterfly@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1408,'Bloody Butterfly@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1409,'Rice Cake Boy@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@AM_POTIONPITCHER','attack',231,2,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@AM_POTIONPITCHER','idle',231,3,10000,500,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@AM_POTIONPITCHER','chase',231,2,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@NPC_GUIDEDATTACK','chase',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1410,'Live Peach Tree@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Hermit@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Hermit@MG_LIGHTNINGBOLT','attack',20,5,2000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Hermit@MG_LIGHTNINGBOLT','chase',20,5,2000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Hermit@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Hermit@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1412,'Taoist Cloud Hermit@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1413,'Wild Ginseng@NPC_BARRIER','attack',204,1,2000,3000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1413,'Wild Ginseng@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1415,'Baby Leopard@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1415,'Baby Leopard@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1415,'Baby Leopard@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','attack',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','attack',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','idle',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','idle',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','chase',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@AL_HEAL','chase',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@NPC_STOP','attack',342,1,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1416,'Wicked Nymph@SA_DISPELL','attack',289,5,0,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1417,'Zipper Bear@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1417,'Zipper Bear@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@MG_FROSTDIVER','attack',15,40,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@MG_FROSTDIVER','chase',15,40,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_SHIELDBRAKE','attack',346,10,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_DARKBREATH','attack',202,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_STOP','attack',342,1,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','2',1531,1531,1564,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','2',1531,1531,1564,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@SM_MAGNUM','attack',7,25,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1418,'Evil Snake Lord@MG_FROSTDIVER','chase',15,40,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1419,'Farmiliar@NPC_BLINDATTACK','angry',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1419,'Farmiliar@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1419,'Farmiliar@NPC_BLINDATTACK','attack',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1419,'Farmiliar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1420,'Skeleton Archer@AC_DOUBLE','angry',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1420,'Skeleton Archer@NPC_FIREATTACK','angry',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1420,'Skeleton Archer@AC_DOUBLE','attack',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1420,'Skeleton Archer@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1421,'Isis@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1421,'Isis@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1421,'Isis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1421,'Isis@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1422,'Hunter Fly@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1423,'Ghoul@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1423,'Ghoul@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@KN_PIERCE','angry',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@NPC_POISONATTACK','angry',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1424,'Sidewinder@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@NPC_WATERATTACK','angry',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@PR_LEXDIVINA','follow',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@PR_LEXDIVINA','angry',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@PR_LEXDIVINA','chase',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1425,'Obeaune@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1426,'Marc@NPC_STUNATTACK','angry',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1426,'Marc@NPC_WATERATTACK','angry',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1426,'Marc@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1426,'Marc@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_MENTALBREAKER','angry',159,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_TELEKINESISATTACK','angry',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_MENTALBREAKER','attack',159,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1427,'Nightmare@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1428,'Poison Spore@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1428,'Poison Spore@NPC_POISONATTACK','angry',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1428,'Poison Spore@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1428,'Poison Spore@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@AS_VENOMDUST','angry',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@NPC_POISONATTACK','angry',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1429,'Argiope@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1430,'Argos@NPC_POISON','angry',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1430,'Argos@NPC_POISONATTACK','angry',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1430,'Argos@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1430,'Argos@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_DARKBREATH','angry',202,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_DARKNESSATTACK','angry',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_HALLUCINATION','follow',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_HALLUCINATION','angry',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1431,'Baphomet Jr.@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1432,'Desert Wolf@NPC_FIREATTACK','angry',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1432,'Desert Wolf@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1432,'Desert Wolf@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1432,'Desert Wolf@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_BLINDATTACK','follow',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_ENERGYDRAIN','follow',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_ENERGYDRAIN','angry',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKTHUNDER','follow',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKTHUNDER','angry',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKTHUNDER','chase',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1433,'Deviruchi@NPC_DARKTHUNDER','attack',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1434,'Drainliar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_BLINDATTACK','angry',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_BLINDATTACK','follow',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_CURSEATTACK','follow',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_ENERGYDRAIN','follow',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_ENERGYDRAIN','angry',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_PETRIFYATTACK','follow',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_PETRIFYATTACK','angry',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@WZ_HEAVENDRIVE','follow',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@WZ_HEAVENDRIVE','angry',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1435,'Evil Druid@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_CURSEATTACK','follow',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1436,'Jakk@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_FIREBALL','follow',17,5,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_FROSTDIVER','follow',15,40,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_SOULSTRIKE','follow',13,9,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_ENERGYDRAIN','follow',200,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_WINDATTACK','follow',187,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@WZ_HEAVENDRIVE','follow',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_DARKTHUNDER','follow',341,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@WZ_QUAGMIRE','follow',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_FIREBALL','chase',17,5,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_FROSTDIVER','chase',15,40,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@MG_SOULSTRIKE','chase',13,9,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_ATTRICHANGE','idle',161,1,500,2000,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_ENERGYDRAIN','chase',200,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_WINDATTACK','chase',187,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@NPC_DARKTHUNDER','chase',341,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1437,'Joker@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@AS_GRIMTOOTH','follow',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@BS_MAXIMIZE','follow',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@BS_MAXIMIZE','angry',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@NPC_SPLASHATTACK','angry',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@NPC_STUNATTACK','angry',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1438,'Khalitzburg@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1439,'High Orc@NPC_FIREATTACK','angry',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1439,'High Orc@NPC_GUIDEDATTACK','angry',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1439,'High Orc@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1439,'High Orc@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1439,'High Orc@NPC_SMOKING','idle',195,1,500,0,36000000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1440,'Stem Worm@NPC_GUIDEDATTACK','angry',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1440,'Stem Worm@NPC_WINDATTACK','angry',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1440,'Stem Worm@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1440,'Stem Worm@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_POISONATTACK','angry',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1441,'Penomena@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@KN_SPEARSTAB','angry',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@SM_ENDURE','follow',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1442,'Sasquatch@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1443,'Cruiser@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1443,'Cruiser@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','angry',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','follow',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','angry',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','follow',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@MG_FIREWALL','follow',18,10,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@MG_FIREWALL','angry',18,10,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@NPC_FIREATTACK','angry',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@NPC_GUIDEDATTACK','angry',172,5,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@PR_STRECOVERY','follow',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@PR_STRECOVERY','angry',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','attack',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','chase',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','attack',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@AL_HEAL','chase',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@MG_FIREWALL','chase',18,10,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@MG_FIREWALL','attack',18,10,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@NPC_GUIDEDATTACK','attack',172,5,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@PR_STRECOVERY','chase',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1444,'Chepet@PR_STRECOVERY','attack',72,1,2000,700,5000,'yes','friend','friendstatuson','anybad',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1445,'Raggler@NPC_COMBOATTACK','angry',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1445,'Raggler@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1445,'Raggler@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1445,'Raggler@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1446,'Injustice@AS_SONICBLOW','angry',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1446,'Injustice@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1446,'Injustice@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1446,'Injustice@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@KN_PIERCE','angry',56,10,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1447,'Gryphon@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLINDATTACK','angry',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLINDATTACK','follow',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_CURSEATTACK','follow',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_DARKNESSATTACK','angry',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_PETRIFYATTACK','angry',180,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_PETRIFYATTACK','follow',180,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLINDATTACK','attack',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLINDATTACK','chase',177,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_PETRIFYATTACK','attack',180,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1448,'Dark Frame@NPC_PETRIFYATTACK','chase',180,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@MG_FIREBALL','follow',17,44,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@NPC_FIREATTACK','angry',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@SM_MAGNUM','angry',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@MG_FIREBALL','chase',17,44,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1449,'Mutant Dragon@SM_MAGNUM','attack',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@MG_LIGHTNINGBOLT','follow',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@MG_LIGHTNINGBOLT','angry',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@NPC_MAGICALATTACK','angry',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@WZ_JUPITEL','follow',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@WZ_JUPITEL','angry',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@HT_SHOCKWAVE','idle',118,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@MG_LIGHTNINGBOLT','chase',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@MG_LIGHTNINGBOLT','attack',20,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1450,'Wind Ghost@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@KN_PIERCE','angry',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@NPC_RANGEATTACK','follow',160,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@NPC_WATERATTACK','angry',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@NPC_RANGEATTACK','chase',160,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1451,'Merman@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1452,'Orc Lady@AL_INCAGI','follow',29,1,500,700,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1452,'Orc Lady@NPC_GROUNDATTACK','angry',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1452,'Orc Lady@AL_INCAGI','chase',29,1,500,700,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1452,'Orc Lady@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1453,'Raydric Archer@AC_CHARGEARROW','angry',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1453,'Raydric Archer@NPC_DARKNESSATTACK','angry',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1453,'Raydric Archer@AC_CHARGEARROW','attack',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1453,'Raydric Archer@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1454,'Tri Joint@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1454,'Tri Joint@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1455,'Kobold Archer@AC_DOUBLE','angry',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1455,'Kobold Archer@NPC_FIREATTACK','angry',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1455,'Kobold Archer@AC_DOUBLE','attack',46,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1455,'Kobold Archer@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@NPC_SPLASHATTACK','angry',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@WZ_HEAVENDRIVE','follow',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@WZ_HEAVENDRIVE','angry',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1456,'Chimera@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1457,'Mantis@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1457,'Mantis@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1457,'Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1457,'Mantis@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREBOLT','follow',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREBOLT','angry',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREWALL','follow',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_BLINDATTACK','follow',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_FIREATTACK','angry',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_MAGICALATTACK','angry',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1458,'Marduk@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1459,'Marionette@MG_FIREWALL','follow',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1459,'Marionette@NPC_TELEKINESISATTACK','angry',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1459,'Marionette@HT_FREEZINGTRAP','idle',121,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1459,'Marionette@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1459,'Marionette@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1460,'Matyr@AS_SONICBLOW','angry',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1460,'Matyr@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1460,'Matyr@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1460,'Matyr@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@BS_HAMMERFALL','angry',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@WZ_HEAVENDRIVE','angry',91,3,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1461,'Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1462,'Orc Skeleton@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1462,'Orc Skeleton@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1463,'Orc Zombie@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1463,'Orc Zombie@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@SM_BASH','angry',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1464,'Pasana@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1465,'Petite@NPC_GROUNDATTACK','angry',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1465,'Petite@NPC_TELEKINESISATTACK','follow',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1465,'Petite@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1465,'Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1466,'Petite@NPC_TELEKINESISATTACK','follow',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1466,'Petite@NPC_WINDATTACK','angry',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1466,'Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1466,'Petite@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@BS_MAXIMIZE','follow',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@BS_MAXIMIZE','angry',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@NPC_DARKNESSATTACK','angry',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@SM_MAGNUM','angry',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1467,'Raydric@SM_MAGNUM','attack',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1468,'Requim@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1468,'Requim@NPC_STUNATTACK','angry',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1468,'Requim@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1468,'Requim@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1469,'Skeleton Worker@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1469,'Skeleton Worker@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1470,'Zerom@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1470,'Zerom@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1471,'Nine Tail@NPC_ATTRICHANGE','angry',161,1,500,2000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1471,'Nine Tail@NPC_RANDOMATTACK','angry',183,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1471,'Nine Tail@NPC_ATTRICHANGE','attack',161,1,500,2000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1471,'Nine Tail@NPC_RANDOMATTACK','attack',183,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1472,'Bon Gun@KN_SPEARSTAB','angry',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1472,'Bon Gun@NPC_STUNATTACK','angry',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1472,'Bon Gun@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1472,'Bon Gun@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1473,'Orc Archer@AC_SHOWER','angry',47,5,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1473,'Orc Archer@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1473,'Orc Archer@AC_SHOWER','attack',47,5,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1473,'Orc Archer@HT_ANKLESNARE','idle',117,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1473,'Orc Archer@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1474,'Mimic@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1474,'Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_CURSEATTACK','follow',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_ENERGYDRAIN','angry',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1475,'Wraith@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_DARKBREATH','angry',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_SPLASHATTACK','angry',174,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1476,'Alarm@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1477,'Arclouse@NPC_GROUNDATTACK','angry',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1477,'Arclouse@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_BLINDATTACK','angry',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_PIERCINGATT','angry',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1478,'Rideword@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1479,'Skeleton Prisoner@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1479,'Skeleton Prisoner@NPC_SPLASHATTACK','angry',174,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1479,'Skeleton Prisoner@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1479,'Skeleton Prisoner@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1480,'Zombie Prisoner@NPC_POISON','angry',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1480,'Zombie Prisoner@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1481,'Punk@NPC_PETRIFYATTACK','angry',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1481,'Punk@NPC_WINDATTACK','angry',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1481,'Punk@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1481,'Punk@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@AS_SONICBLOW','angry',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@NPC_GUIDEDATTACK','angry',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@NPC_LICK','angry',206,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@SM_PROVOKE','follow',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@NPC_LICK','attack',206,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1482,'Zherlthsh@SM_PROVOKE','chase',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@AS_SONICBLOW','angry',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@NPC_POISON','angry',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@SM_PROVOKE','follow',6,5,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1483,'Rybio@SM_PROVOKE','chase',6,5,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_POISON','angry',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_STUNATTACK','angry',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_WINDATTACK','angry',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1484,'Phendark@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@BS_MAXIMIZE','follow',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@BS_MAXIMIZE','angry',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@NPC_DARKNESSATTACK','angry',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@NPC_GUIDEDATTACK','angry',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1485,'Mysteltainn@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@AS_SONICBLOW','angry',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@BS_MAXIMIZE','follow',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@BS_MAXIMIZE','angry',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@NPC_DARKNESSATTACK','angry',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@NPC_PIERCINGATT','angry',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1486,'Tirfing@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@BS_MAXIMIZE','follow',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@BS_MAXIMIZE','angry',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@KN_AUTOCOUNTER','angry',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@KN_SPEARBOOMERANG','follow',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1487,'Executioner@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@NPC_WATERATTACK','follow',184,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@SM_BASH','angry',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@NPC_WATERATTACK','chase',184,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1488,'Anolian@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@KN_SPEARSTAB','angry',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@NPC_GROUNDATTACK','angry',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@TF_HIDING','angry',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@WZ_QUAGMIRE','follow',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@WZ_QUAGMIRE','angry',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@AL_HEAL','idle',28,5,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@WZ_QUAGMIRE','chase',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1489,'Sting@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@KN_AUTOCOUNTER','angry',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@NPC_SPLASHATTACK','angry',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@NPC_WINDATTACK','angry',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1490,'Wander Man@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@BS_ADRENALINE','follow',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@BS_ADRENALINE','angry',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@MC_MAMMONITE','angry',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@BS_ADRENALINE','chase',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@BS_ADRENALINE','attack',111,1,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1491,'Dokebi@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@AL_TELEPORT','idle',26,1,5000,1000,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_HELLJUDGEMENT','chase',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_HELLJUDGEMENT','attack',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@ASC_BREAKER','idle',379,10,10000,500,5000,'no','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@ASC_BREAKER','chase',379,10,5000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@ASC_BREAKER','attack',379,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_HELMBRAKE','chase',345,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','2',1560,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','2',1560,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@RG_INTIMIDATE','attack',219,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@AL_TELEPORT','chase',26,1,10000,0,0,'yes','self','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1492,'Incantation Samurai@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1493,'Dryad@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1494,'Beetle King@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1494,'Beetle King@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1494,'Beetle King@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1495,'Stone Shooter@NPC_FIREATTACK','attack',186,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1495,'Stone Shooter@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1495,'Stone Shooter@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1497,'Wooden Golem@AL_HEAL','idle',28,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1497,'Wooden Golem@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1497,'Wooden Golem@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1498,'Wootan Shooter@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1498,'Wootan Shooter@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1499,'Wootan Fighter@KN_TWOHANDQUICKEN','attack',60,9,2000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1499,'Wootan Fighter@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1498,'Wootan Shooter@NPC_RUN','attack',354,1,500,0,5000,'yes','self','attackpcge','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1500,'Parasite@NPC_POISON','chase',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@AL_HEAL','idle',28,11,10000,0,2000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@AL_HEAL','walk',28,11,10000,0,2000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@BS_HAMMERFALL','chase',110,10,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@BS_HAMMERFALL','attack',110,10,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@MG_FIREWALL','attack',18,10,10000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@AS_SONICBLOW','attack',136,10,10000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@SM_MAGNUM','attack',7,25,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_VERMILION','chase',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_METEOR','chase',83,11,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_METEOR','attack',83,11,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_STORMGUST','chase',89,10,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@WZ_STORMGUST','attack',89,10,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@NPC_DARKBLESSING','attack',203,1,1000,2000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@NPC_SUMMONSLAVE','idle',196,16,10000,2000,10000,'no','self','slavele','3',1491,1431,1433,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@NPC_SUMMONSLAVE','attack',196,16,10000,2000,10000,'no','self','slavele','3',1491,1431,1433,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@RG_STRIPARMOR','attack',217,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@RG_STRIPHELM','attack',218,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@RG_STRIPSHIELD','attack',216,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1502,'Bring it on!@RG_STRIPWEAPON','attack',215,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1503,'Gibbet@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1503,'Gibbet@NPC_CURSEATTACK','attack',181,3,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1503,'Gibbet@NPC_UNDEADATTACK','attack',347,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1504,'Dullahan@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1504,'Dullahan@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@NPC_DARKSTRIKE','chase',340,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@NPC_SUMMONSLAVE','idle',196,1,500,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@PR_LEXDIVINA','chase',76,10,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@PR_LEXDIVINA','attack',76,10,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1505,'Loli Ruri@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1506,'Disguise@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1506,'Disguise@NPC_METAMORPHOSIS','attack',193,2,500,0,60000,'no','self','myhpltmaxrate','10',1509,1508,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1506,'Disguise@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1507,'Bloody Murderer@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1507,'Bloody Murderer@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1508,'Quve@MG_STONECURSE','attack',16,10,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1508,'Quve@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1508,'Quve@NPC_METAMORPHOSIS','attack',193,1,1000,0,60000,'no','self','myhpltmaxrate','30',1509,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@NPC_METAMORPHOSIS','attack',193,1,1000,0,60000,'no','self','myhpltmaxrate','30',1508,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@PR_LEXAETERNA','chase',78,1,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1509,'Lude@PR_LEXAETERNA','attack',78,1,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1510,'Hylozoist@AL_DECAGI','attack',30,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1510,'Hylozoist@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1510,'Hylozoist@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1510,'Hylozoist@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_BLOODDRAIN','idle',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_BLOODDRAIN','idle',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','longrangeattacked','0',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','casttargeted','0',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','attack',209,5,5000,0,10000,'no','self','longrangeattacked','0',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','attack',209,5,5000,0,10000,'no','self','casttargeted','0',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','18',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','89',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','21',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','28',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','79',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','83',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','21',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','28',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','79',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','83',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','attack',83,11,5000,0,3000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_FIREPILLAR','idle',80,10,10000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_FIREPILLAR','idle',80,10,10000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_FIREPILLAR','attack',80,10,5000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@NPC_DARKBLESSING','attack',203,1,5000,0,2000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@WZ_METEOR','idle',83,11,10000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1511,'Amon Ra@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1512,'Hyegun@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1512,'Hyegun@AS_SONICBLOW','attack',136,5,500,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1512,'Hyegun@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1513,'Civil Servant@NPC_DARKTHUNDER','chase',341,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1513,'Civil Servant@NPC_DARKTHUNDER','attack',341,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1513,'Civil Servant@AM_POTIONPITCHER','idle',231,4,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1513,'Civil Servant@AM_POTIONPITCHER','attack',231,4,10000,500,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1513,'Civil Servant@AM_POTIONPITCHER','idle',231,4,10000,500,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1514,'Dancing Dragon@NPC_AGIUP','loot',350,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1514,'Dancing Dragon@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1514,'Dancing Dragon@NPC_EMOTION','idle',197,1,10000,0,3000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1514,'Dancing Dragon@NPC_EMOTION','attack',197,1,10000,0,3000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1514,'Dancing Dragon@NPC_EMOTION','chase',197,1,10000,0,3000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1515,'Garm Baby@MG_FROSTDIVER','chase',15,40,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1515,'Garm Baby@NPC_LICK','attack',206,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1516,'Increase Soil@NPC_METAMORPHOSIS','attack',193,2,50,5000,5000,'no','self','always','0',1516,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1517,'Li Me Mang Ryang@NPC_RANGEATTACK','chase',160,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1517,'Li Me Mang Ryang@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@HP_ASSUMPTIO','idle',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@HP_ASSUMPTIO','chase',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@HP_ASSUMPTIO','attack',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_WIDESLEEP','chase',668,5,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_WIDESLEEP','attack',668,5,10000,700,25000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_SUMMONSLAVE','idle',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_SUMMONSLAVE','attack',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1518,'Bacsojin@NPC_SUMMONSLAVE','chase',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1519,'Chung E@CR_SHIELDCHARGE','attack',250,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1519,'Chung E@RG_INTIMIDATE','attack',219,1,1000,1000,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1519,'Chung E@PR_LEXDIVINA','chase',76,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1519,'Chung E@TF_POISON','attack',52,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1520,'Boiled Rice@CR_AUTOGUARD','idle',249,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1520,'Boiled Rice@NPC_SUICIDE','attack',175,1,10000,4000,0,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1566,'Wraith Dead@AL_DECAGI','chase',30,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1566,'Wraith Dead@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1566,'Wraith Dead@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1566,'Wraith Dead@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1566,'Wraith Dead@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1555,'Parasite@NPC_REVENGE','idle',333,1,10000,0,0,'yes','target','masterattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1575,'Flora@NPC_REVENGE','idle',333,1,10000,0,0,'yes','target','masterattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1579,'Hydra@NPC_REVENGE','idle',333,1,10000,0,0,'yes','target','masterattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_DARKSTRIKE','attack',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_DARKSTRIKE','chase',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1580,'Incubus@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@NPC_SUMMONSLAVE','attack',196,7,10000,2000,10000,'no','self','slavele','3',1595,1595,1595,1433,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@NPC_SUMMONSLAVE','idle',196,7,10000,2000,10000,'no','self','slavele','3',1595,1595,1595,1433,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@NPC_DARKSTRIKE','chase',340,5,3000,700,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@NPC_DARKSTRIKE','attack',340,5,3000,700,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@NPC_DARKNESSATTACK','attack',190,2,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1582,'Deviling@AL_DECAGI','chase',30,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','4',1793,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','4',1793,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@AL_TELEPORT','chase',26,1,10000,0,0,'yes','self','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_MENTALBREAKER','attack',159,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_MENTALBREAKER','chase',159,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_COMBOATTACK','attack',171,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_COMBOATTACK','chase',171,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_STUNATTACK','chase',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_GUIDEDATTACK','attack',172,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_GUIDEDATTACK','chase',172,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1583,'Tao Gunka@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1584,'Tamruan@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1584,'Tamruan@NPC_SILENCEATTACK','attack',178,5,2000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1584,'Tamruan@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1585,'Mime Monkey@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1585,'Mime Monkey@SM_PROVOKE','chase',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1585,'Mime Monkey@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1585,'Mime Monkey@SM_PROVOKE','attack',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1586,'Leaf Cat@AL_HEAL','attack',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1586,'Leaf Cat@AL_HEAL','attack',28,1,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1586,'Leaf Cat@AL_HEAL','chase',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1586,'Leaf Cat@AL_HEAL','chase',28,1,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1587,'Kraben@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1587,'Kraben@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1587,'Kraben@NPC_STUNATTACK','attack',179,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1588,'Christmas Orc@NPC_EMOTION','idle',197,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1589,'Mandragora@NPC_REVENGE','idle',333,1,10000,0,0,'yes','target','masterattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1590,'Geographer@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1590,'Geographer@NPC_REVENGE','idle',333,1,10000,0,0,'yes','target','masterattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1609,'Dancing Dragon@NPC_AGIUP','loot',350,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1609,'Dancing Dragon@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1609,'Dancing Dragon@NPC_EMOTION','idle',197,1,10000,0,3000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1609,'Dancing Dragon@NPC_EMOTION','attack',197,1,10000,0,3000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1609,'Dancing Dragon@NPC_EMOTION','chase',197,1,10000,0,3000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1613,'Metaling@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1613,'Metaling@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1613,'Metaling@CR_AUTOGUARD','chase',249,5,500,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1613,'Metaling@RG_STRIPWEAPON','attack',215,3,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1613,'Metaling@MC_MAMMONITE','attack',42,5,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@CR_REFLECTSHIELD','attack',252,2,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@NPC_RANDOMATTACK','attack',183,2,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@NPC_BARRIER','attack',204,1,500,3000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@NPC_SPLASHATTACK','attack',174,1,500,0,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@TF_HIDING','attack',51,1,500,500,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1614,'Mineral@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@CR_SHIELDCHARGE','attack',250,2,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@NPC_BARRIER','attack',204,1,500,3000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@TF_HIDING','attack',51,1,500,500,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1615,'Obsidian@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1616,'Pitman@TF_SPRINKLESAND','attack',149,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1616,'Pitman@TF_SPRINKLESAND','chase',149,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1616,'Pitman@NPC_COMBOATTACK','attack',171,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1616,'Pitman@NPC_POWERUP','attack',349,2,3000,500,10000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1616,'Pitman@HT_CLAYMORETRAP','idle',123,5,500,0,300000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1617,'Waste Stove@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1617,'Waste Stove@TF_THROWSTONE','chase',152,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1617,'Waste Stove@NPC_RANDOMATTACK','attack',183,3,500,200,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1617,'Waste Stove@NPC_SPLASHATTACK','attack',174,1,1000,500,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_PETRIFYATTACK','attack',180,2,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_DARKBREATH','attack',202,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_POISONATTACK','attack',188,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@AL_TELEPORT','attack',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_SUMMONSLAVE','attack',196,3,10000,700,30000,'no','self','slavele','0',1625,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1618,'Ungoliant@NPC_SUMMONSLAVE','idle',196,3,10000,700,30000,'no','self','slavele','0',1625,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1619,'Porcellio@AL_PNEUMA','attack',25,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1619,'Porcellio@NPC_GROUNDATTACK','attack',185,2,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1619,'Porcellio@NPC_KEEPING','attack',201,1,2000,1000,5000,'no','self','myhpltmaxrate','25',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_SELFDESTRUCTION','chase',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','skillused','173',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_TELEKINESISATTACK','attack',191,3,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_MENTALBREAKER','attack',159,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1620,'Noxious@NPC_MENTALBREAKER','chase',159,2,500,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_SELFDESTRUCTION','chase',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','skillused','173',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_POISON','attack',176,3,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1621,'Venomous@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_SELFDESTRUCTION','attack',173,1,1000,2000,10000,'no','self','attackpcge','3',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_SELFDESTRUCTION','chase',173,1,1000,2000,10000,'no','self','attackpcge','3',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_CURSEATTACK','attack',181,4,500,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_CURSEATTACK','chase',181,4,500,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@SA_DISPELL','attack',289,1,50,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1622,'Teddy Bear@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always','0',23,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_AGIUP','chase',350,5,2000,0,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_POWERUP','attack',349,5,2000,0,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_EARTHQUAKE','chase',653,5,10000,2000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_EARTHQUAKE','attack',653,5,10000,2000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_SUMMONSLAVE','attack',196,5,10000,2000,10000,'no','self','slavele','3',1624,1624,1621,1620,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_SUMMONSLAVE','idle',196,5,10000,2000,10000,'no','self','slavele','3',1624,1624,1621,1620,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1617,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_WEAPONBRAKER','attack',343,10,1000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@AL_TELEPORT','idle',26,1,10000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@AL_TELEPORT','walk',26,1,10000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@NPC_CRITICALSLASH','attack',170,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1623,'RSX 0806@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1624,'Waste Stove@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1624,'Waste Stove@TF_THROWSTONE','chase',152,1,4000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1624,'Waste Stove@NPC_RANDOMATTACK','attack',183,3,500,200,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1624,'Waste Stove@NPC_SPLASHATTACK','attack',174,1,1000,500,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1625,'Porcellio@AL_PNEUMA','attack',25,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1625,'Porcellio@NPC_GROUNDATTACK','attack',185,2,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1625,'Porcellio@NPC_KEEPING','attack',201,1,2000,1000,5000,'no','self','myhpltmaxrate','25',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_DARKCROSS','attack',338,10,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_CHANGEUNDEAD','attack',348,5,1000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@MO_BODYRELOCATION','chase',264,1,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_DARKBREATH','chase',202,5,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_GRANDDARKNESS','attack',339,5,2000,2000,30000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_GRANDDARKNESS','attack',339,5,500,2000,30000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_DARKSTRIKE','chase',340,10,2000,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1626,'Dark Priest@NPC_DARKNESSATTACK','attack',190,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_BLOODDRAIN','chase',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_COMBOATTACK','chase',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1627,'Anopheles@NPC_WINDATTACK','chase',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@TF_HIDING','idle',51,1,500,500,25000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@NPC_GROUNDATTACK','chase',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1628,'Mole@NPC_COMBOATTACK','chase',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1629,'Hill Wind@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1629,'Hill Wind@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1629,'Hill Wind@NPC_WINDATTACK','chase',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1629,'Hill Wind@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1629,'Hill Wind@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@HP_ASSUMPTIO','idle',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@HP_ASSUMPTIO','chase',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@HP_ASSUMPTIO','attack',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@NPC_SUMMONSLAVE','idle',196,3,10000,0,5000,'no','self','slavele','1',1631,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@NPC_SUMMONSLAVE','attack',196,3,10000,0,5000,'no','self','slavele','1',1631,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1630,'Bacsojin@NPC_SUMMONSLAVE','chase',196,3,10000,0,5000,'no','self','slavele','1',1631,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1631,'Chung E@CR_SHIELDCHARGE','attack',250,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1631,'Chung E@RG_INTIMIDATE','attack',219,1,1000,1000,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1631,'Chung E@PR_LEXDIVINA','chase',76,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1631,'Chung E@TF_POISON','attack',52,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1632,'Gremlin@RG_STRIPARMOR','attack',217,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1632,'Gremlin@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@MG_NAPALMBEAT','chase',11,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@SA_DISPELL','attack',289,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1633,'Beholder@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@LK_SPIRALPIERCE','chase',397,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@LK_SPIRALPIERCE','attack',397,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@SM_MAGNUM','attack',7,9,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@SM_PROVOKE','attack',6,10,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1634,'Seyren Windsor@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@ASC_BREAKER','chase',379,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@ASC_METEORASSAULT','attack',406,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@TF_POISON','attack',52,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_SONICBLOW','attack',136,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_VENOMDUST','idle',140,1,2000,0,50000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@AS_VENOMDUST','attack',140,1,2000,0,50000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1635,'Eremes Guile@NPC_POISONATTACK','attack',188,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@MC_MAMMONITE','attack',42,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@BS_HAMMERFALL','chase',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@BS_HAMMERFALL','attack',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_SPLASHATTACK','attack',174,1,1000,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_FIREATTACK','attack',186,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1636,'Howard Alt-Eisen@NPC_WATERATTACK','attack',184,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_INCAGI','attack',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_INCAGI','chase',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXDIVINA','attack',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXDIVINA','chase',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXDIVINA','idle',76,10,10000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXDIVINA','attack',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXDIVINA','chase',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@PR_LEXAETERNA','chase',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@NPC_HOLYATTACK','attack',189,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','attack',28,9,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','attack',28,9,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','idle',28,9,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','idle',28,9,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','chase',28,9,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1637,'Margaretha Sorin@AL_HEAL','chase',28,9,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@SN_SHARPSHOOTING','attack',382,5,2000,500,1500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_SKIDTRAP','idle',115,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_SKIDTRAP','attack',115,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_FLASHER','idle',120,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_FLASHER','attack',120,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_FREEZINGTRAP','idle',121,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@HT_FREEZINGTRAP','attack',121,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1638,'Cecil Damon@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_JUPITEL','idle',84,28,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBALL','idle',17,44,10000,500,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FROSTDIVER','chase',15,40,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_JUPITEL','chase',84,28,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBALL','chase',17,44,10000,500,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@HW_NAPALMVULCAN','chase',400,5,2000,1000,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_COLDBOLT','attack',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_COLDBOLT','chase',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_SIGHTRASHER','attack',81,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_QUAGMIRE','chase',92,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_SAFETYWALL','attack',12,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBOLT','chase',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1639,'Kathryne Keyron@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@WZ_STORMGUST','idle',89,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@WZ_STORMGUST','walk',89,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@LK_SPIRALPIERCE','chase',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@LK_SPIRALPIERCE','attack',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@KN_BOWLINGBASH','attack',62,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@KN_BOWLINGBASH','attack',62,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_BASH','attack',5,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_MAGNUM','attack',7,25,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_PROVOKE','attack',6,10,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1640,'Lord Knight Seyren@NPC_FIREATTACK','attack',186,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@ASC_BREAKER','chase',379,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@ASC_METEORASSAULT','attack',406,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_INVISIBLE','idle',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_INVISIBLE','attack',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_GUIDEDATTACK','attack',172,10,2000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@TF_POISON','attack',52,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,3000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_VENOMDUST','idle',140,1,2000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@AS_VENOMDUST','attack',140,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1641,'Assassin Cross Eremes@NPC_POISONATTACK','attack',188,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@MC_MAMMONITE','attack',42,22,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_SPLASHATTACK','attack',174,1,1000,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_HELMBRAKE','attack',345,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_SHIELDBRAKE','attack',346,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1642,'Whitesmith Howard@NPC_WATERATTACK','attack',184,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_INCAGI','attack',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_INCAGI','chase',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXDIVINA','attack',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXDIVINA','chase',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXDIVINA','idle',76,10,10000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXDIVINA','attack',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXDIVINA','chase',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@PR_LEXAETERNA','chase',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@NPC_HOLYATTACK','attack',189,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','attack',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','attack',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','idle',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','idle',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','chase',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1643,'High Priest Margaretha@AL_HEAL','chase',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@SN_SHARPSHOOTING','attack',382,5,2000,500,1500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_LANDMINE','idle',116,5,2000,0,60000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_LANDMINE','attack',116,5,500,0,5000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_ANKLESNARE','idle',117,5,2000,0,60000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_ANKLESNARE','attack',117,5,500,0,5000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_BLASTMINE','idle',122,5,2000,0,60000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@HT_BLASTMINE','attack',122,5,500,0,5000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1644,'Sniper Cecil@NPC_WINDATTACK','attack',187,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@RG_INTIMIDATE','attack',219,10,100,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@NPC_SUMMONSLAVE','attack',196,1,10000,700,30000,'no','self','slavele','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@NPC_SUMMONSLAVE','idle',196,1,10000,700,30000,'no','self','slavele','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_STORMGUST','idle',89,10,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_VERMILION','idle',85,21,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','idle',83,11,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@HW_GANBANTEIN','attack',483,5,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@HW_NAPALMVULCAN','chase',400,5,2000,1000,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_STORMGUST','chase',89,10,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_VERMILION','chase',85,21,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','chase',83,11,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@MG_SIGHT','idle',10,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_STORMGUST','chase',89,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_VERMILION','chase',85,21,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_STORMGUST','attack',89,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_VERMILION','attack',85,21,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_FIREPILLAR','idle',80,10,5000,200,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_FIREPILLAR','attack',80,10,2000,200,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','chase',83,11,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_METEOR','attack',83,11,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_HEAVENDRIVE','attack',91,5,2000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_WATERBALL','chase',86,5,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1645,'High Wizard Kathryne@WZ_WATERBALL','attack',86,5,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@LK_SPIRALPIERCE','chase',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@LK_SPIRALPIERCE','attack',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@KN_BOWLINGBASH','attack',62,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@KN_BOWLINGBASH','attack',62,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_BASH','attack',5,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_MAGNUM','attack',7,25,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_PROVOKE','attack',6,10,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_FIREATTACK','attack',186,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CHANGEFIRE','attack',164,1,10000,500,60000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CHANGEHOLY','attack',167,1,10000,500,60000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1646,'Lord Knight Seyren@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,60000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_INVISIBLE','idle',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_INVISIBLE','attack',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_GUIDEDATTACK','attack',172,10,2000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@TF_POISON','attack',52,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,3000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_VENOMDUST','idle',140,1,2000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@AS_VENOMDUST','attack',140,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@ASC_BREAKER','chase',379,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@ASC_METEORASSAULT','attack',406,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_POISONATTACK','attack',188,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CHANGEWIND','attack',165,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1647,'Assassin Cross Eremes@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@MC_MAMMONITE','attack',42,22,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_SPLASHATTACK','attack',174,1,1000,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_HELMBRAKE','attack',345,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_SHIELDBRAKE','attack',346,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@ST_FULLSTRIP','attack',476,5,1500,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_WATERATTACK','attack',184,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CHANGEGROUND','attack',163,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1648,'Whitesmith Howard@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@HP_ASSUMPTIO','idle',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@HP_ASSUMPTIO','chase',361,5,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','attack',28,11,10000,0,1000,'yes','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','attack',28,11,10000,0,1000,'yes','friend','friendhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1642,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1642,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1641,1642,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_INCAGI','attack',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_INCAGI','chase',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXDIVINA','attack',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXDIVINA','chase',76,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXDIVINA','idle',76,10,10000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXDIVINA','attack',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXDIVINA','chase',76,10,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@PR_LEXAETERNA','chase',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_HOLYATTACK','attack',189,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CHANGEWATER','attack',162,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','attack',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','attack',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','idle',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','idle',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','chase',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1649,'High Priest Margaretha@AL_HEAL','chase',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1642,1643,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1642,1643,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1641,1642,1643,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_CLAYMORETRAP','idle',123,5,2000,0,10000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_CLAYMORETRAP','attack',123,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_SANDMAN','idle',119,5,2000,0,10000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_SANDMAN','attack',119,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_SHOCKWAVE','idle',118,5,2000,0,10000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@SN_SHARPSHOOTING','attack',382,5,2000,500,1500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@HT_SHOCKWAVE','attack',118,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_WINDATTACK','attack',187,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CHANGEWATER','attack',162,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1650,'Sniper Cecil@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_SUMMONSLAVE','attack',196,5,10000,700,30000,'no','self','slavele','3',1640,1641,1642,1643,1644,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_SUMMONSLAVE','idle',196,5,10000,700,30000,'no','self','slavele','3',1640,1641,1642,1643,1644,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1641,1642,1643,1644,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_STORMGUST','idle',89,10,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_VERMILION','idle',85,21,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_METEOR','idle',83,11,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@HW_NAPALMVULCAN','chase',400,5,2000,1000,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_STORMGUST','chase',89,10,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_VERMILION','chase',85,21,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_METEOR','chase',83,11,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@MG_SIGHT','idle',10,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_STORMGUST','chase',89,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_VERMILION','chase',85,21,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_STORMGUST','attack',89,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_VERMILION','attack',85,21,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_FIREPILLAR','idle',80,10,5000,200,2000,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_FIREPILLAR','attack',80,10,2000,200,0,'yes','around3','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_METEOR','chase',83,11,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_METEOR','attack',83,11,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_HEAVENDRIVE','attack',91,5,2000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_WATERBALL','chase',86,5,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@WZ_WATERBALL','attack',86,5,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_ATTRICHANGE','attack',161,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1651,'High Wizard Kathryne@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@SM_BASH','attack',5,9,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@SM_BASH','attack',5,9,10000,800,1000,'no','target','skillused','5',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@SM_MAGNUM','attack',7,9,5000,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@SM_PROVOKE','chase',6,9,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@NPC_FIREATTACK','attack',186,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1652,'Egnigem Cenia@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@TF_POISON','attack',52,9,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@TF_POISON','attack',52,9,10000,1000,1000,'no','target','skillused','52',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@TF_THROWSTONE','chase',152,1,4000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@NPC_POISONATTACK','attack',188,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@NPC_POISON','attack',176,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1653,'Wickebine Tres@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',8,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@MC_MAMMONITE','attack',42,9,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@MC_MAMMONITE','attack',42,9,10000,800,1000,'no','target','skillused','42',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_STUNATTACK','attack',179,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_GROUNDATTACK','attack',185,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1654,'Armeyer Dinze@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_PNEUMA','idle',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','attack',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','idle',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','chase',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@NPC_HOLYATTACK','attack',189,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@NPC_SILENCEATTACK','attack',178,4,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_DECAGI','attack',30,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_DECAGI','chase',30,1,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1655,'Errende Ebecee@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@AC_DOUBLE','attack',46,1,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@AC_DOUBLE','attack',46,1,10000,1000,1000,'no','target','skillused','46',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@AC_DOUBLE','chase',46,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@AC_SHOWER','attack',47,5,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@NPC_WINDATTACK','attack',187,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1656,'Kavach Icarus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_SOULSTRIKE','idle',13,9,5000,0,1000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_LIGHTNINGBOLT','idle',20,9,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_FROSTDIVER','chase',15,9,10000,500,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_LIGHTNINGBOLT','chase',20,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_LIGHTNINGBOLT','attack',20,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_SOULSTRIKE','attack',13,9,5000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_SOULSTRIKE','attack',13,5,3000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_SOULSTRIKE','attack',13,3,2000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@MG_SAFETYWALL','attack',12,10,2000,1000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1657,'Laurell Weinder@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@SM_BASH','attack',5,10,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@SM_BASH','attack',5,10,10000,800,1000,'no','target','skillused','5',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@SM_MAGNUM','attack',7,25,5000,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@SM_PROVOKE','attack',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@SM_PROVOKE','chase',6,10,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@LK_SPIRALPIERCE','attack',397,2,2500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_CHANGEWATER','attack',162,1,10000,500,10000,'no','self','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_CHANGEWIND','attack',165,1,10000,500,10000,'no','self','skillused','84',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1659,1660,1661,1662,1663,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1659,1660,1661,1662,1663,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1658,'Egnigem Cenia@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@TF_POISON','attack',52,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@TF_POISON','attack',52,9,10000,1000,1000,'no','target','skillused','52',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@TF_THROWSTONE','chase',152,1,4000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@NPC_POISONATTACK','attack',188,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@NPC_POISON','attack',176,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1659,'Wickebine Tres@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',8,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@MC_MAMMONITE','attack',42,9,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@MC_MAMMONITE','attack',42,9,10000,800,1000,'no','target','skillused','42',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_STUNATTACK','attack',179,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_GROUNDATTACK','attack',185,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1660,'Armeyer Dinze@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','attack',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','attack',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','idle',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','idle',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','chase',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_HEAL','chase',28,11,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@NPC_HOLYATTACK','attack',189,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@NPC_SILENCEATTACK','attack',178,4,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_DECAGI','attack',30,48,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1661,'Errende Ebecee@AL_DECAGI','chase',30,48,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@AC_DOUBLE','attack',46,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@AC_DOUBLE','attack',46,10,10000,1000,1000,'no','target','skillused','46',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@AC_DOUBLE','chase',46,10,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@AC_SHOWER','attack',47,10,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@NPC_WINDATTACK','attack',187,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@NPC_PIERCINGATT','attack',158,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1662,'Kavach Icarus@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_FROSTDIVER','chase',15,9,10000,500,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_LIGHTNINGBOLT','chase',20,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_LIGHTNINGBOLT','attack',20,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_SOULSTRIKE','attack',13,9,5000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@MG_SAFETYWALL','attack',12,10,2000,1000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1663,'Laurell Weinder@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1664,'Photon Cannon@NPC_WINDATTACK','attack',187,5,1000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1664,'Photon Cannon@NPC_WINDATTACK','attack',187,3,2000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1664,'Photon Cannon@NPC_WINDATTACK','attack',187,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1665,'Photon Cannon@NPC_FIREATTACK','attack',186,5,1000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1665,'Photon Cannon@NPC_FIREATTACK','attack',186,3,2000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1665,'Photon Cannon@NPC_FIREATTACK','attack',186,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1666,'Photon Cannon@NPC_WATERATTACK','attack',184,5,1000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1666,'Photon Cannon@NPC_WATERATTACK','attack',184,3,2000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1666,'Photon Cannon@NPC_WATERATTACK','attack',184,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1667,'Photon Cannon@NPC_GROUNDATTACK','attack',185,5,1000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1667,'Photon Cannon@NPC_GROUNDATTACK','attack',185,3,2000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1667,'Photon Cannon@NPC_GROUNDATTACK','attack',185,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1668,'Archdam@NPC_PIERCINGATT','attack',158,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1668,'Archdam@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1668,'Archdam@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1668,'Archdam@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1668,'Archdam@NPC_RANDOMATTACK','attack',183,5,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1669,'Dimik@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1669,'Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1669,'Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1669,'Dimik@NPC_COMBOATTACK','attack',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1669,'Dimik@NPC_COMBOATTACK','chase',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@NPC_COMBOATTACK','attack',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@NPC_COMBOATTACK','chase',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1670,'Dimik@NPC_WINDATTACK','attack',187,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@NPC_COMBOATTACK','attack',171,6,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@NPC_COMBOATTACK','chase',171,6,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1671,'Dimik@NPC_WATERATTACK','attack',184,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@NPC_RANDOMATTACK','attack',183,2,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@NPC_COMBOATTACK','attack',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@NPC_COMBOATTACK','chase',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1672,'Dimik@NPC_GROUNDATTACK','attack',185,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@NPC_COMBOATTACK','attack',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@NPC_COMBOATTACK','chase',171,4,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1673,'Dimik@NPC_FIREATTACK','attack',186,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1674,'Monemus@NPC_FIREATTACK','attack',186,10,500,1000,30000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1674,'Monemus@NPC_FIREATTACK','attack',186,3,1000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1674,'Monemus@NPC_FIREATTACK','attack',186,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_FIREATTACK','attack',186,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1675,'Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1676,'Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_WINDATTACK','attack',187,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1677,'Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_RANDOMATTACK','attack',183,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_GROUNDATTACK','attack',185,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1678,'Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@NPC_WATERATTACK','attack',184,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1679,'Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',0,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1680,'Hill Wind@NPC_WINDATTACK','chase',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1680,'Hill Wind@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1680,'Hill Wind@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1680,'Hill Wind@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1680,'Hill Wind@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',30,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',31,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@KN_PIERCE','attack',56,5,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@NPC_PIERCINGATT','attack',158,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1681,'Gemini-S58@NPC_COMBOATTACK','chase',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1682,'Remover@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1682,'Remover@NPC_UNDEADATTACK','attack',347,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1684,'Archdam@NPC_PIERCINGATT','attack',158,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1684,'Archdam@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1684,'Archdam@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1684,'Archdam@NPC_RANDOMATTACK','attack',183,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_SUMMONSLAVE','attack',196,3,10000,2000,5000,'no','self','slavele','15',1365,1669,1675,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1365,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_POWERUP','attack',349,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_CRITICALSLASH','attack',170,1,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@WZ_METEOR','attack',83,11,500,10000,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1685,'Vesper@NPC_EMOTION','idle',197,1,1000,0,0,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@NPC_EMOTION','idle',197,1,10000,0,5000,'no','self','slavele','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@NPC_EMOTION','attack',197,1,10000,0,5000,'no','self','slavele','0',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@NPC_SUMMONSLAVE','idle',196,2,10000,0,6000000,'yes','self','slavele','0',1023,1273,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@NPC_SUMMONSLAVE','attack',196,2,10000,0,6000000,'yes','self','slavele','0',1023,1273,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1686,'Orc Baby@NPC_SUMMONSLAVE','chase',196,2,10000,0,6000000,'yes','self','slavele','0',1023,1273,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AL_TELEPORT','attack',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_EXPULSION','attack',674,1,1000,1000,10000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@AC_CHARGEARROW','attack',148,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_COMBOATTACK','attack',171,1,1000,0,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_STUNATTACK','attack',179,5,500,0,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_DEFENDER','attack',205,1,500,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_SUMMONSLAVE','attack',196,5,10000,2000,5000,'no','self','slavele','2',1691,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_SUMMONSLAVE','idle',196,5,10000,2000,5000,'no','self','slavele','2',1691,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1688,'Lady Tany@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@NPC_SUMMONSLAVE','idle',196,4,10000,0,5000,'yes','self','slavele','2',1690,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@NPC_SUMMONSLAVE','attack',196,4,10000,0,5000,'yes','self','slavele','2',1690,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1689,'Bacsojin@NPC_SUMMONSLAVE','chase',196,4,10000,0,5000,'yes','self','slavele','2',1690,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1690,'Spring Rabbit@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1690,'Spring Rabbit@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1690,'Spring Rabbit@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x83,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1690,'Spring Rabbit@SM_BASH','attack',5,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1690,'Spring Rabbit@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1691,'Kraben@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1691,'Kraben@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1691,'Kraben@NPC_STUNATTACK','attack',179,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);

REPLACE INTO `mob_skill_db_re` VALUES (1692,'Breeze@NPC_DEFENDER','attack',205,1,1000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1692,'Breeze@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1692,'Breeze@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1692,'Breeze@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1692,'Breeze@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@NPC_STUNATTACK','chase',179,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@NPC_STUNATTACK','attack',179,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@MG_SOULSTRIKE','chase',13,9,3000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@MG_SOULSTRIKE','attack',13,9,3000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@MG_NAPALMBEAT','attack',11,7,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@MG_NAPALMBEAT','attack',11,7,4000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1693,'Plasma@NPC_TELEKINESISATTACK','attack',191,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@NPC_SLEEPATTACK','chase',182,5,1000,300,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@NPC_SLEEPATTACK','attack',182,5,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@MG_FIREBOLT','chase',19,7,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@MG_FIREBOLT','attack',19,7,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@MG_FIREBALL','chase',17,5,3000,500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@MG_FIREBALL','attack',17,7,2000,500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1694,'Plasma@NPC_FIREATTACK','attack',186,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@MG_STONECURSE','chase',16,8,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@MG_STONECURSE','attack',16,8,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@WZ_EARTHSPIKE','chase',90,5,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@WZ_EARTHSPIKE','attack',90,5,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@WZ_QUAGMIRE','chase',92,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1695,'Plasma@NPC_GROUNDATTACK','attack',185,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_DARKSTRIKE','chase',340,9,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_DARKSTRIKE','attack',340,9,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1696,'Plasma@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@MG_FROSTDIVER','chase',15,9,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@MG_FROSTDIVER','attack',15,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@MG_COLDBOLT','chase',14,6,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@MG_COLDBOLT','attack',14,6,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@WZ_WATERBALL','attack',86,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1697,'Plasma@NPC_WATERATTACK','attack',184,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);

# Thanatos Tower Mobs
REPLACE INTO `mob_skill_db_re` VALUES (1698,'Deathword@NPC_SPLASHATTACK','attack',174,1,1000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1698,'Deathword@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1698,'Deathword@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1698,'Deathword@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1699,'Ancient Mimic@CR_AUTOGUARD','chase',249,3,1000,0,250000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1699,'Ancient Mimic@CR_AUTOGUARD','attack',249,3,500,0,250000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1699,'Ancient Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@RG_INTIMIDATE','attack',219,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@MG_SAFETYWALL','attack',12,5,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@WZ_JUPITEL','attack',84,4,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@WZ_JUPITEL','chase',84,4,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1700,'Observation@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_DARKNESSATTACK','chase',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_DARKTHUNDER','chase',341,4,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_DARKTHUNDER','attack',341,4,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@AS_VENOMDUST','idle',140,1,2000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@AS_VENOMDUST','attack',140,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_BLINDATTACK','angry',177,7,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1701,'Shelter@NPC_BLINDATTACK','follow',177,7,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@NPC_GRANDDARKNESS','attack',339,1,500,700,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@NPC_DARKCROSS','attack',338,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@KN_BOWLINGBASH','attack',62,7,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1702,'Retribution@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@CR_GRANDCROSS','attack',254,1,1000,1500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@PR_LEXDIVINA','chase',76,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@PR_LEXDIVINA','attack',76,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@MG_NAPALMBEAT','attack',11,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1703,'Solace@MG_NAPALMBEAT','chase',11,5,1000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_DARKNESSATTACK','attack',190,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_CHANGEDARKNESS','attack',168,1,1000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_POWERUP','attack',349,5,2000,0,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_POWERUP','attack',349,5,2000,0,10000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1704,'Thanatos Odium@NPC_TELEKINESISATTACK','attack',191,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_FIREATTACK','attack',186,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_STUNATTACK','attack',179,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@WZ_SIGHTRASHER','attack',81,10,1000,0,10000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@KN_BOWLINGBASH','attack',62,4,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_AGIUP','attack',350,5,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@NPC_AGIUP','attack',350,5,2000,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@SM_PROVOKE','attack',6,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1705,'Thanatos Despero@SM_PROVOKE','chase',6,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@NPC_DARKSTRIKE','attack',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@NPC_DARKSTRIKE','chase',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@AS_SONICBLOW','attack',136,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@NPC_TELEKINESISATTACK','attack',191,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@NPC_INVISIBLE','attack',353,1,500,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1706,'Thanatos Maero@NPC_INVISIBLE','idle',353,1,500,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@NPC_MAGICALATTACK','attack',192,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@NPC_MENTALBREAKER','attack',159,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@NPC_MENTALBREAKER','chase',159,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@NPC_DARKTHUNDER','attack',341,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@NPC_DARKTHUNDER','chase',341,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_DECAGI','attack',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','attack',28,8,1000,0,5000,'yes','friend','friendhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','attack',28,8,1000,0,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','idle',28,8,1000,0,5000,'yes','friend','friendhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','idle',28,8,1000,0,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','chase',28,8,1000,0,5000,'yes','friend','friendhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1707,'Thanatos Dolor@AL_HEAL','chase',28,8,1000,0,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_SUMMONSLAVE','attack',196,6,10000,700,30000,'no','self','slavele','0',1711,1710,1709,1712,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_SUMMONSLAVE','idle',196,6,10000,700,30000,'no','self','slavele','0',1711,1710,1709,1712,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_EXPULSION','attack',674,1,1000,1000,10000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@KN_TWOHANDQUICKEN','attack',60,30,3000,0,300000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@KN_BOWLINGBASH','attack',62,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@SM_PROVOKE','attack',6,10,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@NPC_CHANGEHOLY','attack',167,1,5000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_STORMGUST','chase',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_VERMILION','chase',85,21,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_STORMGUST','attack',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_VERMILION','attack',85,21,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_METEOR','chase',83,11,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@WZ_METEOR','attack',83,11,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@MG_SOULSTRIKE','chase',13,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@MG_SOULSTRIKE','attack',13,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@AL_TELEPORT','walk',26,1,10000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1708,'Thanatos@AL_TELEPORT','attack',26,1,500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_DARKNESSATTACK','attack',190,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_CHANGEDARKNESS','attack',168,1,1000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_POWERUP','attack',349,5,2000,0,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_POWERUP','attack',349,5,2000,0,10000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1709,'Thanatos Odium@NPC_TELEKINESISATTACK','attack',191,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_FIREATTACK','attack',186,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_STUNATTACK','attack',179,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@WZ_SIGHTRASHER','attack',81,10,1000,0,10000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_AGIUP','attack',350,5,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@NPC_AGIUP','attack',350,5,2000,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@SM_PROVOKE','attack',6,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1710,'Thanatos Despero@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@NPC_DARKSTRIKE','attack',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@NPC_DARKSTRIKE','chase',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@AS_SONICBLOW','attack',136,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@NPC_TELEKINESISATTACK','attack',191,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@NPC_INVISIBLE','attack',353,1,500,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1711,'Thanatos Maero@NPC_INVISIBLE','idle',353,1,500,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@NPC_MAGICALATTACK','attack',192,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@NPC_MENTALBREAKER','attack',159,2,500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@NPC_MENTALBREAKER','chase',159,2,500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@NPC_DARKTHUNDER','attack',341,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@NPC_DARKTHUNDER','chase',341,9,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_DECAGI','attack',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','attack',28,11,1000,0,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','attack',28,11,1000,0,5000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','idle',28,9,1000,0,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','idle',28,9,1000,0,5000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','chase',28,9,1000,0,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1712,'Thanatos Dolor@AL_HEAL','chase',28,9,1000,0,5000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'3',NULL);

# Abyss Lake
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@NPC_FIREATTACK','attack',186,6,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@NPC_STUNATTACK','attack',179,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@MG_FIREBALL','idle',17,9,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@MG_FIREBALL','chase',17,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@SM_MAGNUM','attack',7,9,2000,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@WZ_FIREPILLAR','idle',80,3,100,0,30000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1713,'Acidus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@KN_BRANDISHSPEAR','attack',57,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',36,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1714,'Ferus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1715,'Novus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1715,'Novus@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1715,'Novus@SM_BASH','attack',5,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1715,'Novus@NPC_PIERCINGATT','attack',158,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1715,'Novus@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'37',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@NPC_WINDATTACK','attack',187,6,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@MG_LIGHTNINGBOLT','idle',20,8,2000,1000,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@MG_LIGHTNINGBOLT','chase',20,8,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@MG_LIGHTNINGBOLT','attack',20,8,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@MG_THUNDERSTORM','attack',21,9,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@MG_THUNDERSTORM','attack',21,9,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1716,'Acidus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@KN_BRANDISHSPEAR','attack',57,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@NPC_PIERCINGATT','attack',158,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1717,'Ferus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1718,'Novus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1718,'Novus@SM_BASH','attack',5,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1718,'Novus@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1718,'Novus@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_SUMMONSLAVE','attack',196,1,10000,2000,5000,'no','self','slavele','0',1756,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_SUMMONSLAVE','idle',196,1,10000,2000,5000,'no','self','slavele','0',1756,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_SUMMONSLAVE','chase',196,1,10000,2000,5000,'no','self','slavele','0',1756,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_FIREBREATH','attack',654,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_DARKBREATH','attack',202,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@NPC_WEAPONBRAKER','attack',343,10,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@CR_REFLECTSHIELD','attack',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@WZ_METEOR','attack',83,11,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@WZ_METEOR','chase',83,11,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@MG_FIREBALL','chase',17,44,5000,500,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@SM_PROVOKE','attack',6,10,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1719,'Detale@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_RANDOMATTACK','attack',183,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@AS_SONICBLOW','attack',136,10,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_POWERUP','attack',349,3,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@WZ_METEOR','attack',83,11,2000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_SUMMONSLAVE','attack',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_SUMMONSLAVE','idle',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_SUMMONSLAVE','chase',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1720,'Hydro@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1721,'Dragon Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',1715,1718,NULL,NULL,NULL,NULL,NULL);

# Kiel dungeon
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@SM_MAGNUM','chase',7,5,2500,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@SM_MAGNUM','attack',7,5,2500,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@MG_SAFETYWALL','idle',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@MG_SAFETYWALL','attack',12,10,4000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@SM_ENDURE','chase',8,10,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@SM_ENDURE','attack',8,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@PR_LEXDIVINA','attack',76,5,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@PR_LEXDIVINA','attack',76,5,1000,0,30000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@KN_BRANDISHSPEAR','chase',57,6,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@KN_BRANDISHSPEAR','attack',57,6,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@AL_PNEUMA','attack',25,1,10000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1733,'Kiel@AL_TELEPORT','walk',26,1,10000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_SUMMONSLAVE','attack',196,6,10000,700,30000,'no','self','slavele','0',1739,1740,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_SUMMONSLAVE','idle',196,6,10000,700,30000,'no','self','slavele','0',1739,1740,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@HW_GANBANTEIN','attack',483,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@SM_MAGNUM','chase',7,25,1000,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@SM_MAGNUM','attack',7,25,1000,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_WIDESILENCE','chase',663,5,10000,500,25000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_WIDESILENCE','attack',663,5,10000,500,25000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_DARKBLESSING','attack',203,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@MG_SAFETYWALL','attack',12,10,4000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@SM_ENDURE','chase',8,10,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@SM_ENDURE','attack',8,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_POWERUP','attack',349,4,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@NPC_POWERUP','attack',349,4,3000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@PR_LEXDIVINA','attack',76,5,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@PR_LEXDIVINA','attack',76,5,1000,0,30000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@KN_BRANDISHSPEAR','chase',57,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@KN_BRANDISHSPEAR','attack',57,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@AL_TELEPORT','walk',26,1,10000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@AL_PNEUMA','attack',25,1,10000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1734,'Kiel-D-01@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1735,'Alicel@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1735,'Alicel@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1735,'Alicel@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1735,'Alicel@NPC_GUIDEDATTACK','attack',172,1,1000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1735,'Alicel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@KN_SPEARSTAB','attack',58,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@CR_AUTOGUARD','attack',249,5,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1736,'Aliot@NPC_REBIRTH','dead',208,3,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@SM_BASH','attack',5,10,2000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'52',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@NPC_EMOTION','walk',197,1,500,0,30000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@NPC_EMOTION','idle',197,1,500,0,30000,'yes','self','always','0',54,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@AL_HEAL','idle',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@AL_HEAL','walk',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@AL_HEAL','chase',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1737,'Aliza@AL_HEAL','attack',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1738,'Constant@NPC_SELFDESTRUCTION','attack',173,1,10000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1738,'Constant@NPC_SELFDESTRUCTION','chase',173,1,2000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1739,'Alicel@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1739,'Alicel@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1739,'Alicel@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1739,'Alicel@NPC_GUIDEDATTACK','attack',172,1,1000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@KN_SPEARSTAB','attack',58,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@CR_AUTOGUARD','attack',249,5,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1740,'Aliot@NPC_REBIRTH','dead',208,3,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@NPC_HOLYATTACK','attack',189,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1741,'Christmas Cookie@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1742,'Carat@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1742,'Carat@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1742,'Carat@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1742,'Carat@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1743,'Myst Case@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1744,'Wild Rose@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1744,'Wild Rose@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1744,'Wild Rose@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1745,'Constant@NPC_SELFDESTRUCTION','attack',173,1,10000,0,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1745,'Constant@NPC_SELFDESTRUCTION','chase',173,1,2000,0,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_SUMMONSLAVE','attack',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_SUMMONSLAVE','idle',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_SUMMONSLAVE','chase',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_SUMMONSLAVE','idle',196,3,10000,0,0,'no','self','onspawn','0',1765,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@HP_ASSUMPTIO','chase',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@HP_ASSUMPTIO','attack',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@HW_GANBANTEIN','attack',483,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_EARTHQUAKE','chase',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1751,'Valkyrie Randgris@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@TF_HIDING','idle',51,1,2000,0,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@MG_SOULSTRIKE','chase',13,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@AS_GRIMTOOTH','idle',137,5,10000,0,700,'yes','randomtarget','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@NPC_CURSEATTACK','attack',181,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@NPC_DARKNESSATTACK','attack',190,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1752,'Skogul@NPC_SUMMONSLAVE','attack',196,1,100,0,3600000,'no','self','slavele','0',1762,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@RG_STRIPARMOR','attack',217,3,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@NPC_POISON','attack',176,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@NPC_DARKNESSATTACK','attack',190,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@NPC_CHANGEUNDEAD','attack',348,2,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@MG_FIREWALL','chase',18,5,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1753,'Frus@NPC_SUMMONSLAVE','attack',196,1,100,0,3600000,'no','self','slavele','0',1761,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_HEAL','attack',28,9,5000,500,5000,'no','friend','friendhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_HEAL','attack',28,9,5000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_HEAL','idle',28,9,5000,500,5000,'no','friend','friendhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_HEAL','idle',28,9,5000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@MG_THUNDERSTORM','attack',21,7,1000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@MG_THUNDERSTORM','chase',21,7,1000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_DECAGI','attack',30,48,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_DECAGI','chase',30,48,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_PETRIFYATTACK','attack',180,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_PETRIFYATTACK','chase',180,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@RG_STRIPARMOR','attack',217,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@RG_STRIPARMOR','chase',217,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@MG_SAFETYWALL','attack',12,10,1000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_HOLYATTACK','attack',189,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_HOLYATTACK','chase',189,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1754,'Skeggiold@NPC_SUMMONSLAVE','any',196,2,10000,700,60000,'no','self','onspawn','0',1763,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_HEAL','attack',28,9,5000,500,5000,'no','friend','friendhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_HEAL','attack',28,9,5000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_HEAL','idle',28,9,5000,500,5000,'no','friend','friendhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_HEAL','idle',28,9,5000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@CR_HOLYCROSS','attack',253,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@MG_SOULSTRIKE','chase',13,7,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@MG_SOULSTRIKE','attack',13,7,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@MG_SAFETYWALL','attack',12,10,1000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@NPC_HOLYATTACK','attack',189,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@NPC_HOLYATTACK','chase',189,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1755,'Skeggiold@NPC_SUMMONSLAVE','any',196,2,10000,700,60000,'no','self','onspawn','0',1764,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_RANDOMATTACK','attack',183,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@AS_SONICBLOW','attack',136,10,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_POWERUP','attack',349,3,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@WZ_METEOR','attack',83,11,2000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_SUMMONSLAVE','attack',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_SUMMONSLAVE','idle',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_SUMMONSLAVE','chase',196,4,10000,2000,5000,'no','self','slavele','2',1757,1758,1759,1760,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1756,'Hydro@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1761,'Skogul@TF_HIDING','idle',51,1,2000,0,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1761,'Skogul@MG_SOULSTRIKE','chase',13,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1761,'Skogul@AS_GRIMTOOTH','idle',137,5,10000,0,700,'yes','randomtarget','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1761,'Skogul@NPC_CURSEATTACK','attack',181,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1761,'Skogul@NPC_DARKNESSATTACK','attack',190,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1762,'Frus@RG_STRIPARMOR','attack',217,3,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1762,'Frus@NPC_POISON','attack',176,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1762,'Frus@NPC_DARKNESSATTACK','attack',190,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1762,'Frus@NPC_CHANGEUNDEAD','attack',348,2,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1762,'Frus@MG_FIREWALL','chase',18,5,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@MG_THUNDERSTORM','attack',21,4,500,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@MG_THUNDERSTORM','chase',21,4,500,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@AL_DECAGI','attack',30,5,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@AL_DECAGI','chase',30,5,500,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@NPC_PETRIFYATTACK','attack',180,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@MG_SAFETYWALL','attack',12,5,500,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@NPC_HOLYATTACK','attack',189,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@NPC_HOLYATTACK','chase',189,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1763,'Skeggiold@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@CR_HOLYCROSS','attack',253,2,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@MG_SOULSTRIKE','chase',13,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@MG_SOULSTRIKE','attack',13,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@MG_SAFETYWALL','attack',12,5,1000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@NPC_HOLYATTACK','attack',189,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@NPC_HOLYATTACK','chase',189,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1764,'Skeggiold@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@AL_HEAL','idle',28,10,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@AL_HEAL','attack',28,11,10000,500,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@KN_BRANDISHSPEAR','attack',57,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@NPC_ARMORBRAKE','attack',344,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1765,'Valkyrie@WZ_JUPITEL','attack',84,28,100,2000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@MG_SAFETYWALL','attack',12,5,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@NPC_HOLYATTACK','attack',189,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@NPC_HOLYATTACK','chase',189,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@NPC_SUMMONSLAVE','attack',196,5,10000,2000,60000,'no','self','slavele','3',1031,1062,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@NPC_SUMMONSLAVE','idle',196,5,10000,2000,60000,'no','self','slavele','3',1031,1062,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1766,'Angeling@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@AL_DECAGI','chase',30,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@NPC_DARKNESSATTACK','attack',190,2,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@NPC_DARKSTRIKE','attack',340,5,3000,700,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@NPC_DARKSTRIKE','chase',340,5,3000,700,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@NPC_SUMMONSLAVE','attack',196,7,10000,2000,10000,'no','self','slavele','3',1595,1595,1595,1433,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1767,'Deviling@NPC_SUMMONSLAVE','idle',196,7,10000,2000,10000,'no','self','slavele','3',1595,1595,1595,1433,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_FIREATTACK','attack',186,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_TELEKINESISATTACK','attack',191,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@WZ_FIREPILLAR','idle',80,10,1000,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@AM_DEMONSTRATION','attack',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@KN_PIERCE','attack',56,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_POWERUP','attack',349,5,10000,700,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_POWERUP','attack',349,5,2000,700,50000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_FIREBREATH','chase',654,10,2000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@WZ_SIGHTRASHER','attack',81,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@MG_FIREBALL','attack',17,44,1000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@MG_FIREBALL','chase',17,44,10000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_SUMMONSLAVE','attack',196,4,10000,700,30000,'no','self','slavele','3',1786,1787,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@NPC_SUMMONSLAVE','idle',196,4,10000,700,30000,'no','self','slavele','3',1786,1787,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1768,'Gloom Under Night@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_SIGHT','idle',10,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_SOULSTRIKE','chase',13,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_SOULSTRIKE','attack',13,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_THUNDERSTORM','attack',21,9,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_THUNDERSTORM','chase',21,9,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@NPC_HOLYATTACK','attack',189,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@PR_LEXDIVINA','attack',76,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@PR_LEXDIVINA','chase',76,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1769,'Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@CR_HOLYCROSS','attack',253,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@SM_BASH','attack',5,7,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@NPC_SLEEPATTACK','attack',182,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@AS_SONICBLOW','attack',136,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@NPC_HOLYATTACK','attack',189,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@BS_MAXIMIZE','attack',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@BS_MAXIMIZE','chase',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1770,'Echio@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@CR_HOLYCROSS','attack',253,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@BS_MAXIMIZE','attack',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1771,'Vanberk@BS_MAXIMIZE','chase',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@MG_THUNDERSTORM','attack',21,5,500,1200,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@MG_THUNDERSTORM','chase',21,5,500,1200,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@NPC_HOLYATTACK','attack',189,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@PR_LEXDIVINA','attack',76,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@PR_LEXDIVINA','chase',76,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','attack',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','attack',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','idle',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','idle',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','chase',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1772,'Isilla@AL_HEAL','chase',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@AS_SONICBLOW','attack',136,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@NPC_DARKNESSATTACK','attack',190,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',52,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1773,'Hodremlin@RG_STRIPARMOR','attack',217,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'52',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@WZ_JUPITEL','chase',84,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@NPC_MENTALBREAKER','attack',159,3,200,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@NPC_MENTALBREAKER','chase',159,3,200,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@NPC_PETRIFYATTACK','attack',180,3,200,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@NPC_PETRIFYATTACK','chase',180,3,200,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1774,'Seeker@SA_DISPELL','attack',289,1,200,0,100000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@NPC_WATERATTACK','attack',184,4,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1775,'Snowier@SM_ENDURE','chase',8,3,5000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',45,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','myhpltmaxrate','1',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','myhpltmaxrate','1',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_SILENCEATTACK','chase',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@NPC_WATERATTACK','attack',184,2,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1776,'Siroma@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1777,'Ice Titan@KN_BOWLINGBASH','attack',62,7,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1777,'Ice Titan@MG_FROSTDIVER','chase',15,7,10000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1777,'Ice Titan@MG_FROSTDIVER','attack',15,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1777,'Ice Titan@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1777,'Ice Titan@NPC_WATERATTACK','attack',184,3,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1778,'Gazeti@CR_AUTOGUARD','attack',249,3,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1778,'Gazeti@MG_COLDBOLT','chase',14,3,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1778,'Gazeti@MG_COLDBOLT','attack',14,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1778,'Gazeti@NPC_GUIDEDATTACK','attack',172,2,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1778,'Gazeti@NPC_WATERATTACK','attack',184,2,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_WATERATTACK','attack',184,10,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_RANDOMATTACK','attack',183,8,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_SUMMONSLAVE','attack',196,4,10000,700,30000,'no','self','slavele','3',1788,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_SUMMONSLAVE','idle',196,4,10000,700,30000,'no','self','slavele','3',1788,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_WIDEFREEZE','chase',664,5,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_WIDEFREEZE','attack',664,5,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@SA_DISPELL','chase',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@MG_FROSTDIVER','idle',15,40,5000,1000,7000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@MG_FROSTDIVER','chase',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_STORMGUST','chase',89,10,5000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_STORMGUST','attack',89,10,5000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_WATERBALL','chase',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_WATERBALL','attack',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_FROSTNOVA','chase',88,10,5000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@WZ_FROSTNOVA','attack',88,10,5000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@NPC_AGIUP','attack',350,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1779,'Ktullanux@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@NPC_GROUNDATTACK','attack',185,1,1000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1780,'Muscipular@NPC_PETRIFYATTACK','attack',180,5,500,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@KN_PIERCE','attack',56,5,500,700,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@NPC_GROUNDATTACK','attack',185,1,1000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1781,'Drosera@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1782,'Roween@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1782,'Roween@NPC_COMBOATTACK','attack',171,4,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1782,'Roween@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_COMBOATTACK','attack',171,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_SUMMONSLAVE','attack',196,3,10000,1000,30000,'no','self','slavele','3',1794,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1783,'Galion@NPC_SUMMONSLAVE','idle',196,3,10000,1000,30000,'no','self','slavele','3',1794,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1784,'Stapo@NPC_KEEPING','attack',201,1,1000,1000,5000,'no','self','myhpltmaxrate','25',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1784,'Stapo@SM_BASH','attack',5,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_SUMMONSLAVE','attack',196,3,10000,700,30000,'no','self','slavele','0',1791,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_SUMMONSLAVE','idle',196,3,10000,700,30000,'no','self','slavele','0',1791,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_DEFENDER','idle',205,1,10000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@KN_TWOHANDQUICKEN','attack',60,30,2000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_BLEEDING','attack',660,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_DARKNESSATTACK','attack',190,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_POWERUP','attack',349,2,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@NPC_POWERUP','attack',349,2,2000,0,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1785,'Atroce@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_SIGHT','idle',10,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_SOULSTRIKE','chase',13,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_SOULSTRIKE','attack',13,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_THUNDERSTORM','attack',21,9,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_THUNDERSTORM','chase',21,9,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@NPC_HOLYATTACK','attack',189,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@PR_LEXDIVINA','attack',76,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@PR_LEXDIVINA','chase',76,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1786,'Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@CR_HOLYCROSS','attack',253,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@SM_BASH','attack',5,7,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@NPC_SLEEPATTACK','attack',182,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@AS_SONICBLOW','attack',136,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@NPC_HOLYATTACK','attack',189,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@BS_MAXIMIZE','attack',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@BS_MAXIMIZE','chase',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1787,'Echio@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1788,'Ice Titan@KN_BOWLINGBASH','attack',62,7,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1788,'Ice Titan@MG_FROSTDIVER','chase',15,7,10000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1788,'Ice Titan@MG_FROSTDIVER','attack',15,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1788,'Ice Titan@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1788,'Ice Titan@NPC_WATERATTACK','attack',184,3,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@NPC_COMBOATTACK','attack',171,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1791,'Galion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1794,'Roween@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1794,'Roween@NPC_COMBOATTACK','attack',171,4,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1794,'Roween@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@KN_BOWLINGBASH','attack',62,10,1000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@KN_BOWLINGBASH','attack',62,10,2000,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@CR_GRANDCROSS','attack',254,10,2000,1500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_STORMGUST','idle',89,10,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_VERMILION','idle',85,21,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_METEOR','idle',83,11,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_STORMGUST','chase',89,10,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_VERMILION','chase',85,21,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_METEOR','chase',83,11,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_STORMGUST','chase',89,10,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_VERMILION','chase',85,21,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_METEOR','chase',83,11,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_STORMGUST','attack',89,10,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_VERMILION','attack',85,21,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1795,'Bloody Knight@WZ_METEOR','attack',83,11,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# Guild Dungeon SE (13.1)
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@MG_THUNDERSTORM','attack',21,5,500,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@MG_THUNDERSTORM','chase',21,5,500,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@NPC_HOLYATTACK','attack',189,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@PR_LEXDIVINA','attack',76,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@PR_LEXDIVINA','chase',76,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','attack',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','attack',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','idle',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','idle',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','chase',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1796,'Aunoe@AL_HEAL','chase',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@CR_HOLYCROSS','attack',253,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@BS_MAXIMIZE','attack',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1797,'Fanat@BS_MAXIMIZE','chase',114,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@NPC_METAMORPHOSIS','attack',193,1,1000,0,60000,'no','self','myhpltmaxrate','30',1814,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@PR_LEXAETERNA','chase',78,1,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1812,'Delightful Lude@PR_LEXAETERNA','attack',78,1,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@AL_HEAL','attack',28,11,2000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_DARKTHUNDER','chase',341,5,8000,700,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_GRANDDARKNESS','attack',339,10,2000,900,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_DARKCROSS','attack',338,10,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@WZ_VERMILION','attack',85,21,2500,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_COMBOATTACK','attack',171,6,2500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@AS_VENOMDUST','attack',140,10,3500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_LICK','attack',206,5,4000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@KN_TWOHANDQUICKEN','attack',60,30,5000,0,120000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1813,'Hydrolancer@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@AL_HEAL','attack',28,9,10000,0,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@AL_HEAL','chase',28,9,10000,0,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MG_FIREBOLT','chase',19,10,3000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MG_FIREBOLT','attack',19,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MG_FIREBOLT','chase',19,10,4000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@WZ_WATERBALL','attack',86,5,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@WZ_WATERBALL','chase',86,5,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@WZ_METEOR','attack',83,5,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@WZ_METEOR','chase',83,5,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MC_MAMMONITE','attack',42,22,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MG_FIREWALL','attack',18,10,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@MG_FIREWALL','chase',18,10,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@SA_DISPELL','attack',289,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1814,'Moonlight Flower@SA_DISPELL','chase',289,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@NPC_DARKBREATH','attack',202,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','12',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','myhpltmaxrate','12',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@BS_HAMMERFALL','attack',110,10,10000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@BS_HAMMERFALL','chase',110,10,5000,0,5000,'yes','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@NPC_WEAPONBRAKER','attack',343,10,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@CR_REFLECTSHIELD','attack',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@CR_GRANDCROSS','attack',254,10,5000,100,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@WZ_STORMGUST','attack',89,10,2000,1000,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@MG_FIREBALL','chase',17,44,5000,500,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@WZ_METEOR','attack',83,11,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@WZ_METEOR','chase',83,11,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@BS_MAXIMIZE','chase',114,5,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1817,'Detale@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',30,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1819,'Bathory@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1820,'Bigfoot@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1820,'Bigfoot@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1820,'Bigfoot@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1820,'Bigfoot@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1820,'Bigfoot@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1821,'Desert Wolf@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1821,'Desert Wolf@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1821,'Desert Wolf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1821,'Desert Wolf@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_DARKTHUNDER','attack',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1822,'Deviruchi@NPC_DARKTHUNDER','chase',341,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@MG_FROSTDIVER','attack',15,9,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1823,'Freezer@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1824,'Garm Baby@MG_FROSTDIVER','chase',15,40,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1824,'Garm Baby@NPC_LICK','attack',206,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1825,'Christmas Goblin@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1825,'Christmas Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',18,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1825,'Christmas Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1825,'Christmas Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1825,'Christmas Goblin@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1826,'Myst@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1826,'Myst@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1826,'Myst@NPC_MENTALBREAKER','attack',159,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1826,'Myst@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1827,'Sasquatch@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1827,'Sasquatch@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1827,'Sasquatch@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1827,'Sasquatch@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1827,'Sasquatch@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);

# Thor Volcano (11.2)
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@KN_SPEARBOOMERANG','chase',59,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@CR_SHIELDCHARGE','attack',250,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@NPC_STUNATTACK','attack',179,5,1500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@NPC_COMBOATTACK','attack',171,5,1500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1829,'Sword Guardian@NPC_WIDEBLEEDING','attack',665,4,10000,0,15000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_DOUBLE','idle',46,10,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_SHOWER','idle',47,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_CHARGEARROW','idle',148,1,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_CHARGEARROW','chase',148,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_DOUBLE','attack',46,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_SHOWER','attack',47,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@AC_CHARGEARROW','attack',148,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@NPC_STUNATTACK','attack',179,5,1500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@NPC_COMBOATTACK','attack',171,5,1500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@NPC_BLEEDING','attack',660,5,1500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1830,'Bow Guardian@NPC_WIDEBLEEDING','attack',665,4,10000,0,15000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@MG_FIREBALL','idle',17,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@MG_FIREBALL','chase',17,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@CR_REFLECTSHIELD','idle',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1831,'Salamander@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@AL_HEAL','idle',28,1,10000,0,10000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_SUMMONSLAVE','attack',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_SUMMONSLAVE','idle',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_SUMMONSLAVE','chase',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@AS_SONICBLOW','attack',136,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_FIREBREATH','chase',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_FIREBREATH','attack',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@SA_LANDPROTECTOR','attack',288,3,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@WZ_METEOR','attack',83,11,2000,500,5000,'no','target','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@KN_TWOHANDQUICKEN','attack',60,9,10000,0,100000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_EARTHQUAKE','chase',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_EARTHQUAKE','attack',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_PULSESTRIKE','attack',661,5,10000,0,15000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_DRAGONFEAR','attack',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@NPC_DRAGONFEAR','chase',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1832,'Ifrit@WZ_METEOR','idle',83,9,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@CR_REFLECTSHIELD','idle',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@CR_REFLECTSHIELD','chase',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@CR_REFLECTSHIELD','attack',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@WZ_FIREPILLAR','idle',80,5,10000,500,30000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@NPC_FIREBREATH','chase',654,10,2000,1500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@NPC_FIREBREATH','attack',654,10,2000,1500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@MG_FIREBALL','idle',17,44,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@MG_FIREBALL','chase',17,44,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@MG_FIREBALL','attack',17,44,5000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@WZ_SIGHTRASHER','attack',81,10,1000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@LK_SPIRALPIERCE','attack',397,1,1500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@WZ_METEOR','chase',83,1,5000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@WZ_METEOR','attack',83,1,5000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1833,'Kasa@HW_GANBANTEIN','attack',483,5,5000,500,5000,'no','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@MG_FIREBALL','idle',17,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@MG_FIREBALL','chase',17,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1834,'Salamander@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@CR_REFLECTSHIELD','idle',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@CR_REFLECTSHIELD','chase',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@CR_REFLECTSHIELD','attack',252,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@WZ_FIREPILLAR','idle',80,5,10000,500,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@NPC_FIREBREATH','chase',654,10,2000,3000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@NPC_FIREBREATH','attack',654,10,2000,3000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@MG_FIREBALL','idle',17,44,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@MG_FIREBALL','chase',17,44,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@MG_FIREBALL','attack',17,44,5000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@WZ_SIGHTRASHER','attack',81,10,1000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@LK_SPIRALPIERCE','attack',397,1,1500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@WZ_METEOR','chase',83,1,5000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1835,'Kasa@WZ_METEOR','attack',83,1,5000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1836,'Magmaring@NPC_FIREATTACK','attack',186,2,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1836,'Magmaring@MG_FIREBOLT','attack',19,2,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1836,'Magmaring@SM_BASH','attack',5,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1836,'Magmaring@MG_FIREBALL','chase',17,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1836,'Magmaring@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@KN_BRANDISHSPEAR','attack',57,8,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@NPC_FIREATTACK','attack',186,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@NPC_PULSESTRIKE','attack',661,3,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@MG_SIGHT','idle',10,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@WZ_SIGHTRASHER','attack',81,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1837,'Imp@RG_STRIPARMOR','attack',217,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1838,'Knocker@HT_LANDMINE','idle',116,2,4000,1000,18000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1838,'Knocker@NPC_GROUNDATTACK','attack',185,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1838,'Knocker@TF_SPRINKLESAND','attack',149,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1838,'Knocker@TF_THROWSTONE','chase',152,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1838,'Knocker@TF_THROWSTONE','attack',152,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_SUMMONSLAVE','idle',196,2,10000,1000,60000000,'no','self','slavele','1',1829,1830,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_SUMMONSLAVE','chase',196,2,10000,1000,60000000,'no','self','slavele','1',1829,1830,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_SUMMONSLAVE','attack',196,2,10000,1000,60000000,'no','self','slavele','1',1829,1830,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@AS_GRIMTOOTH','idle',137,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@AS_GRIMTOOTH','chase',137,5,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@ASC_BREAKER','chase',379,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@ASC_METEORASSAULT','attack',406,2,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@AS_SONICBLOW','attack',136,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1839,'Byorgue@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@ST_FULLSTRIP','attack',476,5,100,1500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@KN_TWOHANDQUICKEN','attack',60,5,500,0,30000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@WZ_EARTHSPIKE','attack',90,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@WZ_HEAVENDRIVE','attack',91,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@WZ_QUAGMIRE','attack',92,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@WZ_QUAGMIRE','chase',92,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1840,'Golden Savage@NPC_PULSESTRIKE','attack',661,1,500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);

# Cursed Abbey (11.3)
REPLACE INTO `mob_skill_db_re` VALUES (1864,'Zombie Slaughter@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1864,'Zombie Slaughter@NPC_POISON','attack',176,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1864,'Zombie Slaughter@AS_SONICBLOW','attack',136,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1864,'Zombie Slaughter@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1865,'Ragged Zombie@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1865,'Ragged Zombie@AC_DOUBLE','attack',46,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1865,'Ragged Zombie@NPC_MAGICALATTACK','attack',192,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1865,'Ragged Zombie@AC_SHOWER','attack',47,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1866,'Hell Poodle@AL_HEAL','idle',28,1,10000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1866,'Hell Poodle@AS_SONICBLOW','attack',136,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1866,'Hell Poodle@NPC_RUN','attack',354,1,10000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1866,'Hell Poodle@NPC_RUN','chase',354,1,10000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_VAMPIRE_GIFT','attack',679,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_EVILLAND','attack',670,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1867,'Banshee@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',NULL,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_VAMPIRE_GIFT','attack',679,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1868,'Banshee@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',NULL,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1869,'Flame Skull@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1869,'Flame Skull@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1869,'Flame Skull@MG_FIREBALL','chase',17,5,1500,500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1869,'Flame Skull@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_SUMMONSLAVE','idle',196,2,10000,0,300000,'yes','self','slavele','0',1864,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_SUMMONSLAVE','chase',196,2,10000,0,300000,'yes','self','slavele','0',1864,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_SUMMONSLAVE','attack',196,2,10000,0,300000,'yes','self','slavele','0',1864,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_CALLSLAVE','attack',352,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_CALLSLAVE','idle',352,1,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@MG_FIREBALL','chase',17,5,2000,300,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@MG_THUNDERSTORM','attack',21,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@MG_THUNDERSTORM','chase',21,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@WZ_QUAGMIRE','attack',92,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@WZ_QUAGMIRE','chase',92,5,500,200,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_WIDECURSE','chase',677,1,500,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_WIDECURSE','attack',677,1,500,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_SLOWCAST','chase',672,1,2000,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_SLOWCAST','attack',672,1,2000,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_DARKSTRIKE','chase',340,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_DARKSTRIKE','attack',340,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_CHANGEUNDEAD','attack',348,1,500,500,6000000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_DARKNESSBREATH','attack',658,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@NPC_DRAGONFEAR','attack',659,2,500,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@WZ_METEOR','chase',83,3,3000,500,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1870,'Necromancer@WZ_METEOR','chase',83,3,1000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_SUMMONSLAVE','attack',196,4,10000,700,10000,'no','self','slavele','3',1868,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_SUMMONSLAVE','idle',196,4,10000,700,10000,'no','self','slavele','3',1868,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_WIDECURSE','attack',677,5,2000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_DARKSTRIKE','chase',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_DARKSTRIKE','attack',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_DARKCROSS','attack',338,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@CR_HOLYCROSS','attack',253,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_GRANDDARKNESS','chase',339,10,500,1300,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_GRANDDARKNESS','attack',339,10,500,1300,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@CR_GRANDCROSS','chase',254,10,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@CR_GRANDCROSS','attack',254,10,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_VAMPIRE_GIFT','attack',679,2,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_SLOWCAST','chase',672,5,2000,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_SLOWCAST','attack',672,5,2000,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_EVILLAND','attack',670,10,10000,1000,600000,'no','target','myhpltmaxrate','44',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_EVILLAND','attack',670,7,500,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_MAGICMIRROR','attack',671,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_HELLJUDGEMENT','chase',662,4,10000,800,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_HELLJUDGEMENT','attack',662,4,10000,800,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_WIDESILENCE','chase',663,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_WIDESILENCE','attack',663,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_DARKNESSATTACK','attack',190,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1871,'Fallen Bishop@NPC_DARKNESSBREATH','attack',658,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1872,'Hell Fly@NPC_CRITICALWOUND','attack',673,5,10,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1872,'Hell Fly@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1872,'Hell Fly@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1872,'Hell Fly@NPC_WINDATTACK','attack',187,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1872,'Hell Fly@NPC_MENTALBREAKER','attack',159,1,200,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@AL_HEAL','idle',28,9,10000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_CALLSLAVE','attack',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_CALLSLAVE','chase',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_CALLSLAVE','idle',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_SUMMONSLAVE','attack',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_SUMMONSLAVE','chase',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_SUMMONSLAVE','idle',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_METAMORPHOSIS','chase',193,1,10000,5000,5000,'no','self','myhpltmaxrate','50',1874,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_METAMORPHOSIS','attack',193,1,10000,5000,5000,'no','self','myhpltmaxrate','50',1874,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@LK_SPIRALPIERCE','attack',397,5,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_PULSESTRIKE','attack',661,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_WIDESLEEP','attack',668,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_WIDESILENCE','attack',663,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_HELLJUDGEMENT','attack',662,5,10000,700,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_HELLJUDGEMENT','chase',662,5,10000,700,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_EXPULSION','attack',674,1,100,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_VAMPIRE_GIFT','attack',679,10,10000,1000,10000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1873,'Beelzebub@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_RUN','attack',354,1,10000,0,0,'yes','self','myhpltmaxrate','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_RUN','chase',354,1,10000,0,0,'yes','self','myhpltmaxrate','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@AL_HEAL','idle',28,9,10000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_CALLSLAVE','attack',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_CALLSLAVE','chase',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_CALLSLAVE','idle',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_SUMMONSLAVE','attack',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_SUMMONSLAVE','chase',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_SUMMONSLAVE','idle',196,10,10000,0,10000,'yes','self','slavele','5',1872,1872,1011,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_STONESKIN','chase',675,4,10000,1000,30000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_STONESKIN','attack',675,4,10000,1000,30000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_EVILLAND','attack',670,10,10000,0,15000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@SA_LANDPROTECTOR','attack',288,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_SHIELDBRAKE','attack',346,10,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_MAGICMIRROR','attack',671,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_EXPULSION','attack',674,1,50,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_WIDESTONE','attack',666,5,10000,500,15000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_WIDESTONE','chase',666,5,10000,500,15000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@LK_SPIRALPIERCE','attack',397,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_PULSESTRIKE','attack',661,5,10000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_HELLJUDGEMENT','chase',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_HELLJUDGEMENT','attack',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_ACIDBREATH','attack',657,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_ACIDBREATH','chase',657,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_VAMPIRE_GIFT','attack',679,1,1000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1874,'Beelzebub@NPC_WIDESOULDRAIN','attack',680,7,1000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'42',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1875,'Tristan@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Moscovia (12.1)
REPLACE INTO `mob_skill_db_re` VALUES (1880,'Wood Goblin@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1880,'Wood Goblin@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1880,'Wood Goblin@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1880,'Wood Goblin@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1880,'Wood Goblin@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1881,'Les@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1881,'Les@NPC_COMBOATTACK','attack',171,2,3000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1881,'Les@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1881,'Les@NPC_POISON','attack',176,2,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1882,'Baba Yaga@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1882,'Baba Yaga@WZ_WATERBALL','chase',86,1,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1882,'Baba Yaga@NPC_COMBOATTACK','attack',171,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1882,'Baba Yaga@WZ_WATERBALL','attack',86,2,2000,3000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1883,'Uzhas@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1883,'Uzhas@NPC_MAGICALATTACK','chase',192,1,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1883,'Uzhas@NPC_POISON','attack',176,2,2000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1883,'Uzhas@WZ_WATERBALL','attack',86,3,2000,3000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1883,'Uzhas@NPC_CURSEATTACK','attack',181,2,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1884,'Mavka@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1884,'Mavka@AL_HEAL','idle',28,5,10000,2000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1884,'Mavka@WZ_EARTHSPIKE','chase',90,2,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1884,'Mavka@NPC_GROUNDATTACK','attack',185,2,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1884,'Mavka@NPC_SILENCEATTACK','attack',178,2,2000,2000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_SUMMONSLAVE','attack',196,5,10000,1000,30000,'no','self','slavele','1',1886,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_SUMMONSLAVE','chase',196,5,5000,1000,30000,'no','self','slavele','0',1886,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_SUMMONSLAVE','idle',196,5,10000,1000,30000,'no','self','slavele','0',1886,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@WZ_QUAGMIRE','chase',92,5,5000,200,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_CALLSLAVE','attack',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_CALLSLAVE','chase',352,1,5000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_CALLSLAVE','idle',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@WZ_METEOR','attack',83,11,5000,1500,5000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@MG_FIREBALL','chase',17,9,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@MG_FIREBALL','attack',17,44,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_AGIUP','attack',350,5,3000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@KN_TWOHANDQUICKEN','attack',60,30,5000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_POWERUP','attack',349,5,6000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@WZ_HEAVENDRIVE','chase',91,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@SM_MAGNUM','attack',7,25,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_SILENCEATTACK','attack',178,5,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_MENTALBREAKER','chase',159,5,500,800,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@WZ_HEAVENDRIVE','attack',91,5,5000,500,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@NPC_SILENCEATTACK','chase',178,5,5000,700,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1885,'Gopinich@WZ_QUAGMIRE','attack',92,5,5000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1886,'Mavka@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1886,'Mavka@AL_HEAL','idle',28,9,10000,1000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1886,'Mavka@WZ_EARTHSPIKE','chase',90,4,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1886,'Mavka@NPC_GROUNDATTACK','attack',185,4,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1886,'Mavka@NPC_SILENCEATTACK','attack',178,4,2000,2000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1887,'Freezer@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1887,'Freezer@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1887,'Freezer@MG_FROSTDIVER','attack',15,9,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1887,'Freezer@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1887,'Freezer@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1888,'Garm Baby@MG_FROSTDIVER','chase',15,40,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1888,'Garm Baby@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1889,'Marozka\'s Guard@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1889,'Marozka\'s Guard@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1889,'Marozka\'s Guard@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@WZ_QUAGMIRE','chase',92,1,5000,200,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@MG_FIREBALL','chase',17,9,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@MG_FIREBALL','attack',17,44,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@WZ_HEAVENDRIVE','chase',91,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@SM_MAGNUM','attack',7,9,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@NPC_SILENCEATTACK','attack',178,5,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@WZ_HEAVENDRIVE','attack',91,5,5000,500,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@NPC_SILENCEATTACK','chase',178,5,5000,700,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1890,'The Immortal Koshei@WZ_QUAGMIRE','attack',92,1,5000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@AL_HEAL','idle',28,11,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@AL_HEAL','attack',28,11,10000,500,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@KN_BRANDISHSPEAR','attack',57,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@NPC_ARMORBRAKE','attack',344,10,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1891,'Valkyrie@WZ_JUPITEL','attack',84,28,100,2000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@WZ_STORMGUST','chase',89,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@WZ_STORMGUST','attack',89,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@WZ_WATERBALL','attack',86,10,10000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1894,'Pouring@WZ_WATERBALL','chase',86,10,10000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (1904,'Bomb Poring@NPC_SELFDESTRUCTION','any',173,1,10000,3000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Satan Morroc (12.1)
#5% HP - Dragon Fear
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_DRAGONFEAR','chase',659,10,10000,0,600000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,NULL,'17');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_DRAGONFEAR','attack',659,10,10000,0,600000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,NULL,'17');
#10% HP - Talk
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_TALK','chase',682,10,10000,0,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,'16');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_TALK','attack',682,10,10000,0,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,'16');
#30% HP - Earthquake Combo
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDEHELLDIGNITY','chase',684,1,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'6');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDEHELLDIGNITY','attack',684,1,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'6');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESTUN','chase',678,5,10000,1300,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESTUN','attack',678,5,10000,1300,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_EARTHQUAKE','chase',653,10,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_EARTHQUAKE','attack',653,10,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#40% HP - Mixed Combo
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_PULSESTRIKE','chase',661,5,10000,800,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,'14');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_PULSESTRIKE','attack',661,5,10000,800,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,'14');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESTONE','chase',666,5,10000,0,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESTONE','attack',666,5,10000,0,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@WZ_METEOR','chase',83,5,10000,800,60000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@WZ_METEOR','attack',83,5,10000,800,60000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#50% HP - Vampire Combo
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_VAMPIRE_GIFT','chase',679,9,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'15');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_VAMPIRE_GIFT','attack',679,9,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'15');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESOULDRAIN','chase',680,10,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESOULDRAIN','attack',680,10,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_EVILLAND','chase',670,10,10000,1000,50000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_EVILLAND','attack',670,10,10000,1000,50000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#60% HP - Hell Combo
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESLEEP','chase',668,5,10000,700,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'3');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_WIDESLEEP','attack',668,5,10000,700,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'3');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_HELLPOWER','chase',683,1,10000,800,35000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_HELLPOWER','attack',683,1,10000,800,35000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_HELLJUDGEMENT','chase',662,5,10000,800,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_HELLJUDGEMENT','attack',662,5,10000,800,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#80% HP - Magic Combo
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_STONECURSE','chase',16,10,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'2');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_STONECURSE','attack',16,10,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'2');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_FIREBOLT','chase',19,10,10000,300,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_FIREBOLT','attack',19,10,10000,300,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_FROSTDIVER','chase',15,40,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@MG_FROSTDIVER','attack',15,40,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@WZ_JUPITEL','chase',84,28,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@WZ_JUPITEL','attack',84,28,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_TALK','chase',682,1,10000,0,600000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'1');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_SUMMONSLAVE','attack',196,4,10000,0,30000,'yes','self','slavele','1',1922,1923,1924,1925,NULL,NULL,'4');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_SUMMONSLAVE','chase',196,4,10000,0,30000,'yes','self','slavele','1',1922,1923,1924,1925,NULL,NULL,'4');
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@SA_LANDPROTECTOR','attack',288,1,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1916,'Satan Morroc@NPC_SHIELDBRAKE','attack',346,10,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#5% HP - Dragon Fear
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_DRAGONFEAR','chase',659,10,10000,0,600000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,NULL,'17');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_DRAGONFEAR','attack',659,10,10000,0,600000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,NULL,'17');
#10% HP - Talk
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_TALK','chase',682,10,10000,0,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,'16');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_TALK','attack',682,10,10000,0,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,'16');
#30% HP - Earthquake Combo
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDEHELLDIGNITY','chase',684,1,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'6');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDEHELLDIGNITY','attack',684,1,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'6');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESTUN','chase',678,5,10000,1300,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESTUN','attack',678,5,10000,1300,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_EARTHQUAKE','chase',653,10,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_EARTHQUAKE','attack',653,10,10000,2000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#40% HP - Mixed Combo
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_PULSESTRIKE','chase',661,5,10000,800,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,'14');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_PULSESTRIKE','attack',661,5,10000,800,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,'14');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESTONE','chase',666,5,10000,0,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESTONE','attack',666,5,10000,0,60000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@WZ_METEOR','chase',83,5,10000,800,60000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@WZ_METEOR','attack',83,5,10000,800,60000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#50% HP - Vampire Combo
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_VAMPIRE_GIFT','chase',679,9,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'15');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_VAMPIRE_GIFT','attack',679,9,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'15');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESOULDRAIN','chase',680,10,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESOULDRAIN','attack',680,10,10000,1000,50000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_EVILLAND','chase',670,10,10000,1000,50000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_EVILLAND','attack',670,10,10000,1000,50000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#60% HP - Hell Combo
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESLEEP','chase',668,5,10000,700,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'3');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_WIDESLEEP','attack',668,5,10000,700,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'3');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_HELLPOWER','chase',683,1,10000,800,35000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_HELLPOWER','attack',683,1,10000,800,35000,'no','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_HELLJUDGEMENT','chase',662,5,10000,800,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_HELLJUDGEMENT','attack',662,5,10000,800,35000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#80% HP - Magic Combo
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_STONECURSE','chase',16,10,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'2');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_STONECURSE','attack',16,10,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'2');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_FIREBOLT','chase',19,10,10000,300,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_FIREBOLT','attack',19,10,10000,300,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_FROSTDIVER','chase',15,40,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@MG_FROSTDIVER','attack',15,40,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@WZ_JUPITEL','chase',84,28,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@WZ_JUPITEL','attack',84,28,10000,1000,20000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_TALK','chase',682,1,10000,0,600000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'1');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_SUMMONSLAVE','attack',196,4,10000,0,30000,'yes','self','slavele','1',1922,1923,1924,1925,NULL,NULL,'4');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_SUMMONSLAVE','chase',196,4,10000,0,30000,'yes','self','slavele','1',1922,1923,1924,1925,NULL,NULL,'4');
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@SA_LANDPROTECTOR','attack',288,1,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1917,'Wounded Morroc@NPC_SHIELDBRAKE','attack',346,10,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_PULSESTRIKE','chase',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_MAGICMIRROR','chase',671,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_GRANDDARKNESS','attack',339,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@SA_DISPELL','chase',289,5,100,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@SA_DISPELL','attack',289,5,100,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_WIDECURSE','chase',677,5,5000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1918,'Incarnation of Morroc@NPC_WIDECURSE','attack',677,5,5000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@NPC_BLOODDRAIN','attack',199,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@NPC_WIDEBLEEDING','chase',665,1,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@NPC_WIDEBLEEDING','attack',665,1,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@SM_MAGNUM','attack',7,25,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1919,'Incarnation of Morroc@NPC_EARTHQUAKE','attack',653,1,5000,5000,100000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_CRITICALWOUND','attack',673,3,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_CRITICALWOUND','idle',673,3,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_CRITICALWOUND','chase',673,3,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_ENERGYDRAIN','chase',200,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1920,'Incarnation of Morroc@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,500,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_HEAL','chase',28,11,10000,500,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_HEAL','attack',28,11,10000,500,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@AL_HEAL','attack',28,11,10000,500,0,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@HP_ASSUMPTIO','idle',361,5,5000,1000,5000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@HP_ASSUMPTIO','chase',361,5,5000,1000,10000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1921,'Incarnation of Morroc@HP_ASSUMPTIO','attack',361,5,5000,1000,10000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_EMOTION','chase',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_PULSESTRIKE','chase',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@KN_TWOHANDQUICKEN','chase',60,30,10000,800,60000,'no','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@KN_TWOHANDQUICKEN','attack',60,30,10000,800,60000,'no','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_MAGICMIRROR','chase',671,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_GRANDDARKNESS','attack',339,5,500,2000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@NPC_EXPULSION','attack',674,1,100,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@SA_DISPELL','chase',289,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@SA_DISPELL','attack',289,5,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@WZ_VERMILION','chase',85,5,5000,700,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@WZ_VERMILION','attack',85,5,2000,700,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@WZ_JUPITEL','chase',84,5,5000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1922,'Incarnation of Morroc@WZ_JUPITEL','attack',84,5,3000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_EMOTION','chase',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_SUMMONSLAVE','chase',196,7,10000,1000,10000,'no','self','slavele','0',1937,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_SUMMONSLAVE','attack',196,7,10000,1000,10000,'no','self','slavele','0',1937,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_BLOODDRAIN','attack',199,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_WIDEBLEEDING','chase',665,2,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@NPC_EARTHQUAKE','attack',653,1,2000,700,100000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1923,'Incarnation of Morroc@SM_MAGNUM','attack',7,25,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_EMOTION','chase',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_CRITICALWOUND','attack',673,4,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_CRITICALWOUND','idle',673,4,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_CRITICALWOUND','chase',673,4,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_ENERGYDRAIN','chase',200,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_WIDESILENCE','chase',663,2,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1924,'Incarnation of Morroc@NPC_WIDESILENCE','attack',663,2,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@NPC_EMOTION','chase',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@HP_ASSUMPTIO','idle',361,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@HP_ASSUMPTIO','chase',361,5,5000,1000,10000,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@HP_ASSUMPTIO','attack',361,5,5000,1000,10000,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,300,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@AL_HEAL','chase',28,11,10000,300,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@AL_HEAL','attack',28,11,10000,300,0,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1925,'Incarnation of Morroc@AL_HEAL','idle',28,11,10000,300,0,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1927,'Whisper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1927,'Whisper@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1927,'Whisper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1927,'Whisper@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);

# Sealed Shrine (12.1)
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_TALK','chase',682,1,10000,0,6000000,'yes','target','myhpltmaxrate','2',NULL,NULL,NULL,NULL,NULL,NULL,'25');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_TALK','attack',682,1,10000,0,6000000,'yes','target','myhpltmaxrate','2',NULL,NULL,NULL,NULL,NULL,NULL,'25');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_TALK','chase',682,1,10000,0,6000000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'19');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_ALLHEAL','idle',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_ALLHEAL','walk',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@MO_BODYRELOCATION','chase',264,1,2000,200,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_DARKSTRIKE','chase',340,10,2000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@AS_SONICBLOW','attack',136,5,500,0,20000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'20');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_DARKNESSBREATH','attack',658,3,2000,800,5000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_EVILLAND','chase',670,10,10000,1000,50000,'no','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,'24');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_EVILLAND','attack',670,10,10000,1000,50000,'no','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,'24');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_HELLJUDGEMENT','chase',662,3,10000,800,15000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'24');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_HELLJUDGEMENT','attack',662,3,10000,800,15000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'22');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','2',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','2',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_INVINCIBLE','idle',685,1,10000,1000,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'19');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_INVINCIBLE','chase',685,1,10000,1000,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'19');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_INVINCIBLE','attack',685,1,10000,1000,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'19');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_PULSESTRIKE','chase',661,5,10000,2500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'20');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_PULSESTRIKE','attack',661,5,10000,2500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'21');
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@KN_BRANDISHSPEAR','chase',57,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@KN_BRANDISHSPEAR','attack',57,10,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_DARKBREATH','chase',202,1,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_DARKBREATH','attack',202,1,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@WZ_VERMILION','attack',85,21,2000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@WZ_VERMILION','chase',85,21,5000,500,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@WZ_VERMILION','chase',85,21,2000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1929,'Great Demon Baphomet@NPC_ALLHEAL','idle',687,1,10000,30000,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'23');

# Fallacious Okolnir (12.1)
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@WZ_METEOR','idle',83,11,10000,300,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_WIDECURSE','chase',677,5,10000,700,35000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_WIDECURSE','attack',677,5,10000,700,35000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@MG_THUNDERSTORM','attack',21,9,2000,1500,10000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@MG_THUNDERSTORM','chase',21,9,2000,1500,10000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_HELLJUDGEMENT','chase',662,5,5000,800,35000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_HELLJUDGEMENT','attack',662,5,5000,800,35000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_DARKNESSBREATH','chase',658,2,500,800,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_DARKNESSBREATH','attack',658,2,500,800,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@MG_FROSTDIVER','chase',15,40,3000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@MG_FROSTDIVER','attack',15,40,3000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@WZ_JUPITEL','chase',84,28,5000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@WZ_JUPITEL','attack',84,28,5000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_CRITICALWOUND','attack',673,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_CRITICALWOUND','chase',673,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_SUMMONSLAVE','attack',196,3,10000,2000,60000000,'no','self','myhpltmaxrate','70',1947,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1930,'Piamette@NPC_SUMMONSLAVE','chase',196,3,10000,2000,60000000,'no','self','myhpltmaxrate','70',1947,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@WZ_METEOR','idle',83,11,10000,300,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_SUMMONSLAVE','attack',196,5,10000,2000,5000,'no','self','slavele','1',1933,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_SUMMONSLAVE','idle',196,5,10000,2000,5000,'no','self','slavele','1',1933,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_SUMMONSLAVE','chase',196,5,10000,2000,5000,'no','self','slavele','1',1933,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_AGIUP','attack',350,5,10000,0,100000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_AGIUP','chase',350,5,10000,0,100000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@KN_TWOHANDQUICKEN','chase',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@LK_SPIRALPIERCE','attack',397,1,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1931,'Wish Maiden@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1932,'Garden Keeper@NPC_RUN','chase',354,1,10000,0,0,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1932,'Garden Keeper@NPC_RUN','attack',354,1,10000,0,0,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@MG_SAFETYWALL','attack',12,5,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@WZ_JUPITEL','attack',84,4,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@WZ_JUPITEL','chase',84,4,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_DARKBREATH','chase',202,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@NPC_DARKCROSS','attack',338,5,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@CR_REFLECTSHIELD','attack',252,3,10000,0,400000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1933,'Garden Watcher@CR_REFLECTSHIELD','chase',252,3,10000,0,400000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1937,'Constant@NPC_SELFDESTRUCTION','attack',173,1,10000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1937,'Constant@NPC_SELFDESTRUCTION','chase',173,1,2000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_WIDECURSE','chase',677,5,10000,700,35000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_WIDECURSE','attack',677,5,10000,700,35000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@MG_THUNDERSTORM','attack',21,9,2000,1500,10000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@MG_THUNDERSTORM','chase',21,9,2000,1500,10000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_HELLJUDGEMENT','chase',662,2,5000,800,35000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_HELLJUDGEMENT','attack',662,2,5000,800,35000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_DARKNESSBREATH','chase',658,2,500,800,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_DARKNESSBREATH','attack',658,2,500,800,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@MG_FROSTDIVER','chase',15,40,3000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@MG_FROSTDIVER','attack',15,40,3000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@WZ_JUPITEL','chase',84,28,5000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@WZ_JUPITEL','attack',84,28,5000,1000,20000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_DARKBLESSING','attack',203,1,300,0,2000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_CRITICALWOUND','attack',673,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_CRITICALWOUND','chase',673,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@AL_HEAL','chase',28,11,10000,500,5000,'no','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@AL_HEAL','attack',28,11,10000,500,5000,'no','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@HP_ASSUMPTIO','idle',361,5,5000,1000,5000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@HP_ASSUMPTIO','chase',361,5,5000,1000,10000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@HP_ASSUMPTIO','attack',361,5,5000,1000,10000,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_WIDESILENCE','chase',663,5,3000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1947,'Piamette@NPC_WIDESILENCE','attack',663,5,3000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);

# Endless Tower (12.1)
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@KN_BRANDISHSPEAR','attack',57,3,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_AGIUP','attack',350,5,10000,0,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_HELLJUDGEMENT','chase',662,3,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_HELLJUDGEMENT','attack',662,3,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_WIDEFREEZE','attack',664,5,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_DARKSTRIKE','chase',340,10,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_HALLUCINATION','chase',207,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_SHIELDBRAKE','attack',346,10,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_WATERATTACK','chase',184,1,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_WATERATTACK','attack',184,1,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_FIREATTACK','chase',186,1,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_FIREATTACK','attack',186,1,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_STUNATTACK','chase',179,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_STUNATTACK','attack',179,2,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@ASC_BREAKER','chase',379,3,5000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@ASC_BREAKER','attack',379,3,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@WZ_METEOR','attack',83,11,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@WZ_METEOR','chase',83,11,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@WZ_STORMGUST','attack',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@WZ_STORMGUST','chase',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_WIDESILENCE','chase',663,5,3000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_WIDESILENCE','attack',663,5,3000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@WZ_STORMGUST','chase',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_SUMMONSLAVE','attack',196,4,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_SUMMONSLAVE','idle',196,4,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_SUMMONSLAVE','idle',196,4,10000,0,30000,'yes','self','onspawn','0',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_ALLHEAL','idle',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1956,'Naght Sieger@NPC_ALLHEAL','walk',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_ALLHEAL','idle',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_ALLHEAL','walk',687,1,3000,10000,10000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@ASC_BREAKER','attack',379,5,500,1000,20000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_PULSESTRIKE','attack',661,5,10000,2500,30000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_EVILLAND','attack',670,4,10000,800,50000,'no','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_DARKSTRIKE','attack',340,10,2000,0,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_DARKNESSATTACK','attack',190,3,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@AC_SHOWER','attack',47,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','attack',83,11,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_WIDECURSE','attack',677,5,5000,300,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','21',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','28',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','79',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','skillused','83',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@WZ_METEOR','idle',83,11,10000,0,0,'yes','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@ML_DEFENDER','idle',8219,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@ML_DEFENDER','attack',8219,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1957,'Entweihen Crothen@NPC_ALLHEAL','idle',687,1,10000,30000,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1958,'Thorny Skeleton@SN_SHARPSHOOTING','attack',382,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1958,'Thorny Skeleton@AC_SHOWER','attack',47,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1958,'Thorny Skeleton@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1958,'Thorny Skeleton@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1959,'Thorn of Recovery@AL_HEAL','idle',28,11,10000,300,4000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1959,'Thorn of Recovery@AL_HEAL','attack',28,11,10000,300,2000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1959,'Thorn of Recovery@AL_HEAL','idle',28,11,10000,300,4000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1959,'Thorn of Recovery@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1959,'Thorn of Recovery@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_FROSTDIVER','attack',15,9,10000,500,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_LIGHTNINGBOLT','attack',20,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_FIREBOLT','attack',19,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_COLDBOLT','attack',14,7,10000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@MG_SOULSTRIKE','attack',13,9,5000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@WZ_HEAVENDRIVE','attack',91,3,3000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@NPC_DARKTHUNDER','attack',341,5,500,1500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@WZ_STORMGUST','attack',89,5,500,1500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1960,'Thorn of Magic@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@NPC_SLOWCAST','chase',672,5,10000,500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@NPC_SLOWCAST','attack',672,5,10000,500,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@SA_DISPELL','chase',289,5,500,2000,40000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@SA_DISPELL','attack',289,5,500,2000,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1961,'Thorn of Purification@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);

# Guild Dungeon SE (13.1)
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_VAMPIRE_GIFT','attack',679,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_EVILLAND','attack',670,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',NULL,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1974,'Banshee Master@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1868,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'52',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@WZ_JUPITEL','chase',84,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@NPC_MENTALBREAKER','attack',159,3,200,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@NPC_MENTALBREAKER','chase',159,3,200,800,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@NPC_PETRIFYATTACK','attack',180,3,200,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@NPC_PETRIFYATTACK','chase',180,3,200,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@SA_DISPELL','attack',289,1,100,1000,100000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1975,'Beholder Master@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1774,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@CR_REFLECTSHIELD','attack',252,2,1000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@NPC_RANDOMATTACK','attack',183,2,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@NPC_BARRIER','attack',204,1,500,3000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@NPC_SPLASHATTACK','attack',174,1,500,0,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@TF_HIDING','attack',51,1,500,500,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1976,'Cobalt Mineral@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@CR_AUTOGUARD','chase',249,5,500,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@RG_STRIPWEAPON','attack',215,3,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@RG_STRIPARMOR','attack',217,3,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1977,'Heavy Metaling@MC_MAMMONITE','attack',42,5,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@KN_BRANDISHSPEAR','attack',57,5,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@NPC_DARKTHUNDER','attack',341,4,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@NPC_SELFDESTRUCTION','attack',173,1,1000,2000,5000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1978,'Hell Apocalips@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1365,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@NPC_PIERCINGATT','attack',158,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@NPC_RANDOMATTACK','attack',183,5,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1979,'Zukadam@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1668,NULL,NULL,NULL,NULL,NULL,NULL);

# Memory of Orc Dungeon (13.1)
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@SM_BASH','attack',5,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@SM_BASH','chase',5,10,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@SM_MAGNUM','attack',7,7,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@CR_AUTOGUARD','chase',249,5,500,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@NPC_PULSESTRIKE','chase',661,2,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1981,'Safeguard Chief@NPC_PULSESTRIKE','attack',661,2,500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1982,'Orc Sniper@NPC_EMOTION','attack',197,1,100,0,60000,'yes','self','myhpltmaxrate','60',6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1982,'Orc Sniper@AC_SHOWER','attack',47,5,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1982,'Orc Sniper@AC_DOUBLE','attack',46,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1983,'Depraved Orc Spirit@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1983,'Depraved Orc Spirit@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1983,'Depraved Orc Spirit@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1984,'Shaman Cargalache@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1984,'Shaman Cargalache@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1984,'Shaman Cargalache@NPC_EVILLAND','attack',670,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# New World (13.1)
REPLACE INTO `mob_skill_db_re` VALUES (1985,'Dandelion@NPC_CHANGEUNDEAD','attack',348,1,3000,500,600000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1985,'Dandelion@NPC_CHANGEUNDEAD','chase',348,1,3000,500,600000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1985,'Dandelion@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@NPC_STUNATTACK','attack',179,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1986,'Tatacho@HW_GANBANTEIN','attack',483,1,2000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','around1','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@HW_GANBANTEIN','attack',483,1,3000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1987,'Centipede@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1988,'Nepenthes@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1988,'Nepenthes@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1988,'Nepenthes@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1988,'Nepenthes@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1988,'Nepenthes@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@SM_PROVOKE','chase',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@SM_PROVOKE','attack',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@NPC_CRITICALSLASH','chase',170,1,500,300,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@NPC_CRITICALSLASH','attack',170,1,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1989,'Hillsrion@NPC_COMBOATTACK','attack',171,3,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_SUMMONSLAVE','idle',196,3,10000,0,30000,'yes','self','slavele','0',1997,1997,1997,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_SUMMONSLAVE','chase',196,3,10000,0,30000,'yes','self','slavele','0',1997,1997,1997,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_SUMMONSLAVE','attack',196,3,10000,0,30000,'yes','self','slavele','0',1997,1997,1997,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','always','0',7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_EMOTION','attack',197,1,1000,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@WZ_WATERBALL','attack',86,10,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@LK_SPIRALPIERCE','attack',397,1,500,3000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_WIDESTUN','chase',678,1,500,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_WIDESTUN','attack',678,1,500,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@SM_PROVOKE','chase',6,10,500,600,120000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@SM_PROVOKE','attack',6,10,500,600,120000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@WZ_EARTHSPIKE','attack',90,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_DRAGONFEAR','attack',659,2,500,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_EARTHQUAKE','chase',653,2,10000,2000,30000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_EARTHQUAKE','attack',653,2,10000,2000,30000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@HW_GANBANTEIN','attack',483,1,3000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1990,'Hardrock Mammoth@NPC_ALLHEAL','idle',687,1,10000,30000,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@SM_PROVOKE','chase',6,10,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@SM_PROVOKE','attack',6,10,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','always','0',7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_EMOTION','attack',197,1,1000,0,10000,'yes','self','always','0',19,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@MO_BODYRELOCATION','chase',264,1,2000,200,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_WIDESLEEP','chase',668,1,500,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_WIDESLEEP','attack',668,1,500,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_WIDEBLEEDING','attack',665,1,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_WIDEBLEEDING','chase',665,1,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@WZ_EARTHSPIKE','attack',90,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_CRITICALSLASH','chase',170,1,500,300,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_CRITICALSLASH','attack',170,1,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_COMBOATTACK','attack',171,3,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@HW_GANBANTEIN','attack',483,1,3000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_DRAGONFEAR','attack',659,2,500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_GROUNDATTACK','attack',185,4,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@PF_SPIDERWEB','attack',405,1,3000,0,10000,'yes','target','always','0',NULL,0x38A5,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1991,'Tendrilion@NPC_ALLHEAL','idle',687,1,10000,30000,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@CR_REFLECTSHIELD','idle',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@CR_REFLECTSHIELD','attack',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@SM_PROVOKE','chase',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@SM_PROVOKE','attack',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_CRITICALSLASH','chase',170,1,500,300,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_CRITICALSLASH','attack',170,1,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_COMBOATTACK','attack',171,3,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_PULSESTRIKE','attack',661,3,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_ALLHEAL','idle',687,1,10000,1500,5000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_ALLHEAL','chase',687,1,10000,1500,5000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1992,'Cornus@NPC_ALLHEAL','attack',687,1,10000,1500,5000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@KN_PIERCE','attack',56,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@KN_BRANDISHSPEAR','attack',57,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@SM_PROVOKE','chase',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@SM_PROVOKE','attack',6,3,2000,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1993,'Naga@WZ_WATERBALL','attack',86,9,1000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@NPC_POISONATTACK','attack',188,2,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@NPC_PULSESTRIKE','attack',661,2,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1994,'Luciola Vespa@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1995,'Pinguicula@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@HP_ASSUMPTIO','idle',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@HP_ASSUMPTIO','chase',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@HP_ASSUMPTIO','attack',361,5,10000,500,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@NPC_SUMMONSLAVE','idle',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@NPC_SUMMONSLAVE','attack',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (1996,'Bacsojin@NPC_SUMMONSLAVE','chase',196,1,10000,0,5000,'yes','self','slavele','1',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@NPC_STUNATTACK','attack',179,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1997,'Tatacho@HW_GANBANTEIN','attack',483,1,2000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@SM_PROVOKE','chase',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@SM_PROVOKE','attack',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@NPC_CRITICALSLASH','chase',170,1,500,300,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@NPC_CRITICALSLASH','attack',170,1,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1998,'Hillsrion@NPC_COMBOATTACK','attack',171,3,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1999,'Centipede Larva@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','around1','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1999,'Centipede Larva@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1999,'Centipede Larva@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (1999,'Centipede Larva@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@AL_HEAL','idle',28,9,10000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_CALLSLAVE','attack',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_CALLSLAVE','chase',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_CALLSLAVE','idle',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_SUMMONSLAVE','attack',196,1,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_SUMMONSLAVE','chase',196,1,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_SUMMONSLAVE','idle',196,6,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_METAMORPHOSIS','chase',193,6,10000,5000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_METAMORPHOSIS','attack',193,6,10000,5000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_SHIELDBRAKE','attack',346,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@LK_SPIRALPIERCE','attack',397,5,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_PULSESTRIKE','attack',661,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_WIDESLEEP','attack',668,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_WIDESILENCE','attack',663,5,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_HELLJUDGEMENT','attack',662,5,10000,700,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_HELLJUDGEMENT','chase',662,5,10000,700,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_EXPULSION','attack',674,1,100,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_VAMPIRE_GIFT','attack',679,10,10000,1000,10000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2008,'Woomawang@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_RUN','attack',354,10,10000,0,0,'yes','self','myhpltmaxrate','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_RUN','chase',354,10,10000,0,0,'yes','self','myhpltmaxrate','1',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@AL_HEAL','idle',28,9,10000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_CALLSLAVE','attack',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_CALLSLAVE','chase',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_CALLSLAVE','idle',352,1,10000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_SUMMONSLAVE','attack',196,6,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_SUMMONSLAVE','chase',196,6,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_SUMMONSLAVE','idle',196,6,10000,0,10000,'yes','self','slavele','5',2010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_STONESKIN','chase',675,4,10000,1000,30000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_STONESKIN','attack',675,4,10000,1000,30000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_EVILLAND','attack',670,10,10000,0,15000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@SA_LANDPROTECTOR','attack',288,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_SHIELDBRAKE','attack',346,10,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_MAGICMIRROR','attack',671,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_EXPULSION','attack',674,1,50,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_WIDESTONE','attack',666,5,10000,500,15000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_WIDESTONE','chase',666,5,10000,500,15000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@LK_SPIRALPIERCE','attack',397,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_PULSESTRIKE','attack',661,5,10000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_HELLJUDGEMENT','chase',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_HELLJUDGEMENT','attack',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_ACIDBREATH','attack',657,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_ACIDBREATH','chase',657,1,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_VAMPIRE_GIFT','attack',679,1,1000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2009,'Woomawang@NPC_WIDESOULDRAIN','attack',680,7,1000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@NPC_SUMMONSLAVE','idle',196,1,10000,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2010,'Majoruros@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);

# Into the Unknown (13.2)
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@TF_THROWSTONE','chase',152,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@NPC_PIERCINGATT','attack',158,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2013,'Draco@NPC_FIREBREATH','attack',654,3,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2014,'Draco Egg@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2014,'Draco Egg@NPC_METAMORPHOSIS','idle',193,1,10,2000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_BLOODDRAIN','attack',199,1,5000,500,5000,'yes','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2015,'Dark Pinguicula@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@MG_FROSTDIVER','chase',15,9,10000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@WZ_WATERBALL','attack',86,4,2000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@MG_COLDBOLT','attack',14,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@NPC_SLOWCAST','attack',672,2,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2016,'Aqua Elemental@WZ_FROSTNOVA','attack',88,10,10000,1500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@TF_THROWSTONE','chase',152,1,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@MO_BODYRELOCATION','chase',264,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_DARKSTRIKE','chase',340,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_DARKSTRIKE','attack',340,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_ARMORBRAKE','attack',344,3,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@WZ_EARTHSPIKE','attack',90,5,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@SM_MAGNUM','attack',7,25,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2017,'Rata@ASC_METEORASSAULT','attack',406,2,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_DARKSTRIKE','chase',340,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_DARKSTRIKE','attack',340,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@PF_SPIDERWEB','chase',405,1,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@KN_TWOHANDQUICKEN','attack',60,5,500,0,300000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@MG_THUNDERSTORM','attack',21,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@MG_THUNDERSTORM','chase',21,5,1000,1500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@AL_HEAL','attack',28,5,5000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2018,'Duneyrr@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2019,'Ancient Tree@AL_HEAL','idle',28,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2019,'Ancient Tree@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2019,'Ancient Tree@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2019,'Ancient Tree@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2024,'Bradium Golem@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2020,'Rhyncho@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2020,'Rhyncho@NPC_STUNATTACK','attack',179,5,500,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2020,'Rhyncho@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2020,'Rhyncho@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2020,'Rhyncho@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2021,'Phylla@PR_LEXDIVINA','chase',76,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2021,'Phylla@MG_COLDBOLT','attack',14,3,3000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2021,'Phylla@NPC_WATERATTACK','attack',184,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2021,'Phylla@WZ_WATERBALL','attack',86,3,2000,100,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_ALLHEAL','idle',687,1,3000,10000,10000,'no','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_ALLHEAL','walk',687,1,3000,10000,10000,'no','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2027,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2027,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','onspawn','0',2027,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_MAGICMIRROR','idle',671,3,10000,0,120000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_MAGICMIRROR','chase',671,3,10000,0,120000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_MAGICMIRROR','attack',671,3,10000,0,120000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_STONESKIN','chase',675,3,10000,2500,300000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,'34');
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_STONESKIN','attack',675,3,10000,2500,300000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,'34');
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_POWERUP','attack',349,5,6000,0,60000,'yes','self','myhpltmaxrate','15',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_EVILLAND','attack',670,4,500,0,25000,'yes','target','myhpltmaxrate','15',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@WZ_QUAGMIRE','chase',92,5,10000,700,5000,'no','target','myhpltmaxrate','28',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@WZ_QUAGMIRE','attack',92,5,10000,700,120000,'no','target','myhpltmaxrate','28',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_EARTHQUAKE','chase',653,2,10000,10000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_EARTHQUAKE','attack',653,2,10000,10000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_INVISIBLE','idle',353,1,2000,200,5000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_INVISIBLE','chase',353,1,2000,200,5000,'no','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDESTONE','chase',666,5,10000,0,60000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDESTONE','attack',666,5,10000,0,60000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_EVILLAND','idle',670,10,500,0,120000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_EVILLAND','chase',670,10,500,0,120000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_HELLJUDGEMENT','chase',662,2,10000,7000,25000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'32');
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_HELLJUDGEMENT','attack',662,2,10000,7000,25000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'32');
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_DARKNESSBREATH','attack',658,10,10000,800,35000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_SLOWCAST','attack',672,5,10000,0,60000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDECURSE','chase',677,5,10000,1000,25000,'no','self','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDECURSE','attack',677,5,10000,1000,25000,'no','self','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_FIREBREATH','attack',654,5,10000,800,15000,'no','target','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDESLEEP','chase',668,5,10000,3000,20000,'no','self','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_WIDESLEEP','attack',668,5,30000,1000,20000,'no','self','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@MG_FIREBALL','chase',17,5,10000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@WZ_METEOR','attack',83,11,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_PULSESTRIKE','attack',661,5,2000,10000,40000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'31');
REPLACE INTO `mob_skill_db_re` VALUES (2022,'Nidhoggr\'s Shadow@NPC_ALLHEAL','idle',687,1,10000,120000,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2023,'Dark Shadow@NPC_DARKSTRIKE','chase',340,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2023,'Dark Shadow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2023,'Dark Shadow@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2023,'Dark Shadow@NPC_DARKNESSBREATH','attack',658,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2026,'Dandelion@NPC_BLINDATTACK','attack',177,2,1000,0,6000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'29');
REPLACE INTO `mob_skill_db_re` VALUES (2026,'Dandelion@NPC_POISON','attack',176,3,1000,500,60000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2026,'Dandelion@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'27');
REPLACE INTO `mob_skill_db_re` VALUES (2026,'Dandelion@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'28');

REPLACE INTO `mob_skill_db_re` VALUES (2027,'Dark Shadow@NPC_DARKSTRIKE','chase',340,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2027,'Dark Shadow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2027,'Dark Shadow@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2027,'Dark Shadow@NPC_DARKNESSBREATH','attack',658,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

#****
# NC Mechanic Summons
REPLACE INTO `mob_skill_db_re` VALUES (2042,'Silver Sniper@AL_HEAL','chase',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2042,'Silver Sniper@AL_HEAL','idle',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2042,'Silver Sniper@AL_HEAL','attack',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2042,'Silver Sniper@NPC_REVENGE','idle',333,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2043,'Magic Decoy Fire@MG_FIREBOLT','attack',19,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2043,'Magic Decoy Fire@MG_FIREBOLT','chase',19,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2043,'Magic Decoy Fire@AL_HEAL','chase',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2043,'Magic Decoy Fire@AL_HEAL','idle',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2043,'Magic Decoy Fire@AL_HEAL','attack',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2044,'Magic Decoy Water@MG_COLDBOLT','attack',14,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2044,'Magic Decoy Water@MG_COLDBOLT','chase',14,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2044,'Magic Decoy Water@AL_HEAL','chase',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2044,'Magic Decoy Water@AL_HEAL','idle',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2044,'Magic Decoy Water@AL_HEAL','attack',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2045,'Magic Decoy Earth@WZ_EARTHSPIKE','attack',90,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2045,'Magic Decoy Earth@WZ_EARTHSPIKE','chase',90,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2045,'Magic Decoy Earth@AL_HEAL','chase',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2045,'Magic Decoy Earth@AL_HEAL','idle',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2045,'Magic Decoy Earth@AL_HEAL','attack',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2046,'Magic Decoy Wind@MG_LIGHTNINGBOLT','attack',20,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2046,'Magic Decoy Wind@MG_LIGHTNINGBOLT','chase',20,10,10000,800,3500,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2046,'Magic Decoy Wind@AL_HEAL','chase',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2046,'Magic Decoy Wind@AL_HEAL','idle',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2046,'Magic Decoy Wind@AL_HEAL','attack',28,10,10000,0,1000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

#REPLACE INTO `mob_skill_db_re` VALUES (2053,'Dimik@NPC_ATTRICHANGE','idle',161,1,500,2000,50000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'10',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2053,'Dimik@NPC_EMOTION_ON','idle',474,1,100,0,60000,'yes','self','always','0',20,0x81,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2053,'Dimik@NPC_EMOTION','idle',197,1,100,0,60000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2053,'Dimik@NPC_EMOTION','attack',197,1,10000,0,60000,'yes','self','always','0',6,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2053,'Dimik@NPC_EMOTION_ON','idle',474,1,100,0,60000,'yes','self','always','0',22,0xC3,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_BLOODDRAIN','attack',199,1,5000,500,5000,'yes','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2048,'Dark Pinguicula@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@MG_FROSTDIVER','chase',15,9,10000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@WZ_WATERBALL','attack',86,4,2000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@MG_COLDBOLT','attack',14,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@NPC_SLOWCAST','attack',672,2,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2050,'Aqua Elemental@WZ_FROSTNOVA','attack',88,10,10000,1500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2049,'Bradium Golem@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2047,'Naga@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2047,'Naga@KN_BRANDISHSPEAR','attack',57,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2047,'Naga@SM_PROVOKE','chase',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2047,'Naga@SM_PROVOKE','attack',6,3,500,600,5000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2047,'Naga@HW_GANBANTEIN','attack',483,1,2000,0,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@MG_FIREWALL','chase',18,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_HELLJUDGEMENT','chase',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_HELLJUDGEMENT','attack',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_DARKBLESSING','attack',203,1,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_HELMBRAKE','attack',345,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_SUMMONSLAVE','attack',196,2,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@NPC_SUMMONSLAVE','idle',196,2,10000,700,10000,'no','self','slavele','3',1605,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@WZ_METEOR','attack',83,11,5000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@WZ_METEOR','chase',83,11,5000,500,2000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2052,'Dark Lord@WZ_METEOR','chase',83,11,5000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Brasilis
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@MG_FIREBALL','idle',17,5,10000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@MG_FIREBALL','chase',17,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@CR_REFLECTSHIELD','idle',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_SUMMONSLAVE','attack',196,5,10000,1000,30000,'no','self','slavele','1',2082,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_SUMMONSLAVE','idle',196,5,10000,1000,30000,'no','self','slavele','1',2082,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_SUMMONSLAVE','chase',196,5,10000,1000,30000,'no','self','slavele','1',2082,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2068,'Boitata@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@NPC_WATERATTACK','attack',184,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@PR_LEXDIVINA','chase',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2069,'Iara@MG_FROSTDIVER','chase',15,40,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_WATERATTACK','attack',184,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_CRITICALWOUND','attack',673,1,50,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@MG_COLDBOLT','attack',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@MG_COLDBOLT','chase',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2070,'Piranha@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@NPC_CURSEATTACK','attack',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@MG_FIREBOLT','attack',19,5,2000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@MG_FIREBOLT','chase',19,5,2000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2071,'Headless Mule@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@NPC_COMBOATTACK','attack',171,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@AS_SONICBLOW','attack',136,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2072,'Jaguar@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2073,'Toucan@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@MG_STONECURSE','idle',16,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@MG_STONECURSE','attack',16,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2074,'Curupira@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2080,'Crystal@AL_TELEPORT','attack',26,1,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2080,'Crystal@AL_TELEPORT','idle',26,1,4000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_WATERATTACK','attack',184,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_CRITICALWOUND','attack',673,1,50,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@MG_COLDBOLT','attack',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@MG_COLDBOLT','chase',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2082,'Piranha@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# El Dicastes (13.3)
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_COMBOATTACK','chase',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2083,'Scaraba@NPC_EMOTION','idle',197,1,3000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@SM_BASH','attack',5,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_STOP','attack',342,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2084,'Scaraba@NPC_EMOTION','idle',197,1,3000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2085,'Antler Scaraba@PF_SPIDERWEB','chase',405,1,300,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@PF_SPIDERWEB','chase',405,1,300,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2086,'Rake Scaraba@NPC_EMOTION','idle',197,1,3000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_HEAL','attack',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_HEAL','attack',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_HEAL','chase',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_HEAL','chase',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@AL_TELEPORT','walk',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@CR_AUTOGUARD','attack',249,10,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2145,2144,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2145,2144,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@WZ_HEAVENDRIVE','attack',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@WZ_HEAVENDRIVE','chase',91,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2087,'Queen Scaraba@WZ_HEAVENDRIVE','chase',91,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2088,'Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2083,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2089,'Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2084,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2090,'Antler Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2085,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2091,'Rake Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2086,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@PF_SPIDERWEB','idle',405,1,3000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@PF_SPIDERWEB','attack',405,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@AS_VENOMDUST','angry',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2092,'Dolomedes@NPC_BLEEDING','attack',660,2,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@AL_DECAGI','chase',30,48,2000,0,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_ARMORBRAKE','attack',344,10,2000,500,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_ARMORBRAKE','chase',344,10,2000,500,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@LK_SPIRALPIERCE','attack',397,5,1500,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@MG_THUNDERSTORM','attack',21,20,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@MG_THUNDERSTORM','chase',21,20,10000,1500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@CR_AUTOGUARD','attack',249,10,2000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_SUMMONSLAVE','attack',196,8,10000,700,10000,'no','self','slavele','3',1439,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@NPC_SUMMONSLAVE','idle',196,8,10000,700,10000,'no','self','slavele','3',1439,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2094,'Orc Hero@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@MG_FIREBALL','chase',17,44,4000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@MG_FIREBALL','chase',17,44,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_BLEEDING','attack',660,5,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AM_DEMONSTRATION','attack',229,5,2000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AM_DEMONSTRATION','chase',229,5,2000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_SUMMONSLAVE','attack',196,8,10000,700,10000,'no','self','slavele','3',1603,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_SUMMONSLAVE','idle',196,8,10000,700,10000,'no','self','slavele','3',1603,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',1060,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@SM_MAGNUM','attack',7,25,3000,0,2000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2095,'Eddga@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_POWERUP','attack',349,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@AS_VENOMDUST','attack',140,5,2000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@AS_VENOMDUST','attack',140,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@MG_STONECURSE','attack',16,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@ASC_METEORASSAULT','attack',406,10,10000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_CURSEATTACK','attack',181,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_CURSEATTACK','chase',181,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_DARKBREATH','attack',202,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_DARKBREATH','chase',202,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_POISONATTACK','attack',188,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1522,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1522,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_SUMMONSLAVE','idle',196,7,10000,0,0,'no','self','onspawn','0',1029,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_SUMMONSLAVE','idle',196,7,10000,0,0,'no','self','onspawn','0',1029,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@SM_BASH','attack',5,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_DARKTHUNDER','chase',341,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@NPC_DARKTHUNDER','chase',341,10,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2096,'Osiris@WZ_QUAGMIRE','chase',92,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@AL_DECAGI','chase',30,48,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_HELLJUDGEMENT','chase',662,10,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_HELLJUDGEMENT','attack',662,10,10000,500,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_BLOODDRAIN','attack',199,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_ENERGYDRAIN','chase',200,1,3000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@MO_BODYRELOCATION','chase',264,1,2000,200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_CHANGEUNDEAD','attack',348,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_INVISIBLE','attack',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_INVISIBLE','idle',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_INVISIBLE','chase',353,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_SUMMONSLAVE','attack',196,16,10000,700,5000,'no','self','slavele','5',1419,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_SUMMONSLAVE','idle',196,16,10000,700,5000,'no','self','slavele','5',1419,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2097,'Dracula@NPC_ENERGYDRAIN','chase',200,1,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_HEAL','attack',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_HEAL','chase',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@LK_SPIRALPIERCE','attack',397,5,10000,1000,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@CR_AUTOGUARD','attack',249,10,2000,500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@CR_AUTOGUARD','chase',249,10,2000,500,300000,'no','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_DECAGI','attack',30,48,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@AL_DECAGI','chase',30,48,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',1427,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@NPC_SUMMONSLAVE','idle',196,1,10000,700,10000,'no','self','slavele','3',1427,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2098,'Doppelganger@BS_HAMMERFALL','chase',110,10,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_HEAL','attack',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_HEAL','chase',28,11,10000,0,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_PNEUMA','attack',25,1,2000,0,5000,'yes','target','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_PNEUMA','chase',25,1,2000,0,5000,'yes','target','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SILENCEATTACK','attack',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SILENCEATTACK','chase',178,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SILENCEATTACK','chase',178,5,10000,0,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1604,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1604,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',1156,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@NPC_STOP','attack',342,1,2000,0,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@WZ_JUPITEL','attack',84,28,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2099,'Mistress@WZ_JUPITEL','chase',84,28,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@MO_BODYRELOCATION','chase',264,1,2000,200,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_DARKSTRIKE','chase',340,10,2000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_EARTHQUAKE','chase',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_EARTHQUAKE','attack',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_HELLJUDGEMENT','chase',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_HELLJUDGEMENT','attack',662,5,10000,800,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_DARKBREATH','attack',202,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1431,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',1101,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@WZ_VERMILION','chase',85,21,5000,500,2000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2100,'Baphomet@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_HELLJUDGEMENT','chase',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_HELLJUDGEMENT','attack',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_DARKSTRIKE','chase',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_CHANGEUNDEAD','attack',348,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_GRANDDARKNESS','attack',339,10,1000,700,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_STUNATTACK','attack',179,5,2000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1490,1490,1509,1508,1179,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1490,1490,1509,1508,1179,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2101,'Lord of Death@NPC_DARKSTRIKE','chase',340,10,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_WATERATTACK','attack',184,10,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_RANDOMATTACK','attack',183,8,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_SUMMONSLAVE','attack',196,6,10000,700,30000,'no','self','slavele','3',1788,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_SUMMONSLAVE','idle',196,6,10000,700,30000,'no','self','slavele','3',1788,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_WIDEFREEZE','chase',664,5,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_WIDEFREEZE','attack',664,5,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@SA_DISPELL','chase',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@MG_FROSTDIVER','idle',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@MG_FROSTDIVER','chase',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_STORMGUST','chase',89,10,5000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_STORMGUST','attack',89,10,5000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_WATERBALL','chase',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_WATERBALL','attack',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_FROSTNOVA','chase',88,10,5000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@WZ_FROSTNOVA','attack',88,10,5000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@NPC_AGIUP','attack',350,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2103,'Ktullanux@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@MG_FROSTDIVER','attack',15,40,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@MG_FROSTDIVER','chase',15,40,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_SHIELDBRAKE','attack',346,10,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_DARKBREATH','attack',202,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_STOP','attack',342,1,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','2',1531,1531,1564,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','2',1531,1531,1564,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@SM_MAGNUM','attack',7,25,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2104,'Dark Snake Lord@MG_FROSTDIVER','chase',15,40,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@AS_SONICBLOW','attack',136,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@BS_MAXIMIZE','attack',114,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@BS_MAXIMIZE','chase',114,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@WZ_WATERBALL','chase',86,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_WEAPONBRAKER','attack',343,10,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_STUNATTACK','attack',179,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',1364,1594,1600,1601,1602,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',1364,1594,1600,1601,1602,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2105,'Turtle General@WZ_WATERBALL','attack',86,5,2000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_SUMMONSLAVE','attack',196,3,10000,2000,5000,'no','self','slavele','15',1669,1675,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_CALLSLAVE','attack',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_CALLSLAVE','idle',352,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_POWERUP','attack',349,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_CRITICALSLASH','attack',170,1,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@WZ_METEOR','attack',83,11,500,10000,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_EMOTION','idle',197,1,1000,0,0,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2106,'Vesper@NPC_SUMMONSLAVE','idle',196,2,10000,0,0,'no','self','onspawn','0',1365,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1868,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1868,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_WIDECURSE','attack',677,5,2000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_DARKSTRIKE','chase',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_DARKSTRIKE','attack',340,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_DARKCROSS','attack',338,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@CR_HOLYCROSS','attack',253,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_GRANDDARKNESS','chase',339,10,500,1300,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_GRANDDARKNESS','attack',339,10,500,1300,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@CR_GRANDCROSS','chase',254,10,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@CR_GRANDCROSS','attack',254,10,500,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_VAMPIRE_GIFT','attack',679,2,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_SLOWCAST','chase',672,5,2000,300,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_SLOWCAST','attack',672,5,2000,800,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_EVILLAND','attack',670,10,10000,1000,600000,'no','target','myhpltmaxrate','44',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_EVILLAND','attack',670,7,500,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_MAGICMIRROR','attack',671,1,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_HELLJUDGEMENT','chase',662,4,10000,800,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_HELLJUDGEMENT','attack',662,4,10000,800,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_WIDESILENCE','chase',663,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_WIDESILENCE','attack',663,5,5000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_DARKNESSATTACK','attack',190,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2107,'Fallen Bishop@NPC_DARKNESSBREATH','attack',658,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_FIREATTACK','attack',186,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_TELEKINESISATTACK','attack',191,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@WZ_FIREPILLAR','idle',80,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@AM_DEMONSTRATION','attack',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@KN_PIERCE','attack',56,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_POWERUP','attack',349,5,10000,700,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_POWERUP','attack',349,5,2000,700,50000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_FIREBREATH','chase',654,10,2000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@WZ_SIGHTRASHER','attack',81,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@MG_FIREBALL','attack',17,44,1000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@MG_FIREBALL','chase',17,44,10000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_SUMMONSLAVE','attack',196,4,10000,700,30000,'no','self','slavele','3',1787,1787,1786,1786,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@NPC_SUMMONSLAVE','idle',196,4,10000,700,30000,'no','self','slavele','3',1787,1787,1786,1786,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2108,'Gloom Under Night@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_SUMMONSLAVE','attack',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_SUMMONSLAVE','idle',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_SUMMONSLAVE','chase',196,3,10000,2000,5000,'no','self','slavele','1',1891,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_SUMMONSLAVE','idle',196,3,10000,0,0,'no','self','onspawn','0',1765,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@HP_ASSUMPTIO','chase',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@HP_ASSUMPTIO','attack',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@HW_GANBANTEIN','attack',483,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_EARTHQUAKE','chase',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2109,'Valkyrie Randgris@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@AL_HEAL','idle',28,1,10000,0,10000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_SUMMONSLAVE','attack',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_SUMMONSLAVE','idle',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_SUMMONSLAVE','chase',196,2,10000,1000,30000,'no','self','slavele','1',1834,1835,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@AS_SONICBLOW','attack',136,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_FIREBREATH','chase',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_FIREBREATH','attack',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@SA_LANDPROTECTOR','attack',288,3,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@WZ_METEOR','attack',83,11,2000,500,5000,'no','target','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@KN_TWOHANDQUICKEN','attack',60,9,10000,0,100000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_EARTHQUAKE','chase',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_EARTHQUAKE','attack',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_PULSESTRIKE','attack',661,5,10000,0,15000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_DRAGONFEAR','attack',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@NPC_DRAGONFEAR','chase',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2110,'Ifrit@WZ_METEOR','idle',83,9,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1641,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@MC_MAMMONITE','attack',42,22,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@BS_HAMMERFALL','chase',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@BS_HAMMERFALL','attack',110,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_SPLASHATTACK','attack',174,1,1000,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_HELMBRAKE','attack',345,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_SHIELDBRAKE','attack',346,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@ST_FULLSTRIP','attack',476,5,1500,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_WATERATTACK','attack',184,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CHANGEGROUND','attack',163,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2111,'Whitesmith Harword@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1641,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@LK_SPIRALPIERCE','chase',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@LK_SPIRALPIERCE','attack',397,5,2000,0,5000,'yes','target','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_BASH','attack',5,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_MAGNUM','attack',7,25,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_PROVOKE','attack',6,10,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_PROVOKE','chase',6,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_FIREATTACK','attack',186,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CHANGEFIRE','attack',164,1,10000,500,60000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CHANGEHOLY','attack',167,1,10000,500,60000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2112,'Lord Knight Seyren@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,60000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',1640,1642,1643,1644,1645,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_INVISIBLE','idle',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_INVISIBLE','attack',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_GUIDEDATTACK','attack',172,10,2000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@TF_POISON','attack',52,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_SONICBLOW','attack',136,10,3000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_VENOMDUST','idle',140,1,2000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@AS_VENOMDUST','attack',140,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@ASC_BREAKER','chase',379,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@ASC_METEORASSAULT','attack',406,10,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_POISONATTACK','attack',188,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CHANGEWIND','attack',165,1,10000,500,6000000,'no','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CHANGEHOLY','attack',167,1,10000,500,6000000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2113,'Assassin Cross Eremes@NPC_CHANGETELEKINESIS','attack',169,1,10000,500,6000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);

# Bifrost (14.1)
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_ALLHEAL','idle',687,1,3000,20000,20000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_ALLHEAL','walk',687,1,3000,20000,20000,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_MAGICMIRROR','chase',671,1,10000,0,120000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_MAGICMIRROR','attack',671,1,10000,0,120000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_STONESKIN','chase',675,1,10000,2500,300000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,'34');
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_STONESKIN','attack',675,1,10000,2500,300000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,'34');
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_POWERUP','attack',349,1,6000,0,60000,'yes','self','myhpltmaxrate','15',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_EVILLAND','attack',670,2,500,0,25000,'yes','target','myhpltmaxrate','15',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@WZ_QUAGMIRE','chase',92,2,10000,700,5000,'no','target','myhpltmaxrate','28',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@WZ_QUAGMIRE','attack',92,2,10000,700,120000,'no','target','myhpltmaxrate','28',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_EARTHQUAKE','chase',653,1,10000,10000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_EARTHQUAKE','attack',653,1,10000,10000,180000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDESTONE','chase',666,1,10000,0,60000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDESTONE','attack',666,1,10000,0,60000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_EVILLAND','idle',670,5,500,0,120000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_EVILLAND','chase',670,5,500,0,120000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_HELLJUDGEMENT','chase',662,1,10000,7000,25000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'32');
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_HELLJUDGEMENT','attack',662,1,10000,7000,25000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'32');
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_DARKNESSBREATH','attack',658,3,10000,800,35000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_SLOWCAST','attack',672,1,10000,0,60000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_CRITICALWOUND','attack',673,1,2000,0,30000,'yes','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDECURSE','chase',677,1,10000,1000,25000,'no','self','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDECURSE','attack',677,1,10000,1000,25000,'no','self','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_FIREBREATH','attack',654,1,10000,800,15000,'no','target','myhpltmaxrate','75',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDESLEEP','chase',668,1,10000,3000,20000,'no','self','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_WIDESLEEP','attack',668,1,30000,1000,20000,'no','self','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','myhpltmaxrate','95',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@MG_FIREBALL','chase',17,1,10000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_DRAGONFEAR','chase',659,1,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_DRAGONFEAR','attack',659,1,2000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@WZ_METEOR','attack',83,3,2000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@SA_DISPELL','attack',289,1,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_PULSESTRIKE','attack',661,1,2000,10000,40000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'31');
REPLACE INTO `mob_skill_db_re` VALUES (2131,'Lost Dragon@NPC_ALLHEAL','idle',687,1,10000,120000,30000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@NPC_POISON','attack',176,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@PF_SPIDERWEB','idle',405,1,3000,0,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@PF_SPIDERWEB','attack',405,1,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@AS_VENOMDUST','angry',140,1,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2132,'Pom Spider@NPC_BLEEDING','attack',660,2,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@CR_AUTOGUARD','attack',249,2,700,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2133,'Angra Mantis@NPC_SUMMONSLAVE','idle',196,1,10000,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@NPC_GROUNDATTACK','attack',185,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@WZ_EARTHSPIKE','attack',90,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@WZ_EARTHSPIKE','attack',90,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@WZ_EARTHSPIKE','attack',90,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@SM_PROVOKE','angry',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@SM_PROVOKE','follow',6,10,2000,600,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2134,'Parus@WZ_HEAVENDRIVE','attack',91,2,500,500,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@MG_LIGHTNINGBOLT','chase',20,5,10000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2136,'Little Fatum@MG_THUNDERSTORM','attack',21,3,2000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_COMBOATTACK','attack',171,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_CRITICALWOUND','attack',673,1,50,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@MG_COLDBOLT','attack',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@MG_COLDBOLT','chase',14,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2137,'Miming@NPC_SILENCEATTACK','chase',178,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2138,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'35');
REPLACE INTO `mob_skill_db_re` VALUES (2138,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'35');
REPLACE INTO `mob_skill_db_re` VALUES (2138,'Nydhogg\'s Memory@NPC_SUMMONSLAVE','idle',196,1,10000,0,0,'no','self','onspawn','0',2139,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2139,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'36');
REPLACE INTO `mob_skill_db_re` VALUES (2139,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'36');
REPLACE INTO `mob_skill_db_re` VALUES (2140,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'37');
REPLACE INTO `mob_skill_db_re` VALUES (2140,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'37');
REPLACE INTO `mob_skill_db_re` VALUES (2140,'Nydhogg\'s Memory@NPC_SUMMONSLAVE','idle',196,1,10000,0,0,'no','self','onspawn','0',2141,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2141,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'38');
REPLACE INTO `mob_skill_db_re` VALUES (2141,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'38');
REPLACE INTO `mob_skill_db_re` VALUES (2142,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'39');
REPLACE INTO `mob_skill_db_re` VALUES (2142,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'39');
REPLACE INTO `mob_skill_db_re` VALUES (2142,'Nydhogg\'s Memory@NPC_SUMMONSLAVE','idle',196,1,10000,0,0,'no','self','onspawn','0',2143,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2143,'Nydhogg\'s Memory@NPC_TALK','idle',682,10,2500,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'40');
REPLACE INTO `mob_skill_db_re` VALUES (2143,'Nydhogg\'s Memory@NPC_TALK','attack',682,10,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,'40');

# Queen Scaraba Slaves
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2144,'Antler Scaraba@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2145,'Rake Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@WZ_QUAGMIRE','chase',92,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@WZ_QUAGMIRE','attack',92,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_WIDESTONE','chase',666,2,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_WIDESTONE','attack',666,5,10000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_EVILLAND','chase',670,2,500,0,120000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_HELLJUDGEMENT','chase',662,2,10000,7000,25000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_HELLJUDGEMENT','attack',662,1,500,7000,25000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_DARKNESSBREATH','attack',658,1,500,800,35000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_SLOWCAST','attack',672,1,200,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_CRITICALWOUND','attack',673,1,2000,0,30000,'yes','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_FIREBREATH','attack',654,1,1000,800,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_WIDESLEEP','chase',668,1,200,3000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_WIDESLEEP','attack',668,1,200,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@MG_FIREBALL','chase',17,5,3000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_DRAGONFEAR','chase',659,2,1000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_DRAGONFEAR','attack',659,2,2000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@SA_DISPELL','attack',289,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2146,'G_S_NYDHOG@NPC_PULSESTRIKE','attack',661,1,500,10000,40000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Dewata
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@MG_SOULSTRIKE','chase',13,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@NPC_POISONATTACK','chase',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2151,'Alnoldi@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@AL_HEAL','attack',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@AS_VENOMDUST','chase',140,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@AS_VENOMDUST','attack',140,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@NPC_POISON','attack',176,3,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@NPC_POISONATTACK','chase',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2152,'Comodo@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_AGIUP','chase',350,5,2000,0,120000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_AGIUP','attack',350,5,2000,0,120000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_ALLHEAL','idle',687,1,10000,10000,5000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_ALLHEAL','chase',687,1,10000,10000,5000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_ALLHEAL','attack',687,1,10000,10000,5000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_HELLJUDGEMENT','chase',662,3,10000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_HELLJUDGEMENT','attack',662,3,10000,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@SA_LANDPROTECTOR','attack',288,1,10000,500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@AL_PNEUMA','chase',25,1,1000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@LK_SPIRALPIERCE','chase',397,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@LK_SPIRALPIERCE','attack',397,5,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_PULSESTRIKE','attack',661,5,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_SHIELDBRAKE','attack',346,5,1000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2153,'Cendrawasih@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2154,'Banaspaty@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2154,'Banaspaty@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2154,'Banaspaty@MG_FIREBALL','chase',17,9,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2154,'Banaspaty@WZ_SIGHTRASHER','attack',81,7,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2154,'Banaspaty@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@WZ_FIREPILLAR','idle',80,5,10000,0,30000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@NPC_SLEEPATTACK','chase',182,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@NPC_SLEEPATTACK','attack',182,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@NPC_SILENCEATTACK','chase',178,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@NPC_SILENCEATTACK','attack',178,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2155,'Butoijo@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@AL_HEAL','idle',28,11,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@AL_HEAL','chase',28,11,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@AL_HEAL','attack',28,11,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@RG_STRIPWEAPON','attack',215,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_SLOWCAST','attack',672,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_PULSESTRIKE','attack',661,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@SA_LANDPROTECTOR','attack',288,1,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_HELLJUDGEMENT','chase',662,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_HELLJUDGEMENT','attack',662,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_DRAGONFEAR','chase',659,5,10000,0,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_DRAGONFEAR','attack',659,5,10000,0,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDECURSE','chase',677,5,10000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDECURSE','attack',677,5,10000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESILENCE','chase',663,5,10000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESILENCE','attack',663,5,10000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDEBLEEDING','chase',665,5,5000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDEBLEEDING','attack',665,5,5000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESTUN','chase',678,5,2000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESTUN','attack',678,5,2000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESLEEP','chase',668,5,2000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_WIDESLEEP','attack',668,5,2000,500,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_SUMMONSLAVE','chase',196,5,10000,700,10000,'no','self','slavele','3',2157,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2157,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2157,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2156,'Leak@NPC_SUMMONSLAVE','idle',196,5,10000,0,0,'no','self','onspawn','0',2154,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2157,'Banaspaty@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2157,'Banaspaty@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2157,'Banaspaty@MG_FIREBALL','chase',17,9,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2157,'Banaspaty@WZ_SIGHTRASHER','attack',81,7,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2157,'Banaspaty@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Scaraba Hole Nightmare Mode
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@RG_STRIPHELM','attack',218,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_HELMBRAKE','attack',345,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_BLEEDING','attack',660,2,500,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_COMBOATTACK','attack',171,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@KN_TWOHANDQUICKEN','attack',60,30,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','chase',91,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','attack',91,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','chase',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','attack',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2161,'Gold One-Horn Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@RG_STRIPSHIELD','attack',216,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_SHIELDBRAKE','attack',346,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_BLEEDING','attack',660,2,500,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@SM_BASH','attack',5,20,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_COMBOATTACK','attack',171,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@KN_TWOHANDQUICKEN','attack',60,30,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','chase',21,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','attack',21,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','chase',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','attack',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2162,'Gold Two-Horn Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@RG_STRIPARMOR','attack',217,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_ARMORBRAKE','attack',344,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@BS_ADRENALINE','chase',111,10,1000,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@BS_ADRENALINE','attack',111,10,1000,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_BLEEDING','attack',660,3,1000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_HEAVENDRIVE','chase',91,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_HEAVENDRIVE','attack',91,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_HEAVENDRIVE','chase',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_HEAVENDRIVE','attack',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2163,'Gold Antler Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@RG_STRIPWEAPON','attack',215,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_WEAPONBRAKER','attack',343,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@MC_MAMMONITE','attack',42,22,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@BS_HAMMERFALL','attack',110,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_GROUNDATTACK','attack',185,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_BLEEDING','attack',660,3,1000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@MG_THUNDERSTORM','chase',21,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@MG_THUNDERSTORM','attack',21,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@MG_THUNDERSTORM','chase',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@MG_THUNDERSTORM','attack',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2164,'Gold Rake Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_HEAL','attack',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_HEAL','attack',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_HEAL','chase',28,11,10000,0,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_HEAL','chase',28,11,10000,0,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@AL_TELEPORT','walk',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_POWERUP','attack',349,5,10000,0,120000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@WZ_HEAVENDRIVE','chase',91,20,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@WZ_HEAVENDRIVE','attack',91,20,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@CR_AUTOGUARD','attack',249,10,500,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_STONESKIN','idle',675,3,10000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_SUMMONSLAVE','chase',196,4,10000,700,10000,'no','self','slavele','3',2170,2171,2172,2173,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_SUMMONSLAVE','attack',196,4,10000,700,10000,'no','self','slavele','3',2170,2171,2172,2173,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_SUMMONSLAVE','idle',196,4,10000,700,10000,'no','self','slavele','3',2170,2171,2172,2173,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2165,'Gold Queen Scaraba@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',2161,2162,2163,2164,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2166,'Gold One-Horn Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2161,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2167,'Gold Two-Horn Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2162,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2168,'Gold Antler Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2163,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2169,'Gold Rake Scaraba Egg@NPC_METAMORPHOSIS','idle',193,1,100,2000,5000,'no','self','myhpltmaxrate','99',2164,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@RG_STRIPARMOR','attack',217,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_ARMORBRAKE','attack',344,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@BS_ADRENALINE','chase',111,10,1000,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@BS_ADRENALINE','attack',111,10,1000,1500,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_BLEEDING','attack',660,3,1000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_HEAVENDRIVE','chase',91,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_HEAVENDRIVE','attack',91,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_HEAVENDRIVE','chase',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_HEAVENDRIVE','attack',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2170,'Gold Antler Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@RG_STRIPWEAPON','attack',215,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_WEAPONBRAKER','attack',343,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@MC_MAMMONITE','attack',42,22,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@BS_HAMMERFALL','attack',110,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_GROUNDATTACK','attack',185,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_BLEEDING','attack',660,3,1000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@MG_THUNDERSTORM','chase',21,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@MG_THUNDERSTORM','attack',21,20,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@MG_THUNDERSTORM','chase',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@MG_THUNDERSTORM','attack',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2171,'Gold Rake Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@RG_STRIPHELM','attack',218,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_HELMBRAKE','attack',345,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_BLEEDING','attack',660,2,500,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_COMBOATTACK','attack',171,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@KN_TWOHANDQUICKEN','attack',60,30,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','chase',91,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','attack',91,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','chase',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_HEAVENDRIVE','attack',91,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2172,'Gold One-Horn Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@RG_STRIPSHIELD','attack',216,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_SHIELDBRAKE','attack',346,10,500,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_BLEEDING','attack',660,2,500,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@SM_BASH','attack',5,20,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_COMBOATTACK','attack',171,2,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@KN_TWOHANDQUICKEN','attack',60,30,1000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','chase',21,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','attack',21,20,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','chase',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@MG_THUNDERSTORM','attack',21,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@WZ_EARTHSPIKE','chase',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@WZ_EARTHSPIKE','attack',90,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','idle',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','chase',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_WIDECONFUSE','attack',667,3,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2173,'Gold Two-Horn Scaraba@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Malangdo Island
REPLACE INTO `mob_skill_db_re` VALUES (2174,'Deep Sea Vadon@ST_FULLSTRIP','attack',476,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2174,'Deep Sea Vadon@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2174,'Deep Sea Vadon@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2174,'Deep Sea Vadon@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2174,'Deep Sea Vadon@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@NPC_STOP','attack',342,1,200,0,600000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@AL_HEAL','attack',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2175,'Deep Sea Marse@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2176,'Deep Sea Crab@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2176,'Deep Sea Crab@MG_FROSTDIVER','chase',15,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2176,'Deep Sea Crab@MG_FROSTDIVER','attack',15,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@MG_THUNDERSTORM','chase',21,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2177,'Deep Sea Cornutus@MG_THUNDERSTORM','attack',21,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2178,'Deep Sea Shellfish@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2178,'Deep Sea Shellfish@MG_FROSTDIVER','chase',15,9,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2178,'Deep Sea Shellfish@NPC_STUNATTACK','attack',179,3,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2178,'Deep Sea Shellfish@MG_LIGHTNINGBOLT','chase',20,5,200,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2178,'Deep Sea Shellfish@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2179,'Deep Sea Kukre@ST_FULLSTRIP','attack',476,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2179,'Deep Sea Kukre@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2179,'Deep Sea Kukre@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2179,'Deep Sea Kukre@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_FROSTDIVER','attack',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_FROSTDIVER','chase',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_LIGHTNINGBOLT','attack',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_LIGHTNINGBOLT','chase',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_FROSTDIVER','angry',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_FROSTDIVER','follow',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_LIGHTNINGBOLT','angry',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@MG_LIGHTNINGBOLT','follow',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@NPC_STOP','attack',342,1,200,0,600000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@AL_HEAL','attack',28,7,10000,500,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@AL_HEAL','attack',28,7,10000,500,5000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2180,'Deep Sea Strouf@WZ_FROSTNOVA','chase',88,7,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@WZ_WATERBALL','angry',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@WZ_WATERBALL','follow',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@NPC_CRITICALWOUND','attack',673,5,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2181,'Deep Sea Sword Fish@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_STUNATTACK','angry',179,2,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_WATERATTACK','angry',184,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_EVILLAND','attack',670,10,10000,0,15000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2182,'Deep Sea Marc@NPC_CRITICALWOUND','attack',673,5,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2183,'Deep Sea Anolian@NPC_CRITICALWOUND','attack',673,2,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',2,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@PR_LEXDIVINA','chase',76,5,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@NPC_STOP','attack',342,1,100,0,600000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@ST_FULLSTRIP','attack',476,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2184,'Deep Sea Obeaune@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_WATERATTACK','attack',184,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'16',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_WATERATTACK','angry',184,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@WZ_WATERBALL','follow',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'16',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@ST_FULLSTRIP','attack',476,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2185,'Deep Sea Kapha@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_ARMORBRAKE','attack',344,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_AGIUP','attack',350,10,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_CRITICALWOUND','attack',673,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_WIDEBLEEDING','chase',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_WIDEBLEEDING','attack',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@WZ_WATERBALL','attack',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@WZ_WATERBALL','chase',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2186,'Coelacanth@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_ARMORBRAKE','attack',344,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_AGIUP','attack',350,10,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2195,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2195,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_CRITICALWOUND','attack',673,3,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_WIDEBLEEDING','chase',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_WIDEBLEEDING','attack',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_WATERBALL','attack',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_WATERBALL','chase',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2187,'Dark Coelacanth@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@AL_DECAGI','chase',30,48,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_ARMORBRAKE','attack',344,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_AGIUP','attack',350,10,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2195,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2195,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_CRITICALWOUND','attack',673,3,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_WIDEBLEEDING','chase',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_WIDEBLEEDING','attack',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_WATERBALL','attack',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_WATERBALL','chase',86,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2188,'Weird Coelacanth@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_WATERATTACK','attack',184,10,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_RANDOMATTACK','attack',183,8,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2196,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2196,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_WIDEFREEZE','chase',664,5,10000,0,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_WIDEFREEZE','attack',664,5,10000,0,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_WIDEBLEEDING','chase',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_WIDEBLEEDING','attack',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_CRITICALWOUND','attack',673,4,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@SA_DISPELL','chase',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@MG_FROSTDIVER','idle',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@MG_FROSTDIVER','chase',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_WATERBALL','chase',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_WATERBALL','attack',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@NPC_AGIUP','attack',350,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2189,'Mutant Coelacanth@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_WATERATTACK','attack',184,10,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_RANDOMATTACK','attack',183,8,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2196,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2196,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@BS_MAXIMIZE','attack',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@BS_MAXIMIZE','chase',114,1,500,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_WIDEFREEZE','chase',664,5,10000,0,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_WIDEFREEZE','attack',664,5,10000,0,30000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_WIDEBLEEDING','chase',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_WIDEBLEEDING','attack',665,3,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_GUIDEDATTACK','chase',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_CRITICALWOUND','attack',673,4,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@SA_DISPELL','attack',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@SA_DISPELL','chase',289,5,10000,0,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@MG_FROSTDIVER','idle',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@MG_FROSTDIVER','chase',15,40,5000,1000,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_WATERBALL','chase',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_WATERBALL','attack',86,10,5000,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_POWERUP','attack',349,5,10000,0,25000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@NPC_AGIUP','attack',350,5,10000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2190,'Violent Coelacanth@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2191,'Seaweed@NPC_INVINCIBLE','idle',685,1,10000,0,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2191,'Seaweed@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESILENCE','chase',663,1,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESILENCE','attack',663,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDEFREEZE','chase',664,1,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDEFREEZE','attack',664,1,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESLEEP','chase',668,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESLEEP','attack',668,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDEBLEEDING','chase',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDEBLEEDING','attack',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESOULDRAIN','chase',680,1,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESOULDRAIN','attack',680,1,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDECURSE','chase',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESTUN','chase',678,1,10000,1300,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2193,'Octupus Leg@NPC_WIDESTUN','attack',678,1,10000,1300,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESILENCE','chase',663,1,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESILENCE','attack',663,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDEFREEZE','chase',664,1,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDEFREEZE','attack',664,1,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESLEEP','chase',668,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESLEEP','attack',668,1,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDEBLEEDING','chase',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDEBLEEDING','attack',665,1,10000,500,100000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESOULDRAIN','chase',680,1,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESOULDRAIN','attack',680,1,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@WZ_WATERBALL','attack',86,10,10000,500,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@WZ_WATERBALL','chase',86,10,10000,500,25000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDECURSE','chase',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESTUN','chase',678,1,10000,1300,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WIDESTUN','attack',678,1,10000,1300,180000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WATERATTACK','attack',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_WATERATTACK','chase',184,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@HW_GANBANTEIN','attack',483,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_HELLJUDGEMENT','chase',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2194,'Giant Octopus@NPC_HELLJUDGEMENT','attack',662,2,2000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2195,'Deep Sea Kukre@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2195,'Deep Sea Kukre@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2195,'Deep Sea Kukre@ST_FULLSTRIP','attack',476,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2195,'Deep Sea Kukre@WZ_WATERBALL','attack',86,4,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_FROSTDIVER','attack',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_FROSTDIVER','chase',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_LIGHTNINGBOLT','attack',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_LIGHTNINGBOLT','chase',20,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_FROSTDIVER','angry',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@MG_FROSTDIVER','follow',15,9,1000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@NPC_WATERATTACK','angry',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@NPC_STOP','attack',342,1,200,0,600000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@AL_HEAL','attack',28,7,10000,500,5000,'yes','friend','friendhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@AL_HEAL','attack',28,7,10000,500,5000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2196,'Deep Sea Strouf@WZ_FROSTNOVA','chase',88,7,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2197,'Red Eruma@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2198,'King Dramoh@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2198,'King Dramoh@NPC_DEFENDER','attack',205,1,500,0,5000,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2198,'King Dramoh@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'38',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2198,'King Dramoh@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2199,'Siorava@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Izlude Dungeon F6
REPLACE INTO `mob_skill_db_re` VALUES (2201,'Sropho@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2201,'Sropho@MG_FROSTDIVER','chase',15,9,2000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2201,'Sropho@MG_FROSTDIVER','attack',15,9,2000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2201,'Sropho@WZ_JUPITEL','chase',84,4,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2201,'Sropho@WZ_JUPITEL','attack',84,4,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_EARTHQUAKE','chase',653,2,10000,1000,60000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_EARTHQUAKE','attack',653,2,10000,1000,60000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_AGIUP','attack',350,5,10000,0,60000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_AGIUP','chase',350,5,10000,0,60000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_DRAGONFEAR','attack',659,5,10000,0,30000,'no','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDECONFUSE','chase',667,2,10000,0,10000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDECONFUSE','attack',667,2,10000,0,10000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@SM_BASH','attack',5,10,2000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_COMBOATTACK','attack',171,2,2000,700,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_PULSESTRIKE','attack',661,5,2000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@MG_FROSTDIVER','chase',15,40,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@MG_FROSTDIVER','attack',15,40,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@WZ_STORMGUST','chase',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@WZ_STORMGUST','attack',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@CR_REFLECTSHIELD','idle',252,1,1000,0,200000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@CR_REFLECTSHIELD','chase',252,1,1000,0,200000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@CR_REFLECTSHIELD','attack',252,1,1000,0,200000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_HELLJUDGEMENT','chase',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_HELLJUDGEMENT','attack',662,10,10000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDEFREEZE','idle',664,5,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDEFREEZE','chase',664,5,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDEFREEZE','attack',664,5,10000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESLEEP','idle',668,5,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESLEEP','chase',668,5,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESLEEP','attack',668,5,10000,700,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESILENCE','idle',663,5,10000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESILENCE','chase',663,5,10000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDESILENCE','attack',663,5,10000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDECURSE','idle',677,5,10000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDECURSE','chase',677,5,10000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDECURSE','attack',677,5,10000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDEBLEEDING','chase',665,2,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_WIDEBLEEDING','attack',665,2,10000,500,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_SUMMONSLAVE','chase',196,4,10000,700,10000,'no','self','slavele','3',2206,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_SUMMONSLAVE','attack',196,4,10000,700,10000,'no','self','slavele','3',2206,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_SUMMONSLAVE','idle',196,4,10000,700,10000,'no','self','slavele','3',2206,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2202,'Kraken@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',2206,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@AL_HEAL','chase',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@AL_HEAL','attack',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@KN_SPEARBOOMERANG','chase',59,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@NPC_STONESKIN','chase',675,3,1000,500,30000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2203,'Pot Dofle@NPC_STONESKIN','attack',675,3,1000,500,30000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2204,'Sedora@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2204,'Sedora@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2204,'Sedora@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_LICK','attack',206,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDEFREEZE','idle',664,5,2000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDEFREEZE','chase',664,5,2000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDEFREEZE','attack',664,5,2000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDECURSE','idle',677,5,2000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDECURSE','chase',677,5,2000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@NPC_WIDECURSE','attack',677,5,2000,800,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@MG_FROSTDIVER','chase',15,40,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@MG_FROSTDIVER','attack',15,40,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@WZ_STORMGUST','chase',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2206,'Kraken Leg@WZ_STORMGUST','attack',89,10,2000,1000,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2208,'Wild Rider@NPC_WATERATTACK','attack',184,10,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2208,'Wild Rider@NPC_STUNATTACK','attack',179,3,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2208,'Wild Rider@WZ_WATERBALL','chase',86,7,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2208,'Wild Rider@WZ_WATERBALL','attack',86,7,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Bio 4
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_DEFENDER','chase',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@SM_MAGNUM','attack',7,9,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@PA_PRESSURE','chase',367,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_SHIELDBOOMERANG','chase',251,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_SHIELDCHARGE','attack',250,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_GRANDCROSS','chase',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@CR_GRANDCROSS','attack',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2221,'Randel@NPC_HOLYATTACK','attack',189,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@CR_ACIDDEMONSTRATION','chase',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@CR_ACIDDEMONSTRATION','attack',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_ACIDTERROR','chase',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_ACIDTERROR','attack',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_DEMONSTRATION','attack',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2222,'Flamel@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','83',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','85',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','89',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2213',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2216',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2446',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2447',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2449',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2450',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@SA_LANDPROTECTOR','attack',288,1,5000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@PF_SPIDERWEB','attack',405,1,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@PF_SOULBURN','attack',375,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@PF_MINDBREAKER','attack',402,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_SOULSTRIKE','chase',13,20,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREBOLT','chase',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_COLDBOLT','chase',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_COLDBOLT','attack',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREBALL','idle',17,44,10000,500,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2223,'Celia@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@MO_BODYRELOCATION','chase',264,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@MO_EXTREMITYFIST','attack',271,5,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@NPC_COMBOATTACK','chase',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@NPC_COMBOATTACK','attack',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','idle',28,3,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','idle',28,3,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','chase',28,3,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','chase',28,3,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','attack',28,3,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@AL_HEAL','attack',28,3,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2224,'Chen@NPC_WATERATTACK','attack',184,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@ST_REJECTSWORD','idle',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@ST_REJECTSWORD','chase',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@ST_REJECTSWORD','attack',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@TF_HIDING','attack',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_STRIPSHIELD','attack',216,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_STRIPARMOR','attack',217,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_BACKSTAP','attack',212,10,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@RG_RAID','attack',214,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@TF_POISON','attack',52,10,10000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@SM_BASH','chase',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@SM_BASH','attack',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2225,'Gertie@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_PULSESTRIKE','attack',661,5,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@CG_TAROTCARD','chase',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@CG_TAROTCARD','attack',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@WZ_FROSTNOVA','chase',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@WZ_FROSTNOVA','attack',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@BA_MUSICALSTRIKE','chase',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@BA_MUSICALSTRIKE','attack',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2226,'Alphoccio@BA_DISSONANCE','chase',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2226,'Alphoccio@BA_DISSONANCE','attack',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2226,'Alphoccio@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_PULSESTRIKE','attack',661,5,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@CG_TAROTCARD','chase',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@CG_TAROTCARD','attack',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_WIDESTUN','chase',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_WIDESTUN','attack',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@DC_THROWARROW','chase',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@DC_THROWARROW','attack',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2227,'Trentini@DC_UGLYDANCE','chase',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2227,'Trentini@DC_UGLYDANCE','attack',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2227,'Trentini@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@WZ_STORMGUST','idle',89,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@WZ_STORMGUST','walk',89,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_DEFENDER','chase',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@SM_MAGNUM','attack',7,9,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@PA_PRESSURE','chase',367,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_SHIELDBOOMERANG','chase',251,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_SHIELDCHARGE','attack',250,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_HOLYCROSS','attack',253,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_GRANDCROSS','chase',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@CR_GRANDCROSS','attack',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2228,'Paladin Randel@NPC_HOLYATTACK','attack',189,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@CR_ACIDDEMONSTRATION','chase',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@CR_ACIDDEMONSTRATION','attack',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_ACIDTERROR','chase',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_ACIDTERROR','attack',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_DEMONSTRATION','attack',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_DEMONSTRATION','attack',229,5,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2229,'Creator Flamel@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','83',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','85',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','89',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2213',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2216',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2446',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2447',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2449',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2450',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@SA_LANDPROTECTOR','attack',288,1,5000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@PF_SPIDERWEB','attack',405,1,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@PF_SOULBURN','attack',375,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@PF_MINDBREAKER','attack',402,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_SOULSTRIKE','chase',13,20,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBOLT','chase',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBOLT','attack',19,10,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_COLDBOLT','chase',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_COLDBOLT','attack',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_COLDBOLT','attack',14,10,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBALL','idle',17,44,10000,500,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2230,'Professor Celia@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@MO_BODYRELOCATION','chase',264,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@MO_EXTREMITYFIST','attack',271,5,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_COMBOATTACK','chase',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_COMBOATTACK','attack',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_COMBOATTACK','attack',171,5,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','idle',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','idle',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','chase',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','chase',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','attack',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@AL_HEAL','attack',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2231,'Champion Chen@NPC_WATERATTACK','attack',184,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@ST_REJECTSWORD','idle',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@ST_REJECTSWORD','chase',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@ST_REJECTSWORD','attack',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_INVISIBLE','idle',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_INVISIBLE','chase',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_INVISIBLE','attack',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@TF_HIDING','attack',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_STRIPSHIELD','attack',216,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_STRIPARMOR','attack',217,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_BACKSTAP','attack',212,10,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@RG_RAID','attack',214,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@TF_POISON','attack',52,10,10000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@SM_BASH','chase',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@SM_BASH','attack',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_GUIDEDATTACK','attack',172,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2232,'Stalker Gertie@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_PULSESTRIKE','attack',661,5,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@CG_TAROTCARD','chase',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@CG_TAROTCARD','attack',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@CG_TAROTCARD','attack',489,5,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@WZ_FROSTNOVA','chase',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@WZ_FROSTNOVA','attack',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@BA_MUSICALSTRIKE','chase',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@BA_MUSICALSTRIKE','attack',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2233,'Clown Alphoccio@BA_DISSONANCE','chase',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2233,'Clown Alphoccio@BA_DISSONANCE','attack',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2233,'Clown Alphoccio@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_SELFDESTRUCTION','attack',173,1,2000,2000,0,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@WZ_METEOR','idle',83,1,10000,0,0,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@WZ_METEOR','walk',83,1,5000,0,5000,'yes','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_PULSESTRIKE','attack',661,5,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@CG_TAROTCARD','chase',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@CG_TAROTCARD','attack',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@CG_TAROTCARD','attack',489,5,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_WIDESTUN','chase',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_WIDESTUN','attack',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@DC_THROWARROW','chase',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@DC_THROWARROW','attack',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2234,'Gypsy Trentini@DC_UGLYDANCE','chase',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2234,'Gypsy Trentini@DC_UGLYDANCE','attack',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2234,'Gypsy Trentini@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2229,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2229,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2229,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_DEFENDER','idle',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_DEFENDER','chase',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_DEFENDER','attack',205,5,10000,0,180000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@SM_ENDURE','chase',8,10,2000,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@SM_MAGNUM','attack',7,9,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@PA_PRESSURE','chase',367,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_SHIELDBOOMERANG','chase',251,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_SHIELDCHARGE','attack',250,5,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_HOLYCROSS','attack',253,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_GRANDCROSS','chase',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@CR_GRANDCROSS','attack',254,10,5000,100,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2235,'Paladin Randel@NPC_HOLYATTACK','attack',189,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2230,2231,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_CANNIBALIZE','idle',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_CANNIBALIZE','chase',232,5,5000,0,10000,'no','around2','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@CR_ACIDDEMONSTRATION','chase',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@CR_ACIDDEMONSTRATION','attack',490,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_ACIDTERROR','chase',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_ACIDTERROR','attack',230,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_DEMONSTRATION','attack',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_DEMONSTRATION','attack',229,5,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','idle',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','chase',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@AM_POTIONPITCHER','attack',231,4,2000,0,0,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2236,'Creator Flamel@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2231,2232,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2231,2232,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2229,2231,2232,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','83',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','85',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','89',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2213',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2216',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2446',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2447',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2449',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','chase',288,1,10000,0,5000,'no','target','skillused','2450',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@SA_LANDPROTECTOR','attack',288,1,5000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@PF_SPIDERWEB','attack',405,1,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@PF_SOULBURN','attack',375,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@PF_MINDBREAKER','attack',402,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_SOULSTRIKE','chase',13,20,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBOLT','chase',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBOLT','attack',19,10,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_COLDBOLT','chase',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_COLDBOLT','attack',14,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_COLDBOLT','attack',14,10,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBALL','idle',17,44,10000,500,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2237,'Professor Celia@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2229,2230,2232,2234,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_INCAGI','idle',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_INCAGI','chase',29,10,10000,1000,240000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@MO_BODYRELOCATION','chase',264,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@MO_EXTREMITYFIST','attack',271,5,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_COMBOATTACK','chase',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_COMBOATTACK','attack',171,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_COMBOATTACK','attack',171,5,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','idle',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','idle',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','chase',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','chase',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','attack',28,11,3000,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@AL_HEAL','attack',28,11,3000,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2238,'Champion Chen@NPC_WATERATTACK','attack',184,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2229,2230,2231,2233,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@ST_REJECTSWORD','idle',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@ST_REJECTSWORD','chase',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@ST_REJECTSWORD','attack',390,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_INVISIBLE','idle',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_INVISIBLE','chase',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_INVISIBLE','attack',353,1,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@TF_HIDING','attack',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_STRIPSHIELD','attack',216,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_STRIPARMOR','attack',217,5,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_BACKSTAP','attack',212,10,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@RG_RAID','attack',214,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@TF_POISON','attack',52,10,10000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@SM_BASH','chase',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@SM_BASH','attack',5,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_GUIDEDATTACK','attack',172,10,2000,0,0,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_AGIUP','attack',350,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2239,'Stalker Gertie@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_PULSESTRIKE','attack',661,5,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@CG_TAROTCARD','chase',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@CG_TAROTCARD','attack',489,5,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@CG_TAROTCARD','attack',489,5,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@WZ_FROSTNOVA','chase',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@WZ_FROSTNOVA','attack',88,10,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@BA_MUSICALSTRIKE','chase',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@BA_MUSICALSTRIKE','attack',316,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2240,'Clown Alphoccio@BA_DISSONANCE','chase',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2240,'Clown Alphoccio@BA_DISSONANCE','attack',317,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2240,'Clown Alphoccio@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','slavele','3',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_SUMMONMONSTER','attack',209,5,10000,700,60000000,'yes','self','myhpltmaxrate','30',2228,2229,2230,2231,2232,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_PULSESTRIKE','attack',661,5,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_SLOWCAST','chase',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@CG_TAROTCARD','chase',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@CG_TAROTCARD','attack',489,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@CG_TAROTCARD','attack',489,5,2000,0,0,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@CG_ARROWVULCAN','chase',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@CG_ARROWVULCAN','attack',394,10,10000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_WIDESTUN','chase',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_WIDESTUN','attack',678,5,5000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@DC_THROWARROW','chase',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@DC_THROWARROW','attack',324,5,10000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
# currently not working properly
#REPLACE INTO `mob_skill_db_re` VALUES ( 2241,'Gypsy Trentini@DC_UGLYDANCE','chase',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2241,'Gypsy Trentini@DC_UGLYDANCE','attack',325,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_AGIUP','idle',350,5,10000,0,35000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_AGIUP','attack',350,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_POWERUP','attack',349,5,2000,0,100000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_POWERUP','attack',349,5,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2241,'Gypsy Trentini@NPC_WINDATTACK','attack',187,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2242,'Desert Wolf@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2242,'Desert Wolf@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2242,'Desert Wolf@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2242,'Desert Wolf@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2242,'Desert Wolf@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2243,'Desert Wolf B@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2243,'Desert Wolf B@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2243,'Desert Wolf B@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2243,'Desert Wolf B@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2243,'Desert Wolf B@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);

# Guild Dungeon F2
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_SUMMONSLAVE','attack',196,1,10000,2000,5000,'no','self','slavele','1',2277,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_SUMMONSLAVE','idle',196,1,10000,2000,5000,'no','self','slavele','1',2277,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_SUMMONSLAVE','chase',196,1,10000,2000,5000,'no','self','slavele','1',2277,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@HW_GANBANTEIN','attack',483,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@WZ_METEOR','attack',83,11,2000,500,5000,'no','target','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2249,'Pyuriel@NPC_CRITICALWOUND','attack',673,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@SM_BASH','attack',5,9,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@SM_BASH','attack',5,9,10000,800,1000,'no','target','skillused','5',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@SM_MAGNUM','attack',7,9,5000,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_EARTHQUAKE','chase',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@SM_PROVOKE','chase',6,9,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_FIREATTACK','attack',186,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2250,'Lora@NPC_BLEEDING','attack',660,3,10,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@AL_HEAL','idle',28,1,10000,0,10000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@MG_THUNDERSTORM','attack',21,20,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_SUMMONSLAVE','attack',196,1,10000,1000,30000,'no','self','slavele','1',2278,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@MG_SAFETYWALL','attack',12,10,1000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_SUMMONSLAVE','idle',196,1,10000,1000,30000,'no','self','slavele','1',2278,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_SUMMONSLAVE','chase',196,1,10000,1000,30000,'no','self','slavele','1',2278,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@ST_FULLSTRIP','attack',476,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_CRITICALWOUND','attack',673,3,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_FIREBREATH','chase',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_FIREBREATH','attack',654,10,2000,500,5000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@SA_LANDPROTECTOR','attack',288,3,5000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@KN_TWOHANDQUICKEN','attack',60,9,10000,0,100000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_EARTHQUAKE','chase',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_EARTHQUAKE','attack',653,5,10000,500,20000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_PULSESTRIKE','attack',661,5,10000,0,15000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_DRAGONFEAR','attack',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@NPC_DRAGONFEAR','chase',659,5,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2251,'Gioia@WZ_METEOR','idle',83,9,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@WZ_JUPITEL','attack',84,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@WZ_JUPITEL','chase',84,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@WZ_JUPITEL','attack',84,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@WZ_JUPITEL','chase',84,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2252,'Elvira@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@AS_SONICBLOW','attack',136,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@BS_MAXIMIZE','chase',114,5,500,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@WZ_WATERBALL','chase',86,10,10000,500,5000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_AGIUP','attack',350,5,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_WEAPONBRAKER','attack',343,10,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_STUNATTACK','attack',179,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','1',2279,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_SUMMONSLAVE','idle',196,1,10000,700,10000,'no','self','slavele','1',2279,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@WZ_WATERBALL','attack',86,5,2000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2253,'Daehyon@NPC_BLEEDING','attack',660,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@AS_SONICBLOW','attack',136,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@NPC_CRITICALWOUND','attack',673,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2254,'Soheon@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_FIREATTACK','attack',186,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_TELEKINESISATTACK','attack',191,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@WZ_FIREPILLAR','idle',80,10,1000,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@AM_DEMONSTRATION','chase',229,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@KN_PIERCE','attack',56,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_POWERUP','attack',349,5,10000,700,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_POWERUP','attack',349,5,2000,700,50000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_FIREBREATH','chase',654,10,2000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@SM_MAGNUM','attack',7,25,10000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@WZ_SIGHTRASHER','attack',81,10,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@MG_FIREBALL','attack',17,44,1000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@MG_FIREBALL','chase',17,44,10000,500,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_BLEEDING','attack',660,3,2000,0,50000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_SUMMONSLAVE','attack',196,1,10000,700,30000,'no','self','slavele','1',2280,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@NPC_SUMMONSLAVE','idle',196,1,10000,700,30000,'no','self','slavele','1',2280,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2255,'Kades@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@MG_NAPALMBEAT','attack',11,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2256,'Rudo@MG_NAPALMBEAT','chase',11,5,1000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@SM_BASH','attack',5,9,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@SM_BASH','attack',5,9,10000,800,1000,'no','target','skillused','5',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@SM_MAGNUM','attack',7,9,5000,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@SM_PROVOKE','chase',6,9,500,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_FIREATTACK','attack',186,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_EARTHQUAKE','chase',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_EMOTION','chase',197,1,10000,0,5000,'yes','self','skillused','28',15,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2277,'Lora@NPC_BLEEDING','attack',660,3,10,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@WZ_JUPITEL','attack',84,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@WZ_JUPITEL','chase',84,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@WZ_JUPITEL','attack',84,28,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@WZ_JUPITEL','chase',84,28,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2278,'Elvira@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@NPC_INVISIBLE','chase',353,1,2000,1000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@PR_SANCTUARY','attack',70,10,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@AS_SONICBLOW','attack',136,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@WZ_WATERBALL','chase',86,3,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@WZ_WATERBALL','attack',86,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@NPC_CRITICALWOUND','attack',673,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2279,'Soheon@NPC_STOP','attack',342,1,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@MG_NAPALMBEAT','attack',11,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2280,'Rudo@MG_NAPALMBEAT','chase',11,5,1000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2281,'Dark Shadow@NPC_DARKSTRIKE','chase',340,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2281,'Dark Shadow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2281,'Dark Shadow@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2281,'Dark Shadow@NPC_DARKNESSBREATH','attack',658,1,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@CR_REFLECTSHIELD','attack',252,2,500,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2282,'Dark Frame@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@NPC_DARKCROSS','attack',338,3,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@NPC_CHANGEUNDEAD','attack',348,1,300,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@MO_BODYRELOCATION','chase',264,1,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@NPC_DARKBREATH','chase',202,2,500,800,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2283,'Dark Priest@NPC_BLEEDING','attack',660,3,10,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2284,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2284,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2284,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2284,'Kobold@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2285,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2285,'Kobold@NPC_POISONATTACK','attack',188,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2285,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2285,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2286,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2286,'Kobold@NPC_FIREATTACK','attack',186,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2286,'Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2286,'Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2287,'Kobold Archer@AC_DOUBLE','attack',46,3,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2287,'Kobold Archer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2287,'Kobold Archer@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2306,'Golden Thief Bug@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Malaya Port
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@BS_HAMMERFALL','attack',110,5,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@CR_AUTOGUARD','attack',249,10,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@AS_VENOMDUST','chase',140,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@AS_VENOMDUST','attack',140,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2309,'Bungisngis@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2310,'Engkanto@AL_DECAGI','chase',30,5,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2310,'Engkanto@SM_PROVOKE','chase',6,5,1000,600,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2310,'Engkanto@SM_PROVOKE','attack',6,5,1000,600,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2310,'Engkanto@NPC_BLINDATTACK','chase',177,5,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2310,'Engkanto@NPC_BLINDATTACK','attack',177,5,1000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@AL_TELEPORT','idle',26,1,1000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@AL_TELEPORT','attack',26,1,100,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@NPC_BLOODDRAIN','chase',199,1,2000,0,5000,'yes','randomtarget','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@NPC_COMBOATTACK','chase',171,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2311,'Manananggal@NPC_COMBOATTACK','attack',171,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@AL_TELEPORT','idle',26,1,500,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@AL_TELEPORT','attack',26,1,50,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@NPC_DARKTHUNDER','chase',341,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@NPC_DARKTHUNDER','attack',341,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@NPC_DARKSTRIKE','chase',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2312,'Mangkukulam@NPC_DARKSTRIKE','attack',340,9,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@AL_DECAGI','chase',30,48,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@AL_INCAGI','chase',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@AL_INCAGI','attack',29,10,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@TF_HIDING','idle',51,1,1000,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@TF_HIDING','attack',51,1,1000,500,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@BS_MAXIMIZE','chase',114,10,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@BS_MAXIMIZE','attack',114,10,1000,500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@NPC_LICK','attack',206,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2313,'Tikbalang@SM_BASH','attack',5,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@TF_HIDING','attack',51,1,500,500,50000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2314,'Tiyanak@NPC_REBIRTH','dead',208,3,5000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@AL_TELEPORT','idle',26,1,500,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@AL_TELEPORT','attack',26,1,50,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_BLOODDRAIN','chase',199,1,2000,0,5000,'yes','randomtarget','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_DARKTHUNDER','chase',341,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_DARKTHUNDER','attack',341,3,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2315,'Wakwak@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2316,'Jejeling@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2316,'Jejeling@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2316,'Jejeling@WZ_WATERBALL','attack',86,4,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@BS_HAMMERFALL','attack',110,10,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@HW_GANBANTEIN','attack',483,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@WZ_METEOR','attack',83,11,2000,500,5000,'no','target','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@WZ_VERMILION','attack',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2317,'Bangungot@NPC_CRITICALWOUND','attack',673,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2318,'Bangungot@NPC_ALLHEAL','idle',687,1,3000,2000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2318,'Bangungot@NPC_ALLHEAL','attack',687,1,3000,2000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_ICEBREATH','attack',655,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_DARKBREATH','attack',202,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_WEAPONBRAKER','attack',343,10,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@CR_REFLECTSHIELD','attack',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_STORMGUST','attack',89,10,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_STORMGUST','chase',89,10,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_WATERBALL','chase',86,10,5000,500,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@SM_PROVOKE','attack',6,10,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@WZ_JUPITEL','attack',84,28,10000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2319,'Buwaya@NPC_CRITICALWOUND','attack',673,3,2000,0,60000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_RANDOMATTACK','attack',183,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@AS_SONICBLOW','attack',136,10,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_POWERUP','attack',349,3,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@WZ_METEOR','attack',83,11,2000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_BLEEDING','attack',660,2,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2320,'Bakonawa@NPC_CRITICALWOUND','attack',673,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@AL_HEAL','idle',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@AL_HEAL','attack',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@AL_HEAL','chase',28,11,10000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_DRAGONFEAR','chase',659,5,10000,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_ICEBREATH','attack',655,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_DARKBREATH','attack',202,5,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_WEAPONBRAKER','attack',343,10,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@CR_REFLECTSHIELD','attack',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_RANDOMATTACK','attack',183,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_STORMGUST','attack',89,10,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_STORMGUST','chase',89,10,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_VERMILION','attack',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_VERMILION','chase',85,21,2000,500,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_WATERBALL','chase',86,10,5000,500,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@AS_SONICBLOW','attack',136,10,2000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@WZ_METEOR','attack',83,11,2000,1000,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_BLEEDING','attack',660,2,2000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2322,'Bakonawa@NPC_CRITICALWOUND','attack',673,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2323,'Event Bomb@NPC_SELFDESTRUCTION','idle',173,1,10000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2323,'Event Bomb@NPC_SELFDESTRUCTION','attack',173,1,10000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2323,'Event Bomb@NPC_SELFDESTRUCTION','chase',173,1,10000,1000,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2324,'Event Range@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2324,'Event Range@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2324,'Event Range@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2329,'Buwaya Egg@NPC_METAMORPHOSIS','idle',193,1,2000,2000,5000,'no','self','always','0',2330,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@TF_THROWSTONE','chase',152,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@NPC_PIERCINGATT','attack',158,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2330,'Buwaya Slave@NPC_FIREBREATH','attack',654,3,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2331,'Seaweed@AL_DECAGI','idle',30,1,10000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2331,'Seaweed@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2337,'Hidden Mob@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2337,'Hidden Mob@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2337,'Hidden Mob@WZ_METEOR','attack',83,5,10000,0,1000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_COMBOATTACK','attack',171,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_COMBOATTACK','angry',171,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2338,'Bangungot Manananggal@NPC_WINDATTACK','angry',187,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_DARKTHUNDER','attack',341,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_DARKSTRIKE','attack',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_DARKSTRIKE','chase',340,9,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2339,'Bangungot Mangkukulam@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2340,'Tiyanak@NPC_REBIRTH','dead',208,3,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_CALLSLAVE','chase',352,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_AGIUP','chase',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@HP_ASSUMPTIO','chase',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@HP_ASSUMPTIO','attack',361,5,10000,0,100000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@HW_GANBANTEIN','attack',483,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@KN_TWOHANDQUICKEN','attack',60,30,10000,0,90000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@SA_DISPELL','attack',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@SA_DISPELL','chase',289,5,10000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_HOLYATTACK','attack',189,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@KN_BRANDISHSPEAR','attack',57,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@MG_THUNDERSTORM','attack',21,20,10000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_EARTHQUAKE','chase',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_EARTHQUAKE','attack',653,5,10000,1000,25000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@WZ_VERMILION','attack',85,21,1000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@WZ_VERMILION','chase',85,21,2000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2341,'RWC Boss@NPC_CRITICALWOUND','attack',673,4,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2343,'Hidden Mob@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2343,'Hidden Mob@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2343,'Hidden Mob@WZ_STORMGUST','attack',89,5,10000,0,1000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Nightmare Pyramids
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2353,'Nightmare Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2354,'Nightmare Mummy@NPC_UNDEADATTACK','attack',347,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2355,'Nightmare Verit@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always','0',2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2355,'Nightmare Verit@NPC_POISON','attack',176,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2355,'Nightmare Verit@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2356,'Nightmare Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2356,'Nightmare Mimic@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2356,'Nightmare Mimic@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2356,'Nightmare Mimic@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2356,'Nightmare Mimic@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2357,'Nightmare Mimic@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2357,'Nightmare Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2358,'Nightmare Arclouse@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2358,'Nightmare Arclouse@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2358,'Nightmare Arclouse@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2358,'Nightmare Arclouse@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2359,'Nightmare Arclouse@NPC_GROUNDATTACK','angry',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2359,'Nightmare Arclouse@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2360,'Nightmare Ancient Mummy@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2360,'Nightmare Ancient Mummy@NPC_UNDEADATTACK','attack',347,4,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2360,'Nightmare Ancient Mummy@NPC_DARKBREATH','attack',202,4,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_BLOODDRAIN','idle',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_BLOODDRAIN','idle',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_BLOODDRAIN','attack',199,1,2000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','longrangeattacked','0',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','casttargeted','0',1474,1477,1438,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','attack',209,5,5000,0,10000,'no','self','longrangeattacked','0',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','attack',209,5,5000,0,10000,'no','self','casttargeted','0',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','18',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','89',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','21',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','28',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','79',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_SUMMONMONSTER','idle',209,5,5000,0,10000,'no','self','skillused','83',2357,2359,2361,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','21',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','28',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','79',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','skillused','83',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,0,'yes','randomtarget','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','attack',83,11,5000,0,3000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_FIREPILLAR','idle',80,10,10000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_FIREPILLAR','idle',80,10,10000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_FIREPILLAR','attack',80,10,5000,0,2000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@NPC_DARKBLESSING','attack',203,1,5000,0,2000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@WZ_METEOR','idle',83,11,10000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2362,'Nightmare Amon Ra@AL_HEAL','idle',28,11,10000,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Eclage
REPLACE INTO `mob_skill_db_re` VALUES (2363,'Menblatt@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2363,'Menblatt@NPC_DARKSTRIKE','chase',340,5,2000,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2363,'Menblatt@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2363,'Menblatt@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2364,'Petal@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2364,'Petal@NPC_COMBOATTACK','attack',171,4,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2364,'Petal@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',22,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2365,'Cenere@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2365,'Cenere@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2365,'Cenere@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2365,'Cenere@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2365,'Cenere@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2366,'Antique Book@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2366,'Antique Book@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2366,'Antique Book@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@MG_FROSTDIVER','chase',15,9,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@MG_FROSTDIVER','attack',15,9,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@MG_COLDBOLT','chase',14,6,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@MG_COLDBOLT','attack',14,6,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@WZ_WATERBALL','attack',86,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2367,'Lichtern B@NPC_WATERATTACK','attack',184,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@NPC_STUNATTACK','chase',179,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@NPC_STUNATTACK','attack',179,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@MG_SOULSTRIKE','chase',13,9,3000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@MG_SOULSTRIKE','attack',13,9,3000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@MG_NAPALMBEAT','attack',11,7,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@MG_NAPALMBEAT','attack',11,7,4000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2368,'Lichtern Y@NPC_TELEKINESISATTACK','attack',191,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@NPC_SLEEPATTACK','chase',182,5,1000,300,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@NPC_SLEEPATTACK','attack',182,5,1000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@MG_FIREBOLT','chase',19,7,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@MG_FIREBOLT','attack',19,7,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@MG_FIREBALL','chase',17,5,3000,500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@MG_FIREBALL','attack',17,7,2000,500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2369,'Lichtern R@NPC_FIREATTACK','attack',186,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@MG_STONECURSE','chase',16,8,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@MG_STONECURSE','attack',16,8,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@WZ_EARTHSPIKE','chase',90,5,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@WZ_EARTHSPIKE','attack',90,5,3000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@WZ_QUAGMIRE','chase',92,5,500,700,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2370,'Lichtern G@NPC_GROUNDATTACK','attack',185,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@WZ_FIREPILLAR','idle',80,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@SA_DISPELL','attack',289,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2371,'Faithful Manager@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x81,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2408,'Dummy 10@AL_HEAL','idle',28,11,10000,0,10000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2409,'Dummy 50@AL_HEAL','idle',28,11,10000,0,10000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2410,'Dummy 100@AL_HEAL','idle',28,11,10000,0,10000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2411,'Dummy 150@AL_HEAL','idle',28,11,10000,0,10000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_SPEARBOOMERANG','follow',59,5,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@KN_PIERCE','attack',56,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@SM_MAGNUM','attack',7,25,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@SM_PROVOKE','attack',6,5,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@SM_PROVOKE','chase',6,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2415,'Knight Seyren@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_CLOAKING','attack',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_CLOAKING','idle',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_CLOAKING','chase',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_VENOMDUST','idle',140,10,2000,0,50000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_VENOMDUST','attack',140,10,2000,0,50000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@NPC_POISONATTACK','attack',188,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2416,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@BS_HAMMERFALL','chase',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@BS_HAMMERFALL','attack',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@NPC_FIREATTACK','attack',186,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2417,'Blacksmith Howard@NPC_WATERATTACK','attack',184,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_SKIDTRAP','idle',115,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_SKIDTRAP','attack',115,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_FLASHER','idle',120,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_FLASHER','attack',120,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_FREEZINGTRAP','idle',121,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@HT_FREEZINGTRAP','attack',121,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2418,'Hunter Cecil@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FROSTDIVER','idle',15,1,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_JUPITEL','idle',84,28,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FROSTDIVER','attack',15,1,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_COLDBOLT','attack',14,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_COLDBOLT','chase',14,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_SIGHTRASHER','attack',81,2,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_QUAGMIRE','chase',92,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_SAFETYWALL','attack',12,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FIREBOLT','chase',19,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FIREBOLT','attack',19,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_LIGHTNINGBOLT','chase',20,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_LIGHTNINGBOLT','attack',20,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_METEOR','attack',83,11,3000,1500,10000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_STORMGUST','attack',89,10,3000,1500,10000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,10000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,10000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,10000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2419,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,10000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_KYRIE','idle',73,10,10000,1000,10000,'no','randomtarget','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_KYRIE','chase',73,10,10000,1000,10000,'no','target','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_KYRIE','idle',73,10,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_KYRIE','chase',73,10,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_INCAGI','attack',29,1,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_INCAGI','chase',29,1,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXDIVINA','attack',76,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXDIVINA','chase',76,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXDIVINA','idle',76,5,10000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXDIVINA','attack',76,5,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXDIVINA','chase',76,5,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@PR_LEXAETERNA','chase',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@NPC_HOLYATTACK','attack',189,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','attack',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','attack',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','idle',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','idle',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','chase',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2420,'Priest Margaretha@AL_HEAL','chase',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_SHIELDCHARGE','attack',250,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_SHIELDBOOMERANG','idle',251,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_SHIELDBOOMERANG','chase',251,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_SHIELDBOOMERANG','follow',251,5,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_DEFENDER','idle',257,3,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_DEFENDER','chase',257,3,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_DEFENDER','attack',257,3,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_AUTOGUARD','attack',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_GRANDCROSS','attack',254,10,2000,500,2500,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@AL_HEAL','idle',28,5,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2421,'Crusader Egnigem@AL_HEAL','chase',28,5,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@TF_THROWSTONE','chase',152,1,10000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@TF_THROWSTONE','follow',152,1,10000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@NPC_POISON','attack',176,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_STRIPWEAPON','attack',215,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_STRIPARMOR','attack',217,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_STRIPSHIELD','attack',216,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'17',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_STRIPHELM','attack',218,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_INTIMIDATE','attack',219,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2422,'Rogue Wickebine@NPC_POISONATTACK','attack',188,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@ITM_TOMAHAWK','idle',337,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@ITM_TOMAHAWK','chase',337,1,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@ITM_TOMAHAWK','follow',337,1,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@AM_DEMONSTRATION','chase',229,5,5000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@AM_DEMONSTRATION','attack',229,5,5000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@AM_POTIONPITCHER','idle',231,4,10000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@AM_POTIONPITCHER','attack',231,4,10000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2423,'Alchemist Armeyer@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@BA_FROSTJOKER','idle',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@BA_FROSTJOKER','chase',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@BA_FROSTJOKER','follow',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2424,'Bard Kavach@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@DC_SCREAM','idle',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@DC_SCREAM','chase',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@DC_SCREAM','follow',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2425,'Dancer Josephina@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@MG_SOULSTRIKE','idle',13,10,10000,0,1000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@MG_SOULSTRIKE','attack',13,10,5000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@MG_SOULSTRIKE','chase',13,10,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@SA_SPELLBREAKER','attack',277,5,1000,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@SA_DISPELL','attack',289,1,200,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@MG_THUNDERSTORM','attack',21,20,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@SA_LANDPROTECTOR','attack',288,1,3000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@SA_LANDPROTECTOR','chase',288,1,3000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2426,'Sage Laurell@SA_LANDPROTECTOR','chase',288,1,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_FINGEROFFENSIVE','idle',267,3,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_FINGEROFFENSIVE','chase',267,3,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_FINGEROFFENSIVE','follow',267,3,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_TRIPLEATTACK','attack',263,5,1000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_INVESTIGATE','attack',266,5,1000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_BODYRELOCATION','walk',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_EXTREMITYFIST','attack',271,5,100,1500,15000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2427,'Monk Errende@MO_EXTREMITYFIST','attack',271,5,50,1500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_SPEARBOOMERANG','follow',59,5,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@SM_BASH','attack',5,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@KN_PIERCE','attack',56,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@SM_MAGNUM','attack',7,25,1000,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@SM_PROVOKE','attack',6,5,1000,0,300000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@SM_PROVOKE','chase',6,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@NPC_STUNATTACK','attack',179,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2428,'Knight Seyren@NPC_FIREATTACK','attack',186,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_CLOAKING','attack',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_CLOAKING','idle',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_CLOAKING','chase',135,3,2000,200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_GRIMTOOTH','idle',137,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_SONICBLOW','attack',136,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_VENOMDUST','idle',140,10,2000,0,50000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_VENOMDUST','attack',140,10,2000,0,50000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@NPC_POISONATTACK','attack',188,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2429,'Assassin Eremes@AS_GRIMTOOTH','chase',137,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@BS_MAXIMIZE','attack',114,1,1000,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@BS_HAMMERFALL','chase',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@BS_HAMMERFALL','attack',110,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@NPC_FIREATTACK','attack',186,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2430,'Blacksmith Howard@NPC_WATERATTACK','attack',184,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_SKIDTRAP','idle',115,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_SKIDTRAP','attack',115,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_FLASHER','idle',120,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_FLASHER','attack',120,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_FREEZINGTRAP','idle',121,5,2000,0,60000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@HT_FREEZINGTRAP','attack',121,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2431,'Hunter Cecil@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FROSTDIVER','idle',15,1,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_JUPITEL','idle',84,28,10000,1000,10000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FROSTDIVER','attack',15,1,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_JUPITEL','attack',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_COLDBOLT','attack',14,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_COLDBOLT','chase',14,5,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_SIGHTRASHER','attack',81,2,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_QUAGMIRE','chase',92,5,3000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_SAFETYWALL','attack',12,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FIREBOLT','chase',19,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FIREBOLT','attack',19,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_LIGHTNINGBOLT','chase',20,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_LIGHTNINGBOLT','attack',20,4,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_METEOR','attack',83,11,3000,1500,10000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_STORMGUST','attack',89,10,3000,1500,10000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,10000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,10000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@MG_FROSTDIVER','chase',15,1,10000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2432,'Wizard Kathryne@WZ_JUPITEL','chase',84,28,10000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_KYRIE','idle',73,10,10000,1000,10000,'no','randomtarget','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_KYRIE','chase',73,10,10000,1000,10000,'no','target','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_KYRIE','idle',73,10,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_KYRIE','chase',73,10,10000,1000,10000,'no','friend','friendhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_PNEUMA','idle',25,1,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_INCAGI','attack',29,1,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_INCAGI','chase',29,1,2000,0,240000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_PNEUMA','attack',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_PNEUMA','chase',25,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXDIVINA','attack',76,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXDIVINA','chase',76,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXDIVINA','idle',76,5,10000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXDIVINA','attack',76,5,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXDIVINA','chase',76,5,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXAETERNA','attack',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@PR_LEXAETERNA','chase',78,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@NPC_HOLYATTACK','attack',189,10,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','attack',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','attack',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','idle',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','idle',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','chase',28,11,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2433,'Priest Margaretha@AL_HEAL','chase',28,11,3000,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_SHIELDCHARGE','attack',250,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_SHIELDBOOMERANG','idle',251,5,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_SHIELDBOOMERANG','chase',251,5,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_SHIELDBOOMERANG','follow',251,5,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_DEFENDER','idle',257,3,10000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_DEFENDER','chase',257,3,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_DEFENDER','attack',257,3,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_AUTOGUARD','chase',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_AUTOGUARD','attack',249,10,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_REFLECTSHIELD','chase',252,5,10000,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_GRANDCROSS','attack',254,10,2000,500,2500,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@CR_HOLYCROSS','attack',253,10,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@AL_HEAL','idle',28,5,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2434,'Crusader Egnigem@AL_HEAL','chase',28,5,3000,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@TF_THROWSTONE','chase',152,1,10000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@TF_THROWSTONE','follow',152,1,10000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@NPC_POISON','attack',176,5,1000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_STRIPWEAPON','attack',215,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_STRIPARMOR','attack',217,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_STRIPSHIELD','attack',216,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'17',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_STRIPHELM','attack',218,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_INTIMIDATE','attack',219,10,2000,0,60000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2435,'Rogue Wickebine@NPC_POISONATTACK','attack',188,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@ITM_TOMAHAWK','idle',337,1,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@ITM_TOMAHAWK','chase',337,1,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@ITM_TOMAHAWK','follow',337,1,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@BS_ADRENALINE','attack',111,10,2000,0,150000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@MC_MAMMONITE','attack',42,22,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@AM_DEMONSTRATION','chase',229,5,5000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@AM_DEMONSTRATION','attack',229,5,5000,0,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@AM_POTIONPITCHER','idle',231,4,10000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@AM_POTIONPITCHER','attack',231,4,10000,0,0,'yes','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2436,'Alchemist Armeyer@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@BA_FROSTJOKER','idle',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@BA_FROSTJOKER','chase',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@BA_FROSTJOKER','follow',318,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2437,'Bard Kavach@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@NPC_RUN','follow',354,1,10000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@DC_SCREAM','idle',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@DC_SCREAM','chase',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@DC_SCREAM','follow',326,3,100,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2438,'Dancer Josephina@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@MG_SOULSTRIKE','idle',13,10,10000,0,1000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@MG_SOULSTRIKE','attack',13,10,5000,0,1000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@MG_SOULSTRIKE','chase',13,10,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@MG_SAFETYWALL','attack',12,10,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@SA_SPELLBREAKER','attack',277,5,1000,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@SA_DISPELL','attack',289,1,200,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@MG_THUNDERSTORM','attack',21,20,1000,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@SA_LANDPROTECTOR','attack',288,1,3000,500,10000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@SA_LANDPROTECTOR','chase',288,1,3000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2439,'Sage Laurell@SA_LANDPROTECTOR','chase',288,1,3000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_FINGEROFFENSIVE','idle',267,3,10000,0,0,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_FINGEROFFENSIVE','chase',267,3,10000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_FINGEROFFENSIVE','follow',267,3,10000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_TRIPLEATTACK','attack',263,5,1000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_INVESTIGATE','attack',266,5,1000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_BODYRELOCATION','walk',264,1,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_EXTREMITYFIST','attack',271,5,100,1500,15000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2440,'Monk Errende@MO_EXTREMITYFIST','attack',271,5,50,1500,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_AGIUP','attack',350,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_HELLJUDGEMENT','attack',662,5,10000,700,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2428,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2441,'The Last One@NPC_SUMMONSLAVE','idle',196,1,10000,700,10000,'no','self','slavele','3',2428,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_WIDESILENCE','attack',663,5,10000,1000,20000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_WIDESILENCE','chase',663,5,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_STORMGUST','attack',89,10,2000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_STORMGUST','chase',89,10,10000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_EARTHQUAKE','attack',653,5,10000,1500,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_EARTHQUAKE','chase',653,5,10000,1500,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@MG_SAFETYWALL','attack',12,10,1000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@HW_GANBANTEIN','attack',483,1,10000,500,7000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_WIDESLEEP','attack',668,5,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_WIDESLEEP','chase',668,5,10000,1000,20000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_CALLSLAVE','attack',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_CALLSLAVE','idle',352,1,10000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_SUMMONSLAVE','attack',196,1,10000,700,10000,'no','self','slavele','3',2434,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@NPC_SUMMONSLAVE','idle',196,1,10000,700,10000,'no','self','slavele','3',2434,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_JUPITEL','attack',84,28,5000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_JUPITEL','chase',84,28,5000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_JUPITEL','attack',84,28,5000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2442,'King of the Alley@WZ_JUPITEL','chase',84,28,5000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_SPEARBOOMERANG','idle',59,5,10000,0,0,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_SPEARBOOMERANG','chase',59,5,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_SPEARBOOMERANG','follow',59,5,10000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_TWOHANDQUICKEN','attack',60,30,10000,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_BOWLINGBASH','attack',62,5,1000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@SM_BASH','attack',5,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@KN_PIERCE','attack',56,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@SM_MAGNUM','attack',7,25,1000,0,0,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@SM_PROVOKE','attack',6,5,1000,0,300000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@SM_PROVOKE','chase',6,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@NPC_STUNATTACK','attack',179,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2443,'Aira@NPC_FIREATTACK','attack',186,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@MG_SOULSTRIKE','idle',13,10,10000,0,1000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@MG_SOULSTRIKE','attack',13,10,5000,0,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@MG_SOULSTRIKE','chase',13,10,3000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@MG_SAFETYWALL','attack',12,10,1000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@SA_SPELLBREAKER','attack',277,5,1000,0,10000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@SA_DISPELL','attack',289,1,200,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@MG_THUNDERSTORM','attack',21,20,1000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@SA_LANDPROTECTOR','attack',288,1,3000,500,10000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@SA_LANDPROTECTOR','chase',288,1,3000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2444,'Kuluna@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@ITM_TOMAHAWK','idle',337,1,10000,0,0,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@ITM_TOMAHAWK','chase',337,1,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@ITM_TOMAHAWK','follow',337,1,10000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@BS_ADRENALINE','attack',111,10,2000,0,150000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@MC_MAMMONITE','attack',42,22,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@AM_DEMONSTRATION','chase',229,5,5000,0,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@AM_DEMONSTRATION','attack',229,5,5000,0,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@AM_POTIONPITCHER','idle',231,4,10000,0,0,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@AM_POTIONPITCHER','attack',231,4,10000,0,0,'no','friend','friendhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2445,'Mallina@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@AC_DOUBLE','attack',46,10,2000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@AC_SHOWER','attack',47,10,2000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@AC_CHARGEARROW','attack',148,1,2000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@NPC_RUN','follow',354,1,10000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'22',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_SKIDTRAP','idle',115,5,2000,0,60000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_SKIDTRAP','attack',115,5,500,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_FLASHER','idle',120,5,2000,0,60000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_FLASHER','attack',120,5,500,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_FREEZINGTRAP','idle',121,5,2000,0,60000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@HT_FREEZINGTRAP','attack',121,5,500,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@NPC_STUNATTACK','attack',179,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2446,'Ezella@NPC_WINDATTACK','attack',187,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@BS_ADRENALINE','attack',111,10,2000,0,150000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@BS_MAXIMIZE','attack',114,1,1000,0,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@MC_MAMMONITE','attack',42,22,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@BS_HAMMERFALL','chase',110,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@BS_HAMMERFALL','attack',110,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@NPC_ARMORBRAKE','attack',344,10,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@NPC_FIREATTACK','attack',186,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2447,'Lune@NPC_WATERATTACK','attack',184,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_CLOAKING','attack',135,3,2000,200,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_CLOAKING','idle',135,3,2000,200,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_CLOAKING','chase',135,3,2000,200,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_GRIMTOOTH','idle',137,5,10000,0,0,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_GRIMTOOTH','chase',137,5,10000,0,0,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_SONICBLOW','attack',136,10,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_GRIMTOOTH','chase',137,5,2000,0,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_VENOMDUST','idle',140,10,2000,0,50000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@AS_VENOMDUST','attack',140,10,2000,0,50000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@NPC_STUNATTACK','attack',179,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2448,'Morin@NPC_POISONATTACK','attack',188,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@TF_THROWSTONE','chase',152,1,10000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@TF_THROWSTONE','follow',152,1,10000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@NPC_POISON','attack',176,5,1000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_STRIPWEAPON','attack',215,5,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_STRIPARMOR','attack',217,5,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_STRIPSHIELD','attack',216,5,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'17',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_STRIPHELM','attack',218,5,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_INTIMIDATE','attack',219,10,2000,0,60000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2449,'Nasarin@NPC_POISONATTACK','attack',188,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Old Glast Heim
REPLACE INTO `mob_skill_db_re` VALUES (2464,'Corrupted Steward@NPC_UNDEADATTACK','chase',347,3,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2464,'Corrupted Steward@AL_HEAL','idle',28,1,5000,1000,3000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2464,'Corrupted Steward@NPC_UNDEADATTACK','attack',347,3,5000,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2464,'Corrupted Steward@NPC_BLOODDRAIN','attack',199,1,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2465,'Corrupted Monk@NPC_CURSEATTACK','chase',181,5,5000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2465,'Corrupted Monk@NPC_BLINDATTACK','attack',177,5,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2465,'Corrupted Monk@NPC_CURSEATTACK','attack',181,5,5000,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2465,'Corrupted Monk@NPC_ENERGYDRAIN','attack',200,1,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2466,'Grand Chamberlain in pain@NPC_UNDEADATTACK','chase',347,4,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2466,'Grand Chamberlain in pain@AL_HEAL','idle',28,1,5000,1000,3000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2466,'Grand Chamberlain in pain@NPC_BLEEDING','attack',660,4,500,900,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2466,'Grand Chamberlain in pain@NPC_UNDEADATTACK','attack',347,4,5000,500,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2466,'Grand Chamberlain in pain@NPC_BLOODDRAIN','attack',199,2,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2467,'Maggot@NPC_GROUNDATTACK','attack',185,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2467,'Maggot@NPC_EMOTION','chase',197,1,200,0,5000,'no','self','always','0',19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2467,'Maggot@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2468,'Corrupted Palace Guard@SM_BASH','chase',5,8,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2468,'Corrupted Palace Guard@BS_MAXIMIZE','chase',114,5,5000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2468,'Corrupted Palace Guard@CR_AUTOGUARD','chase',249,2,2000,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2468,'Corrupted Palace Guard@SM_MAGNUM','attack',7,9,5000,1000,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2468,'Corrupted Palace Guard@NPC_DARKNESSATTACK','attack',190,5,5000,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2469,'Wandering Archer@AC_DOUBLE','chase',46,5,5000,1000,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2469,'Wandering Archer@AC_CHARGEARROW','attack',148,1,5000,0,50000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2469,'Wandering Archer@NPC_DARKNESSATTACK','attack',190,5,5000,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2469,'Wandering Archer@AC_DOUBLE','attack',46,9,5000,1800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@NPC_DARKSTRIKE','idle',340,3,5000,0,10000,'no','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@CR_AUTOGUARD','chase',249,10,2000,0,300000,'no','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@AC_CHARGEARROW','chase',148,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@KN_BRANDISHSPEAR','attack',57,10,500,2000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@NPC_DARKNESSATTACK','attack',190,9,500,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2470,'Corrupted Knight of Abyss@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2472,'Bloody Knight@WZ_METEOR','chase',83,7,2000,0,10000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2472,'Bloody Knight@WZ_METEOR','attack',83,7,2000,0,5000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2472,'Bloody Knight@KN_BRANDISHSPEAR','attack',57,7,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2472,'Bloody Knight@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2472,'Bloody Knight@NPC_PULSESTRIKE','attack',661,3,500,2000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_EARTHQUAKE','idle',653,2,10000,5000,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_EVILLAND','idle',670,10,7000,3000,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_HELLJUDGEMENT','idle',662,5,10000,5000,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_VAMPIRE_GIFT','idle',679,5,2000,5000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_BLOODDRAIN','idle',199,1,2000,0,2000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@AM_DEMONSTRATION','idle',229,9,10000,0,2000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_HELLJUDGEMENT','idle',662,5,10000,5000,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@AL_HEAL','idle',28,11,10000,0,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_POWERUP','idle',349,5,10000,1000,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_EARTHQUAKE','idle',653,4,10000,5000,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_EVILLAND','idle',670,10,10000,1000,5000,'yes','randomtarget','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_HELLJUDGEMENT','idle',662,10,10000,5000,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_HELLJUDGEMENT','idle',662,8,10000,5000,0,'yes','self','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_HELLJUDGEMENT','idle',662,6,10000,5000,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDESOULDRAIN','idle',680,5,10000,1000,0,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDESOULDRAIN','idle',680,5,10000,1000,0,'yes','self','skillused','271',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_DARKSTRIKE','attack',340,10,5000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_BLOODDRAIN','attack',199,1,2000,1000,0,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_CRITICALWOUND','attack',673,4,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@AS_SONICBLOW','attack',136,10,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDECURSE','attack',677,5,10000,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDESILENCE','attack',663,5,8000,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDESTONE','attack',666,3,10000,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2475,'Corrupted Soul@NPC_WIDESTUN','attack',678,3,5000,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_DARKSTRIKE','idle',340,10,10000,0,0,'no','target','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_DARKSTRIKE','chase',340,5,5000,0,1000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_HELLJUDGEMENT','chase',662,5,5000,4000,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_DRAGONFEAR','chase',659,5,10000,0,0,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_DRAGONFEAR','attack',659,5,2000,1000,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_HELLJUDGEMENT','attack',662,5,10000,4000,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_GRANDDARKNESS','attack',339,10,10000,2000,1000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_POWERUP','attack',349,5,10000,0,50000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_VAMPIRE_GIFT','attack',679,5,10000,0,0,'no','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_CRITICALWOUND','attack',673,4,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_CHANGEUNDEAD','attack',348,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_MENTALBREAKER','attack',159,4,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_CRITICALSLASH','attack',170,1,10000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_DARKCROSS','attack',338,10,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_UNDEADATTACK','attack',347,10,10000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_CURSEATTACK','attack',181,5,4000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@NPC_STUNATTACK','attack',179,5,2000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2476,'Amdarias@SM_MAGNUM','attack',7,25,10000,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Nightmare Glast Heim
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@NJ_ISSEN','attack',544,10,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@NPC_BLEEDING','idle',660,5,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2477,'Wanderer (Nightmare)@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2478,'Rideword (Nightmare)@NPC_PIERCINGATT','attack',158,3,500,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2478,'Rideword (Nightmare)@NPC_BLINDATTACK','attack',177,3,500,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2478,'Rideword (Nightmare)@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2479,'Mimic (Nightmare)@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2479,'Mimic (Nightmare)@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2479,'Mimic (Nightmare)@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2479,'Mimic (Nightmare)@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2479,'Mimic (Nightmare)@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_CURSEATTACK','chase',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_UNDEADATTACK','chase',347,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2480,'Evil Druid (Nightmare)@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2481,'Wraith Dead (Nightmare)@NPC_POISON','attack',176,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2481,'Wraith Dead (Nightmare)@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2481,'Wraith Dead (Nightmare)@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2481,'Wraith Dead (Nightmare)@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2481,'Wraith Dead (Nightmare)@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@AL_HEAL','idle',28,10,10000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@KN_BRANDISHSPEAR','attack',57,10,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@WZ_VERMILION','attack',85,10,2000,500,2000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@WZ_VERMILION','chase',85,10,2000,500,2000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@WZ_VERMILION','chase',85,10,5000,500,2000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_DARKBREATH','attack',202,5,2000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@MO_BODYRELOCATION','chase',264,1,2000,200,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_DARKSTRIKE','chase',340,10,2000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_ARMORBRAKE','attack',344,10,2000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_EARTHQUAKE','attack',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_EARTHQUAKE','chase',653,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_HELLJUDGEMENT','attack',662,5,10000,800,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_HELLJUDGEMENT','chase',662,5,10000,800,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_CALLSLAVE','attack',352,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_CALLSLAVE','idle',352,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_SUMMONSLAVE','attack',196,6,10000,700,10000,'no','self','slavele','3',2484,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_SUMMONSLAVE','idle',196,6,10000,700,10000,'no','self','slavele','3',2484,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2483,'Baphomet (Nightmare)@NPC_SUMMONSLAVE','idle',196,4,10000,0,0,'no','self','onspawn','0',2484,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@SA_DISPELL','attack',289,1,0,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x0081,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2484,'Baphomet Jr. (Nightmare)@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@NPC_FIREATTACK','attack',186,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@NPC_SHIELDBRAKE','attack',346,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2485,'Chimera (Nightmare)@NPC_SUMMONSLAVE','idle',196,4,10000,2000,60000,'yes','self','slavele','0',1253,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (2528,'Faceworm@NPC_POISON','attack',176,4,10000,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2528,'Faceworm@NPC_POISONATTACK','attack',188,2,10000,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@AS_VENOMDUST','idle',140,5,10000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_EARTHQUAKE','attack',653,1,10000,5000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@RG_CLOSECONFINE','chase',1005,1,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_DRAGONFEAR','attack',659,5,10000,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_DRAGONFEAR','chase',659,5,10000,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_ACIDBREATH','chase',657,10,10000,5000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@KN_PIERCE','chase',56,10,10000,500,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@KN_PIERCE','attack',56,10,10000,500,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_CRITICALWOUND','attack',673,3,10000,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2529,'Faceworm Queen@NPC_CRITICALWOUND','chase',673,3,10000,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@AS_VENOMDUST','idle',140,3,10000,1000,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@AS_VENOMDUST','attack',140,3,10000,1000,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@AS_VENOMDUST','chase',140,3,10000,1000,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@NPC_DRAGONFEAR','attack',659,3,10000,1000,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@NPC_DRAGONFEAR','chase',659,3,10000,1000,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@NPC_ACIDBREATH','chase',657,5,10000,5000,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@KN_PIERCE','chase',56,5,10000,500,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@KN_PIERCE','attack',56,5,10000,500,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@NPC_CRITICALWOUND','attack',673,1,10000,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2530,'Faceworm Dark@NPC_CRITICALWOUND','chase',673,1,10000,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2531,'Venom Bug@NPC_SELFDESTRUCTION','idle',173,1,10000,5000,0,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2531,'Venom Bug@NPC_SELFDESTRUCTION','attack',173,1,10000,5000,0,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2531,'Venom Bug@NPC_SELFDESTRUCTION','chase',173,1,10000,5000,0,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_POWERUP','chase',349,5,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_AGIUP','attack',350,10,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_AGIUP','chase',350,10,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_PULSESTRIKE','chase',661,10,1000,3000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_PULSESTRIKE','attack',661,10,1000,3000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_EARTHQUAKE','chase',653,5,10000,3000,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_EARTHQUAKE','attack',653,5,10000,3000,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_SLOWCAST','attack',672,5,10000,500,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_HELLJUDGEMENT','attack',662,5,10000,3000,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@MO_BODYRELOCATION','chase',264,1,10000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_DARKSTRIKE','attack',340,10,5000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2532,'Red Faceworm Queen@NPC_DARKSTRIKE','chase',340,10,5000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@WZ_HEAVENDRIVE','attack',91,5,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@WZ_HEAVENDRIVE','chase',91,5,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@NPC_WIDESTUN','chase',678,5,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@NPC_WIDESTUN','attack',678,5,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@AL_HEAL','attack',28,9,10000,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@AL_HEAL','chase',28,9,10000,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2533,'Green Faceworm Queen@NPC_GROUNDATTACK','attack',185,9,3000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@NPC_WIDEFREEZE','chase',664,5,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@NPC_WIDEFREEZE','attack',664,5,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@WZ_STORMGUST','chase',89,10,2000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@WZ_STORMGUST','attack',89,10,2000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@NPC_WATERATTACK','attack',184,9,3000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2534,'Blue Faceworm Queen@MG_FROSTDIVER','chase',15,10,10000,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@WZ_VERMILION','attack',85,10,2000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@WZ_VERMILION','chase',85,10,5000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@NPC_WIDEBLEEDING','chase',665,3,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@NPC_WIDEBLEEDING','attack',665,3,10000,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@WZ_JUPITEL','attack',84,9,1000,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@WZ_JUPITEL','chase',84,9,1000,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@NPC_WINDATTACK','attack',187,9,3000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2535,'Yellow Faceworm Queen@MG_THUNDERSTORM','attack',21,10,2000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2536,'Monster 3@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2536,'Monster 3@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2536,'Monster 3@NPC_VENOMFOG','idle',706,1,10000,0,3000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2536,'Monster 3@NPC_VENOMFOG','attack',706,1,10000,0,3000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2537,'Monster 4@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2537,'Monster 4@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2537,'Monster 4@NPC_VENOMFOG','idle',706,3,10000,0,3000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2537,'Monster 4@NPC_VENOMFOG','attack',706,3,10000,0,3000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2539,'Monster 5@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2539,'Monster 5@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2539,'Monster 5@WZ_WATERBALL','idle',86,5,10000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2539,'Monster 5@WZ_WATERBALL','attack',86,5,10000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2540,'Faceworm Egg@NPC_SUMMONMONSTER','idle',209,1,10000,10000,10000,'no','self','always','0',2541,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2541,'Faceworm Larva@NPC_BLEEDING','attack',660,5,2000,1000,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2541,'Faceworm Larva@NPC_POISON','attack',176,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2541,'Faceworm Larva@NPC_POISONATTACK','attack',188,10,2000,2000,2000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Heroes' Trail Part 1 - Sara's Memory
# Not Yet Implemented skill
#REPLACE INTO `mob_skill_db_re` VALUES (2542,'Irene Elder@NPC_MILLENNIUMSHIELD','idle',707,1,5000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2542,'Irene Elder@NPC_MILLENNIUMSHIELD','attack',707,1,5000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2542,'Irene Elder@NPC_MILLENNIUMSHIELD','chase',707,1,5000,1000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

#Geffen Magic Tournament
REPLACE INTO `mob_skill_db_re` VALUES (2549,'Arhi@MG_FIREBOLT','attack',19,5,3000,1000,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2549,'Arhi@MG_FIREWALL','attack',18,1,3000,2000,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2550,'Dio Anemos@MG_FROSTDIVER','attack',15,5,3000,1000,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2550,'Dio Anemos@MG_THUNDERSTORM','attack',21,3,3000,2000,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2551,'Geffen Shoplifter@NPC_COMBOATTACK','attack',171,2,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2551,'Geffen Shoplifter@MG_STONECURSE','attack',16,5,5000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2551,'Geffen Shoplifter@TF_POISON','attack',52,3,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2552,'Geffen Bully@BS_HAMMERFALL','attack',110,5,5000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2552,'Geffen Bully@NPC_POWERUP','attack',349,5,5000,1000,25000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2553,'Geffen Gang Member@ASC_BREAKER','attack',379,1,3000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2553,'Geffen Gang Member@AS_VENOMDUST','attack',140,10,3000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2554,'Faymont@AL_INCAGI','idle',29,10,10000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2554,'Faymont@AL_INCAGI','attack',29,10,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2554,'Faymont@AL_DECAGI','attack',30,48,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2554,'Faymont@NPC_COMBOATTACK','attack',171,3,5000,0,3000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2555,'Ordre@MG_COLDBOLT','attack',14,5,3000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2555,'Ordre@WZ_STORMGUST','attack',89,1,3000,3000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2555,'Ordre@WZ_FROSTNOVA','attack',88,5,3000,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2556,'Blut Hase@CR_AUTOGUARD','attack',249,5,3000,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2556,'Blut Hase@CR_GRANDCROSS','attack',254,5,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2557,'Kuro Akuma@NPC_BLEEDING','attack',660,5,5000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2557,'Kuro Akuma@NPC_WIDESOULDRAIN','attack',680,3,3000,500,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2557,'Kuro Akuma@NPC_COMBOATTACK','attack',171,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2557,'Kuro Akuma@NPC_INVISIBLE','attack',353,1,2000,1000,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2558,'Ifodes@LK_SPIRALPIERCE','attack',397,5,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2558,'Ifodes@SM_ENDURE','attack',8,10,3000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2558,'Ifodes@SM_MAGNUM','attack',7,25,3000,500,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2558,'Ifodes@NPC_MILLENNIUMSHIELD','attack',707,1,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2559,'Licheniyes@NPC_HOLYATTACK','attack',189,10,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2559,'Licheniyes@AL_HEAL','attack',28,8,3000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2559,'Licheniyes@AL_PNEUMA','attack',25,1,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2560,'Odoric@NPC_MENTALBREAKER','attack',159,4,2000,1500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2560,'Odoric@AC_DOUBLE','attack',46,5,3000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2560,'Odoric@HT_ANKLESNARE','attack',117,1,2000,1000,10000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2560,'Odoric@HT_FLASHER','attack',120,1,2000,1000,10000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@MG_FIREBOLT','attack',19,5,1000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@MG_COLDBOLT','attack',14,5,1000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@MG_LIGHTNINGBOLT','attack',20,5,1000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@WZ_EARTHSPIKE','attack',90,5,1000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@PF_SPIDERWEB','attack',405,1,1000,1000,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2561,'Ju@RG_STRIPSHIELD','attack',216,5,3000,1000,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2562,'Dwigh@NPC_SLOWCAST','attack',672,5,1000,1000,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2562,'Dwigh@MG_SOULSTRIKE','attack',13,10,1000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2562,'Dwigh@WZ_FIREPILLAR','attack',80,5,3000,500,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2562,'Dwigh@WZ_VERMILION','attack',85,5,1000,2000,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2563,'Fei Kanabian@AM_DEMONSTRATION','attack',229,5,2000,500,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2563,'Fei Kanabian@AM_ACIDTERROR','attack',230,5,2000,1000,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2563,'Fei Kanabian@AM_POTIONPITCHER','attack',231,4,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2564,'Geffen Fenrir@NPC_SHIELDBRAKE','attack',346,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2564,'Geffen Fenrir@WL_EARTHSTRAIN','attack',2216,5,3000,3000,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES (2564,'Geffen Fenrir@NPC_COMET','attack',708,5,2000,5000,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2564,'Geffen Fenrir@WL_TETRAVORTEX','attack',2217,5,1000,5000,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2565,'Alphonse@SM_BASH','attack',5,5,1000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2566,'Alphonse Jr@PR_LEXDIVINA','attack',76,1,1000,2000,60000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2566,'Alphonse Jr@PR_LEXAETERNA','attack',78,1,1000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Champion Monsters
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@AS_SONICBLOW','attack',136,5,1000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2603,'Swift Zombie Slaughter@NPC_POISON','attack',176,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2604,'Solid Zombie Prisoner@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2604,'Solid Zombie Prisoner@NPC_POISONATTACK','attack',188,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2604,'Solid Zombie Prisoner@NPC_UNDEADATTACK','attack',347,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@KN_BRANDISHSPEAR','attack',57,5,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1298,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2605,'Zombie Master Ringleader@NPC_UNDEADATTACK','attack',347,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2606,'Furious Zombie@NPC_POISON','angry',176,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2606,'Furious Zombie@NPC_POISON','attack',176,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2606,'Furious Zombie@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2606,'Furious Zombie@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2607,'Elusive Zakudam@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2607,'Elusive Zakudam@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2607,'Elusive Zakudam@NPC_GUIDEDATTACK','attack',172,2,500,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2607,'Elusive Zakudam@NPC_PIERCINGATT','attack',158,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2607,'Elusive Zakudam@NPC_RANDOMATTACK','attack',183,5,1000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@AM_POTIONPITCHER','idle',231,1,500,1500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@SM_PROVOKE','chase',6,10,500,600,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2608,'Swift Yoyo@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@AM_POTIONPITCHER','idle',231,1,500,1500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@SM_PROVOKE','chase',6,10,500,600,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2609,'Solid Yoyo@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2610,'Wormtail Ringleader@NPC_GROUNDATTACK','attack',185,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2610,'Wormtail Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1024,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2611,'Furious Wootan Fighter@KN_TWOHANDQUICKEN','attack',60,9,2000,0,120000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2611,'Furious Wootan Fighter@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2612,'Elusive Leshij@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2612,'Elusive Leshij@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2612,'Elusive Leshij@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2612,'Elusive Leshij@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2612,'Elusive Leshij@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2613,'Swift Wolf@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2613,'Swift Wolf@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2613,'Swift Wolf@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2613,'Swift Wolf@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@HT_SHOCKWAVE','idle',118,5,500,0,300000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@MG_LIGHTNINGBOLT','attack',20,5,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@MG_LIGHTNINGBOLT','chase',20,5,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@NPC_UNDEADATTACK','attack',347,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2614,'Solid Wind Ghost@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2615,'Willow Ringleader@NPC_GROUNDATTACK','attack',185,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2615,'Willow Ringleader@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2615,'Willow Ringleader@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2615,'Willow Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1010,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2616,'Furious Wild Rose@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2616,'Furious Wild Rose@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2616,'Furious Wild Rose@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','attack',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','idle',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','chase',28,3,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','attack',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','idle',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@AL_HEAL','chase',28,3,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@NPC_STOP','attack',342,1,500,0,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2617,'Elusive Evil Nymph@SA_DISPELL','attack',289,5,0,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,28,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2618,'Solid Whisper@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2619,'Weak Skeleton Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',2405,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@NPC_BLEEDING','attack',660,1,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2620,'Furious Wanderer@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2621,'Elusive Violy@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2621,'Elusive Violy@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2621,'Elusive Violy@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2621,'Elusive Violy@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2622,'Swift Violy@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2623,'Solid Violy@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2623,'Solid Violy@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2623,'Solid Violy@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2623,'Solid Violy@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_POISON','attack',176,5,1000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_SELFDESTRUCTION','chase',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','skillused','173',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2624,'Venomous Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1621,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@NPC_EMOTION','chase',197,1,100,0,5000,'yes','self','always',NULL,7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,NULL,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2625,'Furious Venatu@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2626,'Elusive Baba-Yaga@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2626,'Elusive Baba-Yaga@NPC_COMBOATTACK','attack',171,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2626,'Elusive Baba-Yaga@WZ_WATERBALL','attack',86,2,2000,3000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2626,'Elusive Baba-Yaga@WZ_WATERBALL','chase',86,1,5000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@BS_MAXIMIZE','attack',114,5,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@BS_MAXIMIZE','chase',114,5,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@CR_HOLYCROSS','attack',253,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2627,'Swift Vanberk@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2628,'Solid Vadon@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2628,'Solid Vadon@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2628,'Solid Vadon@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2628,'Solid Vadon@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@AL_TELEPORT','attack',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@NPC_DARKBREATH','attack',202,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@NPC_PETRIFYATTACK','attack',180,2,1000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@NPC_POISONATTACK','attack',188,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2629,'Ungoliant Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1618,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2630,'Furious Toucan@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2631,'Elusive Tiyanak@NPC_REBIRTH','dead',208,3,2000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2632,'Solid Thief Bug@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2632,'Solid Thief Bug@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2633,'Thief Bug Ringleader@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2633,'Thief Bug Ringleader@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2633,'Thief Bug Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1051,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2634,'Furious Thief Bug@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2634,'Furious Thief Bug@NPC_AGIUP','loot',350,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2635,'Elusive Thara Frog@NPC_LICK','attack',206,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2635,'Elusive Thara Frog@NPC_WATERATTACK','attack',184,3,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_CURSEATTACK','attack',181,4,500,800,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_CURSEATTACK','chase',181,4,500,800,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_EMOTION','idle',197,1,10000,0,30000,'yes','self','always',NULL,23,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_SELFDESTRUCTION','attack',173,1,1000,2000,10000,'no','self','attackpcge','3',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@NPC_SELFDESTRUCTION','chase',173,1,1000,2000,10000,'no','self','attackpcge','3',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2636,'Swift Teddy Bear@SA_DISPELL','attack',289,1,50,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@HW_GANBANTEIN','attack',483,1,2000,0,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2637,'Solid Tatacho@NPC_STUNATTACK','attack',179,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2638,'Tarou Ringleader@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2638,'Tarou Ringleader@NPC_POISON','attack',176,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2638,'Tarou Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1175,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2639,'Furious Tamruan@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2639,'Furious Tamruan@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2639,'Furious Tamruan@NPC_SILENCEATTACK','attack',178,5,2000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@AM_POTIONPITCHER','idle',231,4,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@KN_SPEARSTAB','attack',58,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,20,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'10',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@WZ_QUAGMIRE','attack',92,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2640,'Elusive Sting@WZ_QUAGMIRE','chase',92,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2641,'Swift Stem Worm@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2641,'Swift Stem Worm@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2641,'Swift Stem Worm@NPC_GUIDEDATTACK','attack',172,1,500,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2641,'Swift Stem Worm@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2642,'Solid Steel Chonchon@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2642,'Solid Steel Chonchon@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2643,'Stapo Ringleader@NPC_KEEPING','attack',201,1,1000,1000,5000,'no','self','myhpltmaxrate','25',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2643,'Stapo Ringleader@SM_BASH','attack',5,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2643,'Stapo Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1784,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2644,'Furious Stalactic Golem@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2645,'Elusive Stainer@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2645,'Elusive Stainer@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2645,'Elusive Stainer@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@NPC_SLEEPATTACK','attack',182,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2646,'Swift Spore@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2647,'Solid Solider@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2648,'Soldier Skeleton Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1028,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2649,'Furious Soldier Skeleton@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@CR_GRANDCROSS','attack',254,1,1000,1500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@MG_NAPALMBEAT','attack',11,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@MG_NAPALMBEAT','chase',11,5,1000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@PR_LEXAETERNA','chase',78,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@PR_LEXDIVINA','attack',76,10,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2650,'Elusive Lady Solace@PR_LEXDIVINA','chase',76,10,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@NPC_SUICIDE','attack',175,1,500,0,5000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2651,'Swift Sohee@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@NPC_WATERATTACK','attack',184,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@NPC_WATERATTACK','attack',184,4,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@SM_BASH','attack',5,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2652,'Solid Snowier@SM_ENDURE','chase',8,3,5000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2653,'Boa Ringleader@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2653,'Boa Ringleader@NPC_POISON','attack',176,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2653,'Boa Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1025,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2654,'Furious Smokie@AL_HEAL','idle',28,5,10000,0,5000,'yes','self','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2654,'Furious Smokie@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2654,'Furious Smokie@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2654,'Furious Smokie@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2654,'Furious Smokie@TF_HIDING','attack',51,1,500,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@AS_CLOAKING','idle',135,1,100,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_DEFENDER','angry',205,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_DEFENDER','follow',205,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_DEFENDER','attack',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_SLEEPATTACK','angry',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2655,'Elusive Sleeper@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AS_CLOAKING','idle',135,1,100,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_DEFENDER','angry',205,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_DEFENDER','follow',205,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_DEFENDER','attack',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_SLEEPATTACK','angry',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2656,'Swift Sleeper@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@AS_GRIMTOOTH','idle',137,5,10000,0,700,'yes','target','mystatuson','hiding',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@MG_SOULSTRIKE','chase',13,5,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@NPC_CURSEATTACK','attack',181,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@NPC_DARKNESSATTACK','attack',190,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@NPC_SUMMONSLAVE','attack',196,1,100,0,3600000,'no','self','slavele','0',1762,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2657,'Solid Skogul@TF_HIDING','idle',51,1,2000,0,180000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1290,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2658,'Skeleton General Ringleader@NPC_UNDEADATTACK','attack',347,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1028,1016,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2659,'Furious Skeleton General@NPC_UNDEADATTACK','attack',347,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2660,'Elusive Skeleton Worker@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2660,'Elusive Skeleton Worker@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2660,'Elusive Skeleton Worker@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,1,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,45,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','myhpltmaxrate','1',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','myhpltmaxrate','1',28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_SILENCEATTACK','chase',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@NPC_WATERATTACK','attack',184,2,2000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2661,'Swift Siroma@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2662,'Solid Side Winder@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2662,'Solid Side Winder@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2662,'Solid Side Winder@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2663,'Side Winder Ringleader@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2663,'Side Winder Ringleader@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2663,'Side Winder Ringleader@NPC_POISONATTACK','attack',188,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2663,'Side Winder Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1037,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@AS_CLOAKING','idle',135,1,500,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@HT_CLAYMORETRAP','idle',123,5,500,0,300000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@NPC_REBIRTH','dead',208,2,5000,0,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@SM_BASH','attack',5,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2664,'Furious Shinobi@TF_BACKSLIDING','attack',150,1,10000,500,3600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@AS_VENOMDUST','attack',140,1,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@AS_VENOMDUST','idle',140,1,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_BLINDATTACK','angry',177,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_BLINDATTACK','follow',177,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_DARKNESSATTACK','chase',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_DARKTHUNDER','attack',341,4,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2665,'Elusive Mistress of Shelter@NPC_DARKTHUNDER','chase',341,4,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2666,'Swift Shellfish@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2666,'Swift Shellfish@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2666,'Swift Shellfish@NPC_WATERATTACK','attack',184,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@AC_CHARGEARROW','attack',148,1,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@AC_DOUBLE','attack',46,10,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@AC_SHOWER','attack',47,10,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_FLASHER','attack',120,5,500,0,5000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_FLASHER','idle',120,5,2000,0,60000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_FREEZINGTRAP','attack',121,5,500,0,5000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_FREEZINGTRAP','idle',121,5,2000,0,60000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_SKIDTRAP','attack',115,5,500,0,5000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@HT_SKIDTRAP','idle',115,5,2000,0,60000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_AGIUP','attack',350,1,2000,0,25000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_AGIUP','idle',350,1,10000,0,35000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_POWERUP','attack',349,1,2000,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_POWERUP','attack',349,1,2000,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_STUNATTACK','attack',179,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@NPC_WINDATTACK','attack',187,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2667,'Solid Cecil Damon@SN_SHARPSHOOTING','attack',382,5,2000,500,1500,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@NPC_WATERATTACK','attack',184,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@WZ_WATERBALL','attack',86,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@WZ_WATERBALL','chase',86,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2668,'Sea Otter Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1323,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@NPC_BLEEDING','attack',660,1,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,18,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2669,'Furious Lamp Rey@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2670,'Elusive Scorpion@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2670,'Elusive Scorpion@NPC_POISON','attack',176,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2670,'Elusive Scorpion@NPC_POISONATTACK','attack',188,1,500,300,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2671,'Swift Savage Babe@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2671,'Swift Savage Babe@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2671,'Swift Savage Babe@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2671,'Swift Savage Babe@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2672,'Solid Savage@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2672,'Solid Savage@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2672,'Solid Savage@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2673,'Savage Ringleader@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2673,'Savage Ringleader@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2673,'Savage Ringleader@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2673,'Savage Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1166,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@AS_CLOAKING','idle',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@NPC_GROUNDATTACK','angry',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@TF_SPRINKLESAND','angry',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2674,'Furious Sandman@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@AS_SONICBLOW','attack',136,10,1000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@CR_REFLECTSHIELD','idle',252,3,1000,0,200000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@KN_TWOHANDQUICKEN','attack',60,30,2000,0,300000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@MG_FIREBALL','idle',17,5,10000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@MG_FIREBALL','chase',17,5,10000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@NPC_BLEEDING','attack',660,3,500,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@NPC_CRITICALSLASH','attack',170,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@NPC_FIREATTACK','attack',186,5,2000,1000,200000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@NPC_WIDEBLEEDING','attack',665,2,10000,500,100000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2675,'Elusive Salamander@SM_MAGNUM','attack',7,7,1000,0,10000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2677,'Solid Roween@NPC_COMBOATTACK','attack',171,4,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2677,'Solid Roween@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2677,'Solid Roween@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2678,'Roda Frog Ringleader@NPC_WATERATTACK','attack',184,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2678,'Roda Frog Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1012,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2679,'Furious Roda Frog@NPC_WATERATTACK','attack',184,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2680,'Elusive Rocker@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2680,'Elusive Rocker@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2681,'Swift Rideword@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2681,'Swift Rideword@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2681,'Swift Rideword@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2681,'Swift Rideword@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2681,'Swift Rideword@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2682,'Solid Rideword@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2682,'Solid Rideword@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2682,'Solid Rideword@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2683,'Dumpling Child Ringleader@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2683,'Dumpling Child Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1409,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@NPC_DARKCROSS','attack',338,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2684,'Furious Baroness of Retribution@NPC_GRANDDARKNESS','attack',339,1,500,700,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@NPC_DARKCROSS','attack',338,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2685,'Elusive Baroness of Retribution@NPC_GRANDDARKNESS','attack',339,1,500,700,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@NPC_DARKBREATH','attack',202,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@NPC_DARKCROSS','attack',338,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2686,'Swift Baroness of Retribution@NPC_GRANDDARKNESS','attack',339,1,500,700,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2687,'Solid Requiem@NPC_DARKNESSATTACK','attack',190,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2687,'Solid Requiem@NPC_EMOTION','walk',197,1,200,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2687,'Solid Requiem@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2688,'Remover Ringleader@NPC_UNDEADATTACK','attack',347,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2688,'Remover Ringleader@SM_BASH','attack',5,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2688,'Remover Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1682,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2689,'Elusive Red Eruma@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2689,'Elusive Red Eruma@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2689,'Elusive Red Eruma@NPC_WINDATTACK','attack',187,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2690,'Swift Raydric@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@BS_MAXIMIZE','attack',114,5,500,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@BS_MAXIMIZE','chase',114,5,500,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2691,'Solid Raydric@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_FROSTDIVER','chase',15,9,10000,500,20000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_LIGHTNINGBOLT','attack',20,7,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_LIGHTNINGBOLT','idle',20,9,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_LIGHTNINGBOLT','chase',20,7,10000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_SAFETYWALL','attack',12,10,2000,1000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_SOULSTRIKE','attack',13,9,5000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_SOULSTRIKE','attack',13,5,3000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_SOULSTRIKE','attack',13,3,2000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_SOULSTRIKE','idle',13,9,5000,0,1000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2692,'Laurell Weinder Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1657,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_CRITICALWOUND','attack',673,1,2000,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_EMOTION','idle',197,1,3000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2693,'Furious Rake Scaraba@PF_SPIDERWEB','chase',405,1,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2694,'Elusive Raggler@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2694,'Elusive Raggler@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2694,'Elusive Raggler@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2694,'Elusive Raggler@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2696,'Poring Ringleader@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2696,'Poring Ringleader@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2696,'Poring Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1002,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2697,'Furious Poring@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2697,'Furious Poring@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2698,'Elusive Poring@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2698,'Eluisve Poring@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2699,'Swift Poring@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2699,'Swift Poring@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2699,'Swift Poring@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2699,'Swift Poring@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2700,'Solid Porcellio@AL_PNEUMA','attack',25,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2700,'Solid Porcellio@NPC_GROUNDATTACK','attack',185,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2700,'Solid Porcellio@NPC_KEEPING','attack',201,1,2000,1000,5000,'no','self','myhpltmaxrate','25',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2701,'Poporing Ringleader@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2701,'Poporing Ringleader@NPC_POISON','attack',176,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2701,'Poporing Ringleader@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2701,'Poporing Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1031,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2702,'Furious Poporing@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2702,'Furious Poporing@NPC_POISON','attack',176,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2702,'Furious Poporing@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2703,'Elusive Poporing@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2703,'Elusive Poporing@NPC_POISON','attack',176,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2703,'Elusive Poporing@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@NPC_COMBOATTACK','attack',171,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2704,'Swift Poisonous Toad@NPC_POISONATTACK','attack',188,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,1,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_POISON','angry',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_POISON','attack',176,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_POISONATTACK','angry',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2705,'Solid Poison Spore@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@MG_NAPALMBEAT','attack',11,7,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@MG_NAPALMBEAT','attack',11,7,4000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@MG_SOULSTRIKE','attack',13,9,3000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@MG_SOULSTRIKE','chase',13,9,3000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@NPC_STUNATTACK','attack',179,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@NPC_STUNATTACK','chase',179,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'39',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@NPC_TELEKINESISATTACK','attack',191,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2706,'Plasma Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1693,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2707,'Furious Plankton@NPC_WATERATTACK','attack',184,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2708,'Elusive Pitman@HT_CLAYMORETRAP','idle',123,5,500,0,300000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2708,'Elusive Pitman@NPC_COMBOATTACK','attack',171,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2708,'Elusive Pitman@NPC_POWERUP','attack',349,2,3000,500,10000,'no','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2708,'Elusive Pitman@TF_SPRINKLESAND','attack',149,1,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2708,'Elusive Pitman@TF_SPRINKLESAND','chase',149,1,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@MG_COLDBOLT','attack',14,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@MG_COLDBOLT','chase',14,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_CRITICALWOUND','attack',673,1,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@NPC_WATERATTACK','attack',184,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2709,'Swift Piranha@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_BLOODDRAIN','attack',199,1,5000,500,5000,'yes','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2710,'Solid Dark Pinguicula@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@AL_HEAL','idle',28,9,10000,3000,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@AL_HEAL','idle',28,9,5000,500,5000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_SLEEPATTACK','attack',182,3,1000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_SPLASHATTACK','attack',174,1,3000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_STUNATTACK','attack',179,3,1000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2711,'Pinguicula Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1995,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2712,'Furious Picky@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2712,'Furious Picky@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2713,'Elusive Phen@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2713,'Elusive Phen@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2714,'Swift Petite@SM_BASH','attack',5,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2715,'Solid Petite@SM_BASH','attack',5,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2716,'Penomena Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2716,'Penomena Ringleader@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2716,'Penomena Ringleader@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2716,'Penomena Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1216,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2717,'Furious Penomena@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2717,'Furious Penomena@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2717,'Furious Penomena@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2718,'Elusive Peco Peco@AS_SONICBLOW','attack',136,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2718,'Elusive Peco Peco@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2718,'Elusive Peco Peco@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2718,'Elusive Peco Peco@NPC_PROVOCATION','chase',194,1,200,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@NPC_BLEEDING','attack',660,2,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2719,'Swift Pasana@SM_BASH','attack',5,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_ENERGYDRAIN','attack',200,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2720,'Solid Parasite@NPC_POISON','chase',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@MG_LIGHTNINGBOLT','attack',20,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@MG_LIGHTNINGBOLT','chase',20,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@PR_LEXDIVINA','attack',76,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@PR_LEXDIVINA','chase',76,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2721,'Owl Duke Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1320,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,32,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_EMOTION','follow',197,1,2000,0,5000,'yes','self','always',NULL,27,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_GROUNDATTACK','angry',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2722,'Furious Orc Warrior@NPC_SMOKING','idle',195,1,500,0,36000000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2723,'Elusive Orc Zombie@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2723,'Elusive Orc Zombie@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2723,'Elusive Orc Zombie@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2723,'Elusive Orc Zombie@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2723,'Elusive Orc Zombie@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@NPC_UNDEADATTACK','angry',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2724,'Swift Orc Skeleton@NPC_UNDEADATTACK','attack',347,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2725,'Solid Orc Lady@AL_INCAGI','chase',29,1,500,700,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2725,'Solid Orc Lady@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2725,'Solid Orc Lady@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@MG_SAFETYWALL','attack',12,5,1000,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@NPC_HOLYATTACK','attack',189,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@NPC_HOLYATTACK','chase',189,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@RG_INTIMIDATE','attack',219,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@WZ_JUPITEL','attack',84,4,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@WZ_JUPITEL','chase',84,4,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2726,'Dame of Sentinel Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1700,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_MENTALBREAKER','attack',159,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_MENTALBREAKER','chase',159,2,500,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'14',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_SELFDESTRUCTION','chase',173,1,500,2000,5000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_SELFDESTRUCTION','attack',173,1,500,2000,5000,'no','self','skillused','173',NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2727,'Furious Noxious@NPC_TELEKINESISATTACK','attack',191,3,1000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2728,'Elusive Novus@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'37',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2728,'Elusive Novus@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2728,'Elusive Novus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2728,'Elusive Novus@NPC_PIERCINGATT','attack',158,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2728,'Elusive Novus@SM_BASH','attack',5,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'37',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@NPC_PIERCINGATT','attack',158,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2729,'Swift Novus@SM_BASH','attack',5,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2730,'Solid Novus@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'37',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2730,'Solid Novus@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2730,'Solid Novus@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2730,'Solid Novus@NPC_PIERCINGATT','attack',158,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2730,'Solid Novus@SM_BASH','attack',5,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'37',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@NPC_PIERCINGATT','attack',158,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@SM_BASH','attack',5,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2731,'Novus Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1715,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_CURSEATTACK','angry',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_DARKSTRIKE','angry',340,7,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_DARKSTRIKE','attack',340,7,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_FIREATTACK','angry',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_MENTALBREAKER','angry',159,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2732,'Furious Nightmare Terror@NPC_MENTALBREAKER','attack',159,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@AL_DECAGI','chase',30,9,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2733,'Elusive Wraith Dead (Nightmare)@NPC_POISON','attack',176,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@KN_AUTOCOUNTER','attack',61,5,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@NPC_BLEEDING','attack',660,1,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2734,'Swift Wanderer (Nightmare)@RG_INTIMIDATE','attack',219,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2735,'Solid Nepenthes@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2735,'Solid Nepenthes@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2735,'Solid Nepenthes@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2735,'Solid Nepenthes@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2735,'Solid Nepenthes@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@CR_REFLECTSHIELD','idle',252,10,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@MG_FIREBALL','chase',17,5,2000,300,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@MG_THUNDERSTORM','attack',21,5,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@MG_THUNDERSTORM','chase',21,5,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_CALLSLAVE','attack',352,1,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_CALLSLAVE','idle',352,1,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_CHANGEUNDEAD','attack',348,1,500,500,6000000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_CRITICALWOUND','attack',673,2,2000,700,60000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_CRITICALWOUND','chase',673,2,2000,500,60000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_DARKNESSBREATH','attack',658,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_DARKSTRIKE','attack',340,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_DARKSTRIKE','chase',340,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_DRAGONFEAR','attack',659,2,500,0,10000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_SLOWCAST','attack',672,1,2000,800,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_SLOWCAST','chase',672,1,2000,300,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_WIDECURSE','attack',677,1,500,800,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_WIDECURSE','chase',677,1,500,300,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@WZ_METEOR','chase',83,3,1000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@WZ_METEOR','chase',83,3,3000,500,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@WZ_QUAGMIRE','attack',92,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@WZ_QUAGMIRE','chase',92,5,500,200,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2736,'Necromancer Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1870,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@MC_MAMMONITE','attack',42,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2737,'Furious Myst Case@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@KN_SPEARSTAB','attack',58,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@NPC_GROUNDATTACK','attack',185,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2738,'Elusive Muscipular@NPC_PETRIFYATTACK','attack',180,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2739,'Swift Mummy@NPC_UNDEADATTACK','attack',347,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2740,'Solid Mummy@NPC_UNDEADATTACK','attack',347,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2741,'Muka Ringleader@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2741,'Muka Ringleader@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2741,'Muka Ringleader@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2741,'Muka Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1055,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@AL_HEAL','idle',28,10,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_GRANDDARKNESS','attack',339,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_MAGICMIRROR','chase',671,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_PULSESTRIKE','chase',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_WIDECURSE','attack',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@NPC_WIDECURSE','chase',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@SA_DISPELL','attack',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2742,'Furious Incarnation of Morroc@SA_DISPELL','chase',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@AL_HEAL','idle',28,10,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_GRANDDARKNESS','attack',339,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_MAGICMIRROR','chase',671,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_PULSESTRIKE','chase',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_WIDECURSE','attack',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@NPC_WIDECURSE','chase',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@SA_DISPELL','attack',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2743,'Elusive Incarnation of Morroc@SA_DISPELL','chase',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_TELEPORT','chase',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_HEAL','idle',28,10,10000,0,5000,'yes','self','myhpltmaxrate','45',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@AL_PNEUMA','idle',25,1,10000,0,1000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_GRANDDARKNESS','attack',339,5,10000,2000,30000,'no','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_MAGICMIRROR','chase',671,1,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_PULSESTRIKE','chase',661,5,2000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_SLOWCAST','attack',672,5,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_WIDECURSE','attack',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@NPC_WIDECURSE','chase',677,5,5000,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@SA_DISPELL','attack',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2744,'Swift Incarnation of Morroc@SA_DISPELL','chase',289,5,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@NPC_COMBOATTACK','chase',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@NPC_GROUNDATTACK','chase',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@TF_HIDING','idle',51,1,500,500,25000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2745,'Solid Holden@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_CURSEATTACK','chase',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2746,'Miyabi Doll Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1404,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2747,'Furious Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2748,'Elusive Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@BS_HAMMERFALL','attack',110,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2749,'Swift Minorous@WZ_HEAVENDRIVE','attack',91,3,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@CR_REFLECTSHIELD','attack',252,2,1000,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@NPC_BARRIER','attack',204,1,500,3000,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@NPC_RANDOMATTACK','attack',183,2,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@NPC_SPLASHATTACK','attack',174,1,500,0,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@TF_HIDING','attack',51,1,500,500,50000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2750,'Solid Mineral@TF_HIDING','idle',51,1,500,500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@MG_COLDBOLT','attack',14,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@MG_COLDBOLT','chase',14,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_BLEEDING','attack',660,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_COMBOATTACK','attack',171,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_CRITICALWOUND','attack',673,1,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_SILENCEATTACK','attack',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_SILENCEATTACK','chase',178,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2751,'Miming Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',2137,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2752,'Furious Mimic@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2752,'Furious Mimic@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2752,'Furious Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2752,'Furious Mimic@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2752,'Furious Mimic@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2753,'Elusive Mimic@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2753,'Elusive Mimic@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2753,'Elusive Mimic@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2753,'Elusive Mimic@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2753,'Elusive Mimic@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@NPC_FIREATTACK','attack',186,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2754,'Swift Metaller@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2755,'Solid Metaling@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2755,'Solid Metaling@CR_AUTOGUARD','chase',249,5,500,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2755,'Solid Metaling@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2755,'Solid Metaling@MC_MAMMONITE','attack',42,5,1000,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2755,'Solid Metaling@RG_STRIPWEAPON','attack',215,3,500,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@CR_AUTOGUARD','chase',249,5,500,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@CR_REFLECTSHIELD','attack',252,3,1000,0,200000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@MC_MAMMONITE','attack',42,5,1000,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@RG_STRIPWEAPON','attack',215,3,500,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2756,'Metaling Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1613,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2757,'Furious Merman@KN_PIERCE','attack',56,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2757,'Furious Merman@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2757,'Furious Merman@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2757,'Furious Merman@NPC_RANGEATTACK','chase',160,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2757,'Furious Merman@NPC_WATERATTACK','attack',184,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2758,'Elusive Menblatt@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2758,'Elusive Menblatt@NPC_DARKSTRIKE','chase',340,5,2000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2758,'Elusive Menblatt@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2758,'Elusive Menblatt@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@MG_STONECURSE','attack',16,10,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@NPC_BLEEDING','attack',660,5,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2759,'Swift Medusa@NPC_PETRIFYATTACK','chase',180,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2760,'Solid Martin@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2760,'Solid Martin@TF_HIDING','attack',51,1,2000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2760,'Solid Martin@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2761,'Marionette Ringleader@HT_FREEZINGTRAP','idle',121,5,500,0,300000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2761,'Marionette Ringleader@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2761,'Marionette Ringleader@NPC_FIREATTACK','attack',186,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2761,'Marionette Ringleader@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2761,'Marionette Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1143,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2762,'Furious Marin@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2762,'Furious Marin@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2762,'Furious Marin@NPC_WATERATTACK','attack',184,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@MG_FIREWALL','chase',18,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@NPC_FIREATTACK','attack',186,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2763,'Elusive Marduk@NPC_MAGICALATTACK','attack',192,2,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2764,'Swift Mantis@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2765,'Solid Mandragora@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_COMBOATTACK','angry',171,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_COMBOATTACK','attack',171,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_WINDATTACK','angry',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2766,'Manananggal Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',2311,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',1149,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2767,'Furious Majoruros@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2768,'Elusive Magnolia@HT_FLASHER','idle',120,5,500,0,300000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2768,'Elusive Magnolia@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2768,'Elusive Magnolia@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2768,'Elusive Magnolia@NPC_UNDEADATTACK','attack',347,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@MG_FIREBALL','chase',17,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@MG_FIREBOLT','attack',19,2,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@NPC_FIREATTACK','attack',186,2,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@SM_BASH','attack',5,4,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2769,'Swift Magmaring@SM_MAGNUM','attack',7,5,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2771,'Lunatic Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1063,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@AL_HEAL','attack',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@AL_HEAL','chase',28,9,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@NPC_METAMORPHOSIS','attack',193,1,1000,0,60000,'no','self','myhpltmaxrate','30',1508,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@PR_LEXAETERNA','attack',78,1,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2772,'Furious Lude@PR_LEXAETERNA','chase',78,1,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@NPC_POISONATTACK','attack',188,2,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2773,'Elusive Luciola Vespa@NPC_PULSESTRIKE','attack',661,2,1000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@NPC_DARKSTRIKE','chase',340,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@NPC_DARKSTRIKE','attack',340,5,500,700,5000,'no','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@NPC_SUMMONSLAVE','idle',196,1,500,2000,60000,'no','self','slavele','0',0,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@PR_LEXDIVINA','attack',76,10,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@PR_LEXDIVINA','chase',76,10,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2774,'Swift Loli Ruri@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2775,'Solid Enchanted Peach Tree@AM_POTIONPITCHER','idle',231,3,10000,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2775,'Solid Enchanted Peach Tree@AM_POTIONPITCHER','attack',231,2,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2775,'Solid Enchanted Peach Tree@AM_POTIONPITCHER','chase',231,2,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2775,'Solid Enchanted Peach Tree@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2775,'Solid Enchanted Peach Tree@NPC_GUIDEDATTACK','chase',172,3,500,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2776,'Little Poring Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',2398,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@MG_LIGHTNINGBOLT','chase',20,5,10000,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@MG_THUNDERSTORM','attack',21,3,2000,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2777,'Furious Little Fatum@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2778,'Elusive Jing Guai@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2778,'Elusive Jing Guai@NPC_RANGEATTACK','chase',160,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@NPC_COMBOATTACK','attack',171,2,3000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2779,'Swift Lesavka@NPC_POISON','attack',176,2,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@NPC_SUMMONSLAVE','idle',196,3,10000,2000,60000,'no','self','slavele','0',1243,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2780,'Solid Leib Olmai@SM_MAGNUM','attack',7,9,500,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2781,'Leaf Cat Ringleader@AL_HEAL','attack',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2781,'Leaf Cat Ringleader@AL_HEAL','chase',28,1,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2781,'Leaf Cat Ringleader@AL_HEAL','attack',28,1,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2781,'Leaf Cat Ringleader@AL_HEAL','chase',28,1,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2781,'Leaf Cat Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1586,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2784,'Swift Kobold Archer@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2784,'Swift Kobold Archer@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2784,'Swift Kobold Archer@AC_DOUBLE','attack',46,3,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2784,'Swift Kobold Archer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2784,'Swift Kobold Archer@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2785,'Solid Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2785,'Solid Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2785,'Solid Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2785,'Solid Kobold@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@HT_LANDMINE','idle',116,2,4000,1000,18000,'yes','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@NPC_GROUNDATTACK','attack',185,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@TF_SPRINKLESAND','attack',149,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@TF_THROWSTONE','attack',152,1,1000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@TF_THROWSTONE','chase',152,1,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2786,'Knocker Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1838,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2787,'Furious Beetle King@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2787,'Furious Beetle King@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2787,'Furious Beetle King@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@CR_REFLECTSHIELD','attack',252,1,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@CR_REFLECTSHIELD','idle',252,1,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@CR_REFLECTSHIELD','chase',252,1,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@HW_GANBANTEIN','attack',483,1,5000,500,5000,'no','target','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@LK_SPIRALPIERCE','attack',397,1,2000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@MG_FIREBALL','attack',17,44,5000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@MG_FIREBALL','idle',17,44,10000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@MG_FIREBALL','chase',17,44,10000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@NPC_FIREBREATH','attack',654,10,2000,1500,15000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@NPC_FIREBREATH','chase',654,10,2000,1500,15000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@WZ_FIREPILLAR','idle',80,5,10000,500,30000,'no','around2','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@WZ_METEOR','attack',83,1,5000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@WZ_METEOR','chase',83,1,5000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2788,'Elusive Kasa@WZ_SIGHTRASHER','attack',81,10,1000,0,20000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@NPC_EMOTION','attack',197,1,10000,0,30000,'yes','self','always',NULL,28,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'31',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'31',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2789,'Swift Karakasa@WZ_WATERBALL','chase',86,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2790,'Solid Isis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2790,'Solid Isis@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2790,'Solid Isis@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2790,'Solid Isis@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','attack',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','idle',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','chase',28,5,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','attack',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','idle',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@AL_HEAL','chase',28,5,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@MG_THUNDERSTORM','attack',21,5,500,1200,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@MG_THUNDERSTORM','chase',21,5,500,1200,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@NPC_HOLYATTACK','attack',189,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@PR_LEXDIVINA','attack',76,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@PR_LEXDIVINA','chase',76,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2791,'Isilla Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1772,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2792,'Furious Injustice@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2792,'Furious Injustice@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2793,'Elusive Mi Gao@NPC_METAMORPHOSIS','attack',193,1,50,5000,5000,'no','self','always',NULL,1516,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@MG_FROSTDIVER','attack',15,5,2000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@MG_FROSTDIVER','chase',15,7,10000,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@NPC_COMBOATTACK','attack',171,1,1000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2794,'Swift Ice Titan@NPC_WATERATTACK','attack',184,3,2000,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@MG_FROSTDIVER','attack',15,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@MG_FROSTDIVER','chase',15,40,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,3,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@NPC_WATERATTACK','attack',184,3,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@PR_LEXDIVINA','attack',76,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@PR_LEXDIVINA','chase',76,5,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@SM_BASH','attack',5,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2795,'Solid Iara@WZ_WATERBALL','attack',86,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2796,'Yao Jun Ringleader@AS_SONICBLOW','attack',136,5,500,1000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2796,'Yao Jun Ringleader@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2796,'Yao Jun Ringleader@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2796,'Yao Jun Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1512,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_EMOTION','follow',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_WINDATTACK','angry',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2797,'Furious Hunter Fly@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2798,'Elusive Hornet@NPC_PIERCINGATT','attack',158,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2798,'Elusive Hornet@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_BLEEDING','attack',660,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_COMBOATTACK','chase',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2799,'Swift Uni-Horn Scaraba@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2800,'Solid Horn@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2800,'Solid Horn@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@AS_SONICBLOW','attack',136,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_BLINDATTACK','chase',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_DARKNESSATTACK','attack',190,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,52,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@RG_STRIPARMOR','attack',217,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2801,'Hodremlin Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1773,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2802,'Furious Hode@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2802,'Furious Hode@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2802,'Furious Hode@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2803,'Elusive Hill Wind@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2803,'Elusive Hill Wind@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2803,'Elusive Hill Wind@NPC_WINDATTACK','chase',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2803,'Elusive Hill Wind@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2803,'Elusive Hill Wind@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@MG_THUNDERSTORM','attack',21,5,2000,1500,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@NPC_WINDATTACK','chase',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@WZ_JUPITEL','attack',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2804,'Swift Hill Wind@WZ_JUPITEL','chase',84,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@CR_SHIELDCHARGE','attack',250,2,500,0,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2805,'Solid High Orc@NPC_SMOKING','idle',195,1,500,0,36000000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_SILENCEATTACK','angry',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_SILENCEATTACK','follow',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_WINDATTACK','angry',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2806,'Harpy Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1376,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_BLOODDRAIN','angry',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_COMBOATTACK','angry',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_SILENCEATTACK','angry',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_SILENCEATTACK','follow',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_WINDATTACK','angry',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2807,'Furious Harpy@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2808,'Elusive Grove@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2808,'Elusive Grove@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@NPC_BLINDATTACK','idle',177,3,10000,1500,5000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@NPC_BLINDATTACK','idle',177,3,10000,1500,5000,'no','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@WZ_EARTHSPIKE','idle',90,3,5000,1500,5000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2809,'Swift Greatest General@WZ_EARTHSPIKE','idle',90,3,5000,1500,5000,'no','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2810,'Solid Grand Peco@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2810,'Solid Grand Peco@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2810,'Solid Grand Peco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2810,'Solid Grand Peco@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2810,'Solid Grand Peco@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2811,'Grand Peco Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1369,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2812,'Furious Golem@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2812,'Furious Golem@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2812,'Furious Golem@CR_SHIELDCHARGE','attack',250,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2812,'Furious Golem@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2812,'Furious Golem@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2813,'Elusive Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2813,'Elusive Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,2,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2813,'Elusive Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,18,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2813,'Elusive Goblin@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,2,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,18,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2814,'Swift Goblin@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@NPC_CRITICALWOUND','attack',673,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x1089,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2815,'Solid Goat@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@NPC_CRITICALWOUND','attack',673,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x1089,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2816,'Goat Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1372,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2817,'Furious Dark Hammer Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2817,'Furious Dark Hammer Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2817,'Furious Dark Hammer Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2817,'Furious Dark Hammer Kobold@NPC_POISONATTACK','attack',188,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2818,'Elusive Dark Hammer Kobold@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2818,'Elusive Dark Hammer Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x308D,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2818,'Elusive Dark Hammer Kobold@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2818,'Elusive Dark Hammer Kobold@NPC_POISONATTACK','attack',188,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@NPC_DARKNESSBREATH','attack',658,1,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@NPC_DARKSTRIKE','chase',340,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2819,'Swift Dark Shadow@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2820,'Solid Dark Shadow@NPC_DARKNESSBREATH','attack',658,1,500,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2820,'Solid Dark Shadow@NPC_DARKSTRIKE','chase',340,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2820,'Solid Dark Shadow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2820,'Solid Dark Shadow@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,9,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_SILENCEATTACK','chase',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2821,'Giant Hornet Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1303,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2822,'Furious Ghoul@NPC_POISON','angry',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2822,'Furious Ghoul@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2823,'Elusive Ghoul@NPC_POISON','angry',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2823,'Elusive Ghoul@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2824,'Swift Geographer@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2824,'Swift Geographer@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2824,'Swift Geographer@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2824,'Swift Geographer@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2824,'Swift Geographer@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2825,'Solid Geographer@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2825,'Solid Geographer@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2825,'Solid Geographer@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@AC_CHARGEARROW','attack',148,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2826,'Gargoyle Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1253,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2827,'Furious Gargoyle@AC_CHARGEARROW','attack',148,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2827,'Furious Gargoyle@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2827,'Furious Gargoyle@NPC_BLINDATTACK','attack',177,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2827,'Furious Gargoyle@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2827,'Furious Gargoyle@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@AS_SONICBLOW','attack',136,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_COMBOATTACK','attack',171,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_SUMMONSLAVE','attack',196,3,10000,1000,30000,'no','self','slavele','3',1794,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_SUMMONSLAVE','idle',196,3,10000,1000,30000,'no','self','slavele','3',1794,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2828,'Elusive Galion@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@NPC_COMBOATTACK','attack',171,2,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,1,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2829,'Swift Seal@NPC_WATERATTACK','attack',184,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@MG_FROSTDIVER','attack',15,9,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2830,'Solid Freezer@NPC_WATERATTACK','attack',184,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@MG_COLDBOLT','attack',14,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@MG_COLDBOLT','chase',14,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@MG_FROSTDIVER','attack',15,9,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,20,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@NPC_WATERATTACK','attack',184,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2831,'Freezer Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1319,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@KN_BRANDISHSPEAR','attack',57,10,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@KN_SPEARSTAB','attack',58,10,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@NPC_COMBOATTACK','attack',171,2,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2832,'Furious Ferus@SM_PROVOKE','chase',6,10,2000,600,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2833,'Elusive Familiar@NPC_BLINDATTACK','angry',177,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2833,'Elusive Familiar@NPC_BLINDATTACK','attack',177,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2833,'Elusive Familiar@NPC_DARKNESSATTACK','angry',190,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2833,'Elusive Familiar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_BLINDATTACK','angry',177,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_BLINDATTACK','follow',177,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_CURSEATTACK','angry',181,5,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_CURSEATTACK','attack',181,5,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_CURSEATTACK','follow',181,5,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_CURSEATTACK','chase',181,5,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_DARKSTRIKE','attack',340,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_DARKSTRIKE','chase',340,7,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_HALLUCINATION','angry',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_HALLUCINATION','follow',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_SILENCEATTACK','angry',178,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_SILENCEATTACK','follow',178,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2834,'Swift False Angel@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2835,'Solid Fabre@NPC_GROUNDATTACK','attack',185,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2836,'Fabre Ringleader@NPC_GROUNDATTACK','attack',185,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2836,'Fabre Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1007,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@AM_DEMONSTRATION','angry',229,2,100,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@AM_DEMONSTRATION','attack',229,2,100,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@NPC_FIREATTACK','angry',186,2,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@NPC_FIREATTACK','attack',186,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@SM_MAGNUM','angry',7,5,500,500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2837,'Furious Explosion@SM_MAGNUM','attack',7,5,500,500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_BLINDATTACK','chase',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_CURSEATTACK','chase',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_UNDEADATTACK','attack',347,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@NPC_UNDEADATTACK','chase',347,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2838,'Elusive Evil Druid@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2839,'Swift Elder Willow@NPC_FIREATTACK','attack',186,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2840,'Solid Elder Willow@MG_FIREBOLT','attack',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2840,'Solid Elder Willow@MG_FIREBOLT','chase',19,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2840,'Solid Elder Willow@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2840,'Solid Elder Willow@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2840,'Solid Elder Willow@NPC_FIREATTACK','attack',186,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@AS_SONICBLOW','attack',136,5,1000,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@BS_MAXIMIZE','attack',114,5,500,500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@BS_MAXIMIZE','chase',114,5,500,500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@CR_HOLYCROSS','attack',253,5,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@NPC_CRITICALSLASH','angry',170,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@NPC_HOLYATTACK','attack',189,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@NPC_SLEEPATTACK','attack',182,4,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@PR_LEXAETERNA','attack',78,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@SM_BASH','attack',5,7,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2841,'Echio Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1770,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2842,'Furious Dustiness@NPC_BLINDATTACK','attack',177,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2842,'Furious Dustiness@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2842,'Furious Dustiness@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2842,'Furious Dustiness@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@AL_HEAL','attack',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@AL_HEAL','idle',28,9,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2843,'Swift Dryad@NPC_STOP','attack',342,1,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@KN_PIERCE','attack',56,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@NPC_BLOODDRAIN','attack',199,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@NPC_GROUNDATTACK','attack',185,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@NPC_GUIDEDATTACK','attack',172,3,500,1000,20000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2844,'Solid Drosera@RG_CLOSECONFINE','attack',1005,1,1000,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2845,'Drops Ringleader@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2845,'Drops Ringleader@NPC_FIREATTACK','attack',186,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2845,'Drops Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1113,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@AS_CLOAKING','angry',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@AS_CLOAKING','attack',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@AS_CLOAKING','follow',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@AS_CLOAKING','idle',135,1,100,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@AS_CLOAKING','chase',135,1,2000,200,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@NPC_COMBOATTACK','angry',171,2,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@NPC_COMBOATTACK','attack',171,2,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@NPC_GROUNDATTACK','angry',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2846,'Furious Driller@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2847,'Elusive Drainliar@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2847,'Elusive Drainliar@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2847,'Elusive Drainliar@NPC_DARKNESSATTACK','attack',190,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2848,'Swift Dragon Tail@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2848,'Swift Dragon Tail@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2848,'Swift Dragon Tail@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2848,'Swift Dragon Tail@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2848,'Swift Dragon Tail@NPC_WINDATTACK','attack',187,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@NPC_COMBOATTACK','attack',171,2,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@NPC_FIREBREATH','attack',654,3,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@SM_PROVOKE','chase',6,10,2000,600,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'44',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2849,'Solid Draco@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@AS_VENOMDUST','angry',140,1,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@NPC_BLEEDING','attack',660,2,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@NPC_POISON','attack',176,2,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@PF_SPIDERWEB','attack',405,1,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@PF_SPIDERWEB','idle',405,1,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2850,'Dolomedes Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',2092,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2851,'Furious Dokebi@BS_ADRENALINE','attack',111,1,500,1500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2851,'Furious Dokebi@BS_ADRENALINE','chase',111,1,500,1500,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2851,'Furious Dokebi@MC_MAMMONITE','attack',42,5,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2851,'Furious Dokebi@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2851,'Furious Dokebi@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2852,'Elusive Disguise@NPC_COMBOATTACK','attack',171,2,1000,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2852,'Elusive Disguise@NPC_METAMORPHOSIS','attack',193,1,500,0,60000,'yes','self','myhpltmaxrate','10',1509,1508,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2852,'Elusive Disguise@NPC_PIERCINGATT','attack',158,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@NPC_COMBOATTACK','attack',171,4,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@NPC_COMBOATTACK','chase',171,4,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2853,'Swift Dimik@NPC_WINDATTACK','attack',187,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_DARKNESSATTACK','attack',190,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_DARKTHUNDER','attack',341,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_DARKTHUNDER','chase',341,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2854,'Solid Deviruchi@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2855,'Baby Desert Wolf Ringleader@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2855,'Baby Desert Wolf Ringleader@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2855,'Baby Desert Wolf Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1107,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2856,'Furious Baby Desert Wolf@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2856,'Furious Baby Desert Wolf@NPC_FIREATTACK','attack',186,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2857,'Elusive Deniro@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2857,'Elusive Deniro@NPC_GROUNDATTACK','attack',185,2,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2858,'Swift Death Word@NPC_SPLASHATTACK','attack',174,1,1000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2859,'Solid Death Word@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2859,'Solid Death Word@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2859,'Solid Death Word@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2859,'Solid Death Word@NPC_SPLASHATTACK','attack',174,1,1000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2860,'Death Word Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2860,'Death Word Ringleader@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2860,'Death Word Ringleader@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2860,'Death Word Ringleader@NPC_SPLASHATTACK','attack',174,1,1000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2860,'Death Word Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1698,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@MO_BODYRELOCATION','chase',264,1,10000,0,5000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@NPC_BLEEDING','attack',660,3,10,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@NPC_CHANGEUNDEAD','attack',348,1,300,1000,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@NPC_DARKBREATH','chase',202,2,500,800,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@NPC_DARKCROSS','attack',338,3,5000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2861,'Furious Dark Priest@NPC_DARKNESSATTACK','attack',190,2,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2862,'Elusive Zhu Po Long@NPC_AGIUP','idle',350,1,0,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2862,'Elusive Zhu Po Long@NPC_AGIUP','loot',350,1,10000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2862,'Elusive Zhu Po Long@NPC_EMOTION','attack',197,1,10000,0,3000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2862,'Elusive Zhu Po Long@NPC_EMOTION','idle',197,1,10000,0,3000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2862,'Elusive Zhu Po Long@NPC_EMOTION','chase',197,1,10000,0,3000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@NPC_SLEEPATTACK','attack',182,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2863,'Swift Creamy@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2864,'Solid Cornutus@CR_AUTOGUARD','attack',249,2,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2864,'Solid Cornutus@CR_AUTOGUARD','chase',249,2,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2864,'Solid Cornutus@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2864,'Solid Cornutus@NPC_WATERATTACK','attack',184,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','attack',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','idle',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','chase',28,5,10000,500,5000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','idle',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@AL_HEAL','chase',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@NPC_TELEKINESISATTACK','attack',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@NPC_TELEKINESISATTACK','chase',191,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'4',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2865,'Cookie Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1265,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2866,'Furious Constant@NPC_SELFDESTRUCTION','attack',173,1,10000,1000,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2866,'Furious Constant@NPC_SELFDESTRUCTION','chase',173,1,2000,1000,0,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@KN_SPEARSTAB','attack',58,3,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_CRITICALSLASH','attack',170,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_EMOTION','attack',197,1,10000,0,5000,'yes','self','always',NULL,28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_EMOTION','idle',197,1,10000,0,5000,'yes','self','always',NULL,28,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_GROUNDATTACK','attack',185,5,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_POISON','attack',176,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2867,'Elusive Comodo@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2868,'Swift Coco@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2868,'Swift Coco@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2868,'Swift Coco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2868,'Swift Coco@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2868,'Swift Coco@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2869,'Solid Coco@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2869,'Solid Coco@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2869,'Solid Coco@TF_THROWSTONE','chase',152,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_CURSEATTACK','attack',181,4,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_GROUNDATTACK','attack',185,4,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2870,'Clock Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1269,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2871,'Furious Clock@NPC_CURSEATTACK','attack',181,4,500,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2871,'Furious Clock@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2871,'Furious Clock@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2871,'Furious Clock@NPC_GROUNDATTACK','attack',185,4,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2871,'Furious Clock@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2872,'Elusive Chonchon@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','around1','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@HW_GANBANTEIN','attack',483,1,3000,0,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@NPC_COMBOATTACK','attack',171,1,3000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2873,'Swift Centipede@WZ_QUAGMIRE','chase',92,5,500,700,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2874,'Solid Cenere@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2874,'Solid Cenere@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2874,'Solid Cenere@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2874,'Solid Cenere@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2874,'Solid Cenere@NPC_WINDATTACK','attack',187,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_COLDBOLT','attack',14,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_COLDBOLT','chase',14,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBALL','attack',17,44,3000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBALL','idle',17,44,10000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBALL','chase',17,44,3000,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBALL','chase',17,44,10000,500,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBOLT','attack',19,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREBOLT','chase',19,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREWALL','attack',18,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FIREWALL','chase',18,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_FROSTDIVER','chase',15,40,10000,1000,10000,'no','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_SAFETYWALL','attack',12,10,3000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_SIGHT','idle',10,1,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@NPC_MENTALBREAKER','attack',159,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@NPC_MENTALBREAKER','chase',159,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@SA_DISPELL','attack',289,5,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@SA_DISPELL','chase',289,5,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@SA_LANDPROTECTOR','attack',288,1,1000,1500,30000,'no','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'33',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2875,'Celia Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',2223,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2876,'Furious Caterpillar@NPC_PETRIFYATTACK','attack',180,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2876,'Furious Caterpillar@NPC_POISON','attack',176,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2876,'Furious Caterpillar@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@NPC_WINDATTACK','attack',187,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2877,'Elusive Carat@SM_PROVOKE','chase',6,10,2000,600,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2878,'Swift Caramel@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2878,'Swift Caramel@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2878,'Swift Caramel@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2878,'Swift Caramel@NPC_PIERCINGATT','attack',158,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2879,'Solid Bungisngis@BS_HAMMERFALL','attack',110,5,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2879,'Solid Bungisngis@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2879,'Solid Bungisngis@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2879,'Solid Bungisngis@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2879,'Solid Bungisngis@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'20',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_DEFENDER','attack',205,1,1000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2880,'Breeze Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1692,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2881,'Furious Breeze@NPC_COMBOATTACK','attack',171,2,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2881,'Furious Breeze@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2881,'Furious Breeze@NPC_DEFENDER','attack',205,1,1000,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2881,'Furious Breeze@NPC_DEFENDER','chase',205,1,500,0,5000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2881,'Furious Breeze@NPC_WINDATTACK','attack',187,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always',NULL,9,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always',NULL,6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@NPC_STUNATTACK','attack',179,4,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2882,'Elusive Bradium Golem@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@KN_PIERCE','attack',56,10,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@NPC_DARKSTRIKE','chase',340,5,2000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2883,'Swift Bloody Butterfly@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,6,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@NPC_GROUNDATTACK','attack',185,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@NPC_STUNATTACK','attack',179,2,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@SM_ENDURE','chase',8,1,5000,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2884,'Bigfoot Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1060,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_EMOTION','chase',197,1,2000,0,5000,'yes','self','always',NULL,30,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2885,'Furious Bathory@NPC_SILENCEATTACK','attack',178,3,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always',NULL,7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,NULL,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_EVILLAND','attack',670,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_VAMPIRE_GIFT','attack',679,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2886,'Elusive Banshee Master@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_CURSEATTACK','chase',181,1,1000,800,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_DARKNESSATTACK','attack',190,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_DARKSTRIKE','attack',340,9,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_DARKSTRIKE','chase',340,9,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_EMOTION','chase',197,1,1000,0,5000,'yes','self','always',NULL,7,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,NULL,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_EVILLAND','attack',670,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_VAMPIRE_GIFT','attack',679,1,500,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2887,'Swift Banshee@NPC_WIDECURSE','attack',677,1,500,800,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@NPC_FIREATTACK','attack',186,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@NPC_HALLUCINATION','attack',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@NPC_HALLUCINATION','chase',207,1,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@RG_STRIPWEAPON','attack',215,5,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2888,'Solid Banaspaty@WZ_SIGHTRASHER','attack',81,7,1000,1000,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_EMOTION','attack',197,1,2000,0,600000,'yes','self','myhpltmaxrate','20',23,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_WINDATTACK','attack',187,5,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_COMBOATTACK','attack',171,1,1500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@TF_BACKSLIDING','attack',150,1,10000,500,600000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1315,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2889,'Assaulter Ringleader@NPC_SUMMONSLAVE','chase',196,3,10000,0,600000,'yes','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2890,'Furious Argos@NPC_POISON','attack',176,3,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2890,'Furious Argos@PF_SPIDERWEB','idle',405,1,3000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2890,'Furious Argos@PF_SPIDERWEB','attack',405,1,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2890,'Furious Argos@NPC_POISONATTACK','attack',188,1,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2891,'Elusive Argiope@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2891,'Elusive Argiope@NPC_POISON','attack',176,3,1500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2891,'Elusive Argiope@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2891,'Elusive Argiope@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2891,'Elusive Argiope@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@AS_VENOMDUST','attack',140,1,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@NPC_POISON','attack',176,3,1500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@NPC_POISONATTACK','attack',188,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2892,'Swift Argiope@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2893,'Solid Arclouze@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2893,'Solid Arclouze@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2893,'Solid Arclouze@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2893,'Solid Arclouze@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2894,'Arclouze Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2894,'Arclouze Ringleader@NPC_GROUNDATTACK','attack',185,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2894,'Arclouze Ringleader@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2894,'Arclouze Ringleader@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2894,'Arclouze Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1194,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2895,'Furious Apocalypse@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2895,'Furious Apocalypse@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2895,'Furious Apocalypse@KN_BRANDISHSPEAR','attack',57,5,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2895,'Furious Apocalypse@NPC_DARKTHUNDER','attack',341,4,1000,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2895,'Furious Apocalypse@NPC_SELFDESTRUCTION','attack',173,1,1000,2000,5000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@BS_ADRENALINE','attack',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@BS_ADRENALINE','chase',111,10,500,1500,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_BLEEDING','attack',660,3,2000,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_PIERCINGATT','attack',158,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_STONESKIN','attack',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_STONESKIN','chase',675,3,10000,1000,30000,'no','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2896,'Elusive Antler Scaraba@PF_SPIDERWEB','chase',405,1,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2897,'Swift Antique Book@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2897,'Swift Antique Book@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2897,'Swift Antique Book@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2897,'Swift Antique Book@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2897,'Swift Antique Book@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2898,'Solid Antique Book@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2898,'Solid Antique Book@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2898,'Solid Antique Book@NPC_PIERCINGATT','attack',158,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2899,'Furious Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,22,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2899,'Furious Anolian@NPC_WATERATTACK','attack',184,3,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2899,'Furious Anolian@SM_BASH','attack',5,7,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2899,'Furious Anolian@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2899,'Furious Anolian@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,2,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@CR_AUTOGUARD','attack',249,2,700,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2900,'Elusive Angra Mantis@NPC_SUMMONSLAVE','idle',196,1,10000,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@AS_SONICBLOW','attack',136,10,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@CR_AUTOGUARD','attack',249,2,700,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@NPC_COMBOATTACK','attack',171,1,500,700,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2901,'Swift Angra Mantis@NPC_SUMMONSLAVE','idle',196,1,10000,2000,60000,'no','self','slavele','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2902,'Solid Andre@NPC_EMOTION','loot',197,1,2000,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2902,'Solid Andre@NPC_GROUNDATTACK','attack',185,2,1500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2903,'Ancient Mimic Ringleader@CR_AUTOGUARD','attack',249,3,500,0,250000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2903,'Ancient Mimic Ringleader@CR_AUTOGUARD','chase',249,3,1000,0,250000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2903,'Ancient Mimic Ringleader@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2903,'Ancient Mimic Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1699,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2904,'Furious Anacondaq@NPC_POISON','attack',176,4,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2904,'Furious Anacondaq@NPC_POISONATTACK','attack',188,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2905,'Elusive Ambernite@NPC_EMOTION','walk',197,1,2000,0,5000,'yes','self','always',NULL,19,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2905,'Elusive Ambernite@NPC_KEEPING','attack',201,1,2000,0,5000,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2905,'Elusive Ambernite@NPC_WATERATTACK','attack',184,2,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@AL_HEAL','attack',28,8,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@AL_HEAL','attack',28,8,10000,500,5000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@MG_SOULSTRIKE','attack',13,3,500,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2906,'Swift Rafflesia Arnoldi@NPC_POISONATTACK','attack',188,5,2000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2907,'Solid Alligator@NPC_WATERATTACK','attack',184,4,1000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2907,'Solid Alligator@NPC_CRITICALSLASH','attack',170,2,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2907,'Solid Alligator@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x91,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@AL_HEAL','attack',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@AL_HEAL','idle',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@AL_HEAL','walk',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@AL_HEAL','chase',28,5,5000,1000,5000,'yes','friend','friendhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@NPC_EMOTION','idle',197,1,500,0,30000,'yes','self','always',NULL,54,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@NPC_EMOTION','walk',197,1,500,0,30000,'yes','self','always',NULL,2,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@SM_BASH','attack',5,10,2000,1000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'52',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2908,'Aliza Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1737,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2909,'Furious Alicel@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2909,'Furious Alicel@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2909,'Furious Alicel@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2909,'Furious Alicel@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2909,'Furious Alicel@NPC_GUIDEDATTACK','attack',172,1,1000,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2910,'Elusive Alarm@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2910,'Elusive Alarm@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2910,'Elusive Alarm@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always',NULL,19,0x81,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2910,'Elusive Alarm@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always',NULL,6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2910,'Elusive Alarm@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_DECAGI','chase',30,48,2000,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','attack',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','idle',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','chase',28,9,3000,0,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@AL_HEAL','attack',28,5,10000,500,5000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_SAFETYWALL','attack',12,10,2000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_SIGHT','idle',10,1,1000,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_SIGHT','idle',10,1,10000,0,5000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_SOULSTRIKE','attack',13,9,1000,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_SOULSTRIKE','chase',13,9,1000,500,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_THUNDERSTORM','attack',21,9,2000,1500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@MG_THUNDERSTORM','chase',21,9,2000,1500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@NPC_CURSEATTACK','attack',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@NPC_CURSEATTACK','chase',181,5,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@NPC_HOLYATTACK','attack',189,3,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@PR_LEXDIVINA','attack',76,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@PR_LEXDIVINA','chase',76,1,1000,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@PR_LEXDIVINA','attack',76,1,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2911,'Swift Agav@PR_LEXDIVINA','chase',76,1,5000,0,5000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@MG_LIGHTNINGBOLT','attack',20,8,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@MG_LIGHTNINGBOLT','idle',20,8,2000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@MG_LIGHTNINGBOLT','chase',20,8,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@MG_THUNDERSTORM','attack',21,9,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@MG_THUNDERSTORM','attack',21,9,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2912,'Solid Acidus@NPC_WINDATTACK','attack',187,6,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@MG_LIGHTNINGBOLT','attack',20,8,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@MG_LIGHTNINGBOLT','idle',20,8,2000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@MG_LIGHTNINGBOLT','chase',20,8,1000,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@MG_THUNDERSTORM','attack',21,9,1000,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@MG_THUNDERSTORM','attack',21,9,2000,1000,5000,'no','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@NPC_STOP','attack',342,1,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@NPC_WINDATTACK','attack',187,6,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2913,'Acidus Ringleader@NPC_SUMMONSLAVE','idle',196,2,10000,700,30000,'no','self','slavele','0',1716,NULL,NULL,NULL,NULL,NULL,NULL);

# Nightmare Clock Tower Dungeon
REPLACE INTO `mob_skill_db_re` VALUES (2916,'Big Ben@NPC_CURSEATTACK','attack',181,3,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2916,'Big Ben@NPC_GROUNDATTACK','attack',185,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2916,'Big Ben@NPC_STUNATTACK','attack',179,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2916,'Big Ben@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x0091,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2916,'Big Ben@NPC_POWERUP','attack',349,5,10000,0,30000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_DARKBREATH','attack',202,2,500,800,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x0081,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@LK_SPIRALPIERCE','attack',397,5,10000,1000,10000,'yes','target','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2917,'Big Bell@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@AL_TELEPORT','idle',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@WZ_FIREPILLAR','idle',80,5,500,0,5000,'yes','around2','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@SA_DISPELL','attack',289,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_SILENCEATTACK','attack',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_SILENCEATTACK','chase',178,5,500,700,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_SLEEPATTACK','attack',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_SLEEPATTACK','chase',182,5,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_EMOTION_ON','attack',474,1,10000,0,30000,'yes','self','always','0',6,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2918,'Time Keeper@NPC_EMOTION','idle',197,1,200,0,30000,'yes','self','always','0',9,0x0091,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x0081,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',7,0x3095,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_WINDATTACK','attack',187,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2919,'Neo Punk@NPC_MENTALBREAKER','attack',159,1,500,800,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_FIREBOLT','attack',19,7,10000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_FROSTDIVER','chase',15,9,10000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_LIGHTNINGBOLT','chase',20,7,10000,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_STONECURSE','attack',16,10,10000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_FIREBOLT','idle',19,7,10000,1500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_FROSTDIVER','idle',15,9,10000,500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_LIGHTNINGBOLT','idle',20,7,10000,1500,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'12',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@MG_STONECURSE','idle',16,10,10000,0,5000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'11',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@NPC_RUN','attack',354,1,10000,0,0,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2920,'Arc Elder@NPC_RUN','chase',354,1,10000,0,0,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@MG_LIGHTNINGBOLT','attack',20,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@MG_LIGHTNINGBOLT','chase',20,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@CR_REFLECTSHIELD','attack',252,10,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@CR_REFLECTSHIELD','chase',252,10,10000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x0081,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2921,'Owl Viscount@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@MG_LIGHTNINGBOLT','attack',20,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@MG_LIGHTNINGBOLT','chase',20,3,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',32,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@NPC_EMOTION','chase',197,1,200,0,5000,'yes','self','always','0',19,0x0081,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@NPC_EMOTION','idle',197,1,2000,0,5000,'yes','self','always','0',6,0x3695,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2923,'Owl Marquees@NPC_SUMMONSLAVE','idle',196,2,10000,2000,60000,'no','self','slavele','0',2922,NULL,NULL,NULL,NULL,NULL,NULL);

# Devil Tower Memorial
REPLACE INTO `mob_skill_db_re` VALUES (2939,'Evil Shadow@NPC_PIERCINGATT','chase',158,1,4000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2939,'Evil Shadow@NPC_CURSEATTACK','attack',181,5,3500,0,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2940,'Evil Shadow@NPC_DARKSTRIKE','attack',340,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2941,'Evil Shadow@NPC_CRITICALSLASH','chase',170,1,3000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2941,'Evil Shadow@NPC_PETRIFYATTACK','attack',180,5,3500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','idle',685,1,10000,0,20000,'yes','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','100',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','80',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,'42');
REPLACE INTO `mob_skill_db_re` VALUES (2942,'Evil Fanatics@NPC_INVINCIBLE','attack',685,1,10000,0,20000,'yes','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,'42');
#REPLACE INTO `mob_skill_db_re` VALUES ( 2942,'Evil Fanatics@NPC_DANCINGBLADE','attack',713,1,1000,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2942,'Evil Fanatics@NPC_PULSESTRIKE2','attack',712,1,800,0,15000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 2942,'Evil Fanatics@NPC_DARKPIERCING','attack',715,1,2500,0,60000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2943,'Icemine@NPC_INVISIBLE','idle',353,1,10000,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2943,'Icemine@NPC_ICEMINE','idle',709,1,10000,0,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2960,'Flamecross@NPC_INVISIBLE','idle',353,1,10000,0,300000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2960,'Flamecross@NPC_FLAMECROSS','idle',711,1,10000,0,300000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Ghost Palace Memorial
REPLACE INTO `mob_skill_db_re` VALUES (2948,'Cursed Soldier@NPC_BLINDATTACK','attack',177,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2948,'Cursed Soldier@AC_DOUBLE','attack',46,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2948,'Cursed Soldier@NPC_MAGICALATTACK','attack',192,1,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2948,'Cursed Soldier@AC_SHOWER','attack',47,3,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2949,'Cursed Sentinel@AS_SONICBLOW','attack',136,7,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2949,'Cursed Sentinel@NPC_CURSEATTACK','attack',181,5,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_BLINDATTACK','attack',177,3,1500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_BLINDATTACK','chase',177,3,1500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'1',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'26',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_CURSEATTACK','attack',181,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_CURSEATTACK','chase',181,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'0',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@CR_REFLECTSHIELD','attack',252,2,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2950,'Broken Mind@NPC_PETRIFYATTACK','chase',180,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2951,'Floating Word@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2951,'Floating Word@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2951,'Floating Word@MG_FIREBALL','chase',17,5,1500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2951,'Floating Word@MO_BODYRELOCATION','chase',264,1,2000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@HT_SHOCKWAVE','idle',118,5,500,0,300000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@MG_LIGHTNINGBOLT','attack',20,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@MG_LIGHTNINGBOLT','chase',20,5,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',29,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@NPC_MAGICALATTACK','attack',192,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@NPC_UNDEADATTACK','attack',347,2,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@WZ_JUPITEL','attack',84,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2952,'Like Love@WZ_JUPITEL','chase',84,3,500,1500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2953,'Cursed Memory@NPC_POISON','attack',176,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2953,'Cursed Memory@NPC_POISONATTACK','attack',188,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2953,'Cursed Memory@NPC_UNDEADATTACK','attack',347,2,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2954,'Colorless Vow@NPC_CRITICALSLASH','attack',170,1,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2954,'Colorless Vow@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2955,'Old Friendship@NPC_POISON','attack',176,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2955,'Old Friendship@NPC_POISON','angry',176,3,500,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2956,'Sweet Slaughter@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2956,'Sweet Slaughter@BS_MAXIMIZE','attack',114,5,500,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2956,'Sweet Slaughter@BS_MAXIMIZE','chase',114,5,500,1000,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2956,'Sweet Slaughter@NPC_SPLASHATTACK','attack',174,1,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2956,'Sweet Slaughter@NPC_STUNATTACK','attack',179,3,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2957,'Forgotten Name@NPC_CRITICALSLASH','attack',170,1,1000,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2957,'Forgotten Name@NPC_DARKNESSATTACK','attack',190,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2958,'Fatal Days@MG_SOULSTRIKE','chase',13,5,2000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2958,'Fatal Days@AS_GRIMTOOTH','attack',137,5,10000,0,700,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2959,'Torturous Redeemer@SM_MAGNUM','attack',7,5,500,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2959,'Torturous Redeemer@NPC_FIREATTACK','attack',186,3,1000,1000,200000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2959,'Torturous Redeemer@WZ_FIREPILLAR','attack',80,5,500,0,10000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2959,'Torturous Redeemer@SM_BASH','attack',5,5,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Horror Toy Factory
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@AS_GRIMTOOTH','chase',137,5,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@NPC_CURSEATTACK','attack',181,3,1000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@NPC_UNDEADATTACK','attack',347,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@MG_THUNDERSTORM','attack',21,20,2000,1500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@TF_SPRINKLESAND','attack',149,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2987,'Decorated Evil Tree@MG_LIGHTNINGBOLT','chase',20,10,2000,1500,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@AL_TELEPORT','attack',26,1,500,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@AL_TELEPORT','idle',26,1,1000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@NPC_RUN','attack',354,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@NPC_RUN','chase',354,1,10000,0,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@SA_REVERSEORCISH','chase',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@SA_REVERSEORCISH','attack',294,1,500,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@MG_SAFETYWALL','attack',12,10,1000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2988,'Wicked Vice Plant Manager@AL_INCAGI','chase',29,1,500,700,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@MG_COLDBOLT','attack',14,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@MG_COLDBOLT','chase',14,3,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'21',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@NPC_ENERGYDRAIN','attack',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@NPC_ENERGYDRAIN','chase',200,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'30',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@PR_LEXAETERNA','attack',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@PR_LEXAETERNA','chase',78,1,500,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@SA_REVERSEORCISH','attack',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2989,'Vicious Cookie@SA_REVERSEORCISH','chase',294,1,50,0,30000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2990,'Corrupt Cruiser@NPC_COMBOATTACK','attack',171,2,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'24',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2990,'Corrupt Cruiser@AC_CHARGEARROW','chase',148,1,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'27',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2990,'Corrupt Cruiser@AC_CHARGEARROW','attack',148,1,2000,1000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2990,'Corrupt Cruiser@AC_DOUBLE','attack',46,7,2000,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2991,'Evil Dwelling Box@NPC_WIDESILENCE','attack',663,1,500,1200,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2991,'Evil Dwelling Box@BS_HAMMERFALL','attack',110,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2991,'Evil Dwelling Box@NPC_CURSEATTACK','attack',181,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2991,'Evil Dwelling Box@NPC_HALLUCINATION','attack',207,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2991,'Evil Dwelling Box@NPC_HALLUCINATION','chase',207,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2992,'Creepy Demon@AS_CLOAKING','attack',135,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2992,'Creepy Demon@AS_CLOAKING','idle',135,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2992,'Creepy Demon@AS_CLOAKING','chase',135,1,2000,0,5000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2992,'Creepy Demon@MO_BODYRELOCATION','chase',264,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2992,'Creepy Demon@MO_BODYRELOCATION','attack',264,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@NPC_SILENCEATTACK','attack',178,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@NPC_HALLUCINATION','chase',207,1,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@NPC_HALLUCINATION','attack',207,1,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@MO_BODYRELOCATION','chase',264,1,2000,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@AS_CLOAKING','idle',135,1,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@AS_CLOAKING','chase',135,1,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2993,'Malicious Baby Ghost@NPC_TELEKINESISATTACK','attack',191,8,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2994,'Dancing Marionette@AS_CLOAKING','idle',135,1,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2994,'Dancing Marionette@AS_CLOAKING','chase',135,1,2000,0,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2994,'Dancing Marionette@MG_FIREBALL','attack',17,8,2000,800,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2994,'Dancing Marionette@MG_FIREWALL','chase',18,8,500,500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2994,'Dancing Marionette@NPC_TELEKINESISATTACK','attack',191,8,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2995,'Abandoned Teddy Bear@NPC_CURSEATTACK','attack',181,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2995,'Abandoned Teddy Bear@NPC_CURSEATTACK','chase',181,4,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2995,'Abandoned Teddy Bear@NPC_CRITICALSLASH','attack',170,1,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2995,'Abandoned Teddy Bear@SA_DISPELL','attack',289,1,50,1000,15000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2995,'Abandoned Teddy Bear@BS_HAMMERFALL','attack',110,5,500,1500,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@AL_HEAL','idle',28,11,10000,500,5000,'yes','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_CRITICALWOUND','attack',673,2,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_PULSESTRIKE','attack',661,3,10000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_DARKSTRIKE','attack',340,10,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_DARKTHUNDER','attack',341,9,500,1000,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_WIDESTONE','attack',666,5,10000,1000,30000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_WIDESLEEP','attack',668,5,10000,1000,30000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_WIDESOULDRAIN','attack',680,5,10000,5000,50000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_MAXPAIN','attack',716,9,10000,3000,50000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2996,'Celine Kimi@NPC_POWERUP','attack',349,5,10000,0,50000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@AL_HEAL','idle',28,11,10000,500,5000,'yes','self','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_CRITICALWOUND','attack',673,1,1000,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_PULSESTRIKE','attack',661,1,10000,0,20000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_DARKSTRIKE','attack',340,5,500,0,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_DARKTHUNDER','attack',341,4,500,1000,20000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_WIDESTONE','attack',666,1,10000,1000,30000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_WIDESLEEP','attack',668,1,10000,1000,30000,'no','self','myhpltmaxrate','20',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_WIDESOULDRAIN','attack',680,3,10000,5000,50000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,'42',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_MAXPAIN','attack',716,3,10000,3000,50000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2997,'Kimi\'s Phantom@NPC_POWERUP','attack',349,1,10000,0,50000,'no','self','myhpltmaxrate','5',NULL,NULL,NULL,NULL,NULL,'6',NULL);

# Episode 14.3: Morse cave
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_SIGHT','idle',10,1,2000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_SIGHT','idle',10,1,10000,0,5000,'no','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@AL_TELEPORT','idle',26,1,10000,0,0,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@AL_TELEPORT','walk',26,1,5000,0,5000,'no','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FROSTDIVER','idle',15,40,10000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_JUPITEL','idle',84,28,10000,1000,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBALL','idle',17,44,10000,500,10000,'yes','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FROSTDIVER','chase',15,40,10000,1000,10000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_JUPITEL','chase',84,28,10000,1000,10000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBALL','chase',17,44,10000,500,10000,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FROSTDIVER','attack',15,40,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FROSTDIVER','chase',15,40,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@HW_GANBANTEIN','attack',483,1,10000,500,7000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@HW_NAPALMVULCAN','chase',400,5,2000,1000,2000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_JUPITEL','attack',84,28,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_JUPITEL','chase',84,28,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_THUNDERSTORM','attack',21,20,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_THUNDERSTORM','chase',21,20,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_COLDBOLT','attack',14,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_COLDBOLT','chase',14,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBALL','attack',17,44,3000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBALL','chase',17,44,3000,500,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_SIGHTRASHER','attack',81,10,3000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_QUAGMIRE','chase',92,5,3000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_SAFETYWALL','attack',12,10,3000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBOLT','chase',19,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREBOLT','attack',19,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREWALL','chase',18,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_FIREWALL','attack',18,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_LIGHTNINGBOLT','chase',20,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@MG_LIGHTNINGBOLT','attack',20,10,3000,1000,10000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_FROSTNOVA','chase',88,10,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_FROSTNOVA','attack',88,10,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_EARTHSPIKE','chase',90,5,3000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_EARTHSPIKE','attack',90,5,3000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_HEAVENDRIVE','chase',91,5,3000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (2999,'Morocc Necromancer@WZ_HEAVENDRIVE','attack',91,5,3000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_CRITICALWOUND','attack',673,2,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,5000,0,5000,'no','randomtarget','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,3000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','attack',83,11,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','attack',720,5,3000,0,5000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_FIREPILLAR','idle',80,7,2000,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_FIREPILLAR','attack',80,7,2000,0,5000,'no','around2','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','21',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','28',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','79',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','randomtarget','skillused','83',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@WZ_METEOR','idle',83,11,10000,0,10000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','18',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','89',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','21',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','28',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','79',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','skillused','83',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3000,'Morocc Necromancer@NPC_JACKFROST','idle',720,5,10000,0,10000,'no','self','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3004,'Morroc\'s Wraith@NPC_SELFDESTRUCTION','attack',173,1,10000,1700,0,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_POISON','attack',176,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_BLINDATTACK','attack',177,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_SILENCEATTACK','attack',178,5,5000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_STUNATTACK','attack',179,5,5000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_PETRIFYATTACK','attack',180,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_CURSEATTACK','attack',181,5,5000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_SLEEPATTACK','attack',182,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_HALLUCINATION','attack',207,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3006,'Morroc\'s Lude@NPC_BLEEDING','attack',660,5,2000,0,0,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3008,'Morroc Hidden@AL_HEAL','attack',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3008,'Morroc Hidden@AL_HEAL','idle',28,9,10000,500,5000,'no','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3026,'Fire Pit@NPC_DEATHSUMMON','dead',718,3,10000,0,10000,'no','self','always','0',3027,3027,3027,NULL,NULL,NULL,NULL);

# Episode 14.3: Isle of Bios
REPLACE INTO `mob_skill_db_re` VALUES (3029,'Grim Reaper Ankou@NPC_COMET','attack',708,3,1000,10000,75000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3029,'Grim Reaper Ankou@NPC_DARKCROSS','attack',338,10,500,2000,12000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3029,'Grim Reaper Ankou@NPC_DRAGONFEAR','attack',659,5,1000,3000,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3038,'Hidden Mob 7@NPC_INVISIBLE','idle',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3038,'Hidden Mob 7@NPC_INVISIBLE','attack',353,1,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3038,'Hidden Mob 7@NPC_HELLBURNING','idle',719,1,10000,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3038,'Hidden Mob 7@NPC_HELLBURNING','attack',719,1,10000,0,3000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Jitterbug instance
REPLACE INTO `mob_skill_db_re` VALUES (3069,'Ferre@NPC_GROUNDATTACK','attack',185,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3069,'Ferre@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',44,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3070,'Ferre@NPC_MAGICALATTACK','attack',192,1,500,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3070,'Ferre@NPC_REVERBERATION','idle',725,3,2000,1000,4000,'yes','around1','always','0',NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3070,'Ferre@NPC_REVERBERATION','attack',725,3,2000,1000,4000,'yes','around1','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3070,'Ferre@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',44,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@SM_BASH','attack',5,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@BS_HAMMERFALL','chase',110,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@BS_HAMMERFALL','attack',110,5,2000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@SM_ENDURE','attack',8,1,2000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@SM_ENDURE','chase',8,1,2000,0,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3071,'Ferre@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',44,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3072,'Ferre@NPC_WATERATTACK','attack',184,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3072,'Ferre@NPC_ENERGYDRAIN','attack',200,1,1000,100,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3072,'Ferre@NPC_EMOTION','idle',197,1,2000,0,5000,'no','self','always','0',44,0x3885,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3073,'Grand Pere@NPC_LEX_AETERNA','attack',727,5,5000,2000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3073,'Grand Pere@NPC_DARKNESSBREATH','attack',658,10,2000,2000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3073,'Grand Pere@NPC_WIDESILENCE','attack',663,5,2000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3073,'Grand Pere@NPC_WIDECONFUSE','attack',667,5,3000,2000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@AL_TELEPORT','idle',26,1,10000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@AL_TELEPORT','idle',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@AL_TELEPORT','walk',26,1,5000,0,5000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_AGIUP','attack',350,5,10000,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@AL_INCAGI','chase',29,10,2000,700,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@WZ_METEOR','attack',83,11,1000,500,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_HELLJUDGEMENT','chase',662,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_HELLJUDGEMENT','attack',662,10,3000,1000,10000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_PULSESTRIKE','attack',661,5,2000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_SUMMONSLAVE','attack',196,5,1000,700,10000,'no','self','slavele','0',2917,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3074,'Time Holder@NPC_SUMMONSLAVE','idle',196,5,1000,700,10000,'no','self','slavele','0',2917,NULL,NULL,NULL,NULL,NULL,NULL);

# Jitterbug instance
REPLACE INTO `mob_skill_db_re` VALUES (3108,'Jitterbug@NPC_WIDESILENCE','attack',663,5,3000,1000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3108,'Jitterbug@NPC_LEX_AETERNA','attack',727,3,2000,2000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3108,'Jitterbug@NPC_DARKNESSBREATH','attack',658,5,1000,2000,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3109,'Jitterbug@NPC_WIDECONFUSE','attack',667,5,3000,2000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3109,'Jitterbug@NPC_REVERBERATION','attack',725,3,2000,1000,4000,'yes','around1','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3109,'Jitterbug@NPC_LEX_AETERNA','attack',727,3,2000,2000,10000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3200,'Wicked Chimera@WZ_HEAVENDRIVE','attack',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3200,'Wicked Chimera@WZ_HEAVENDRIVE','chase',91,5,500,1200,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3200,'Wicked Chimera@CR_AUTOGUARD','attack',249,5,500,0,300000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3200,'Wicked Chimera@CR_AUTOGUARD','chase',249,5,2000,0,300000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

# Nightmare Biolab
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_VENOMDUST','idle',140,1,200,0,50000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_GRIMTOOTH','chase',137,5,200,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_CLOAKING','chase',135,1,200,200,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_GRIMTOOTH','chase',137,5,1000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@ASC_BREAKER','chase',379,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@ASC_METEORASSAULT','attack',406,10,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_SONICBLOW','attack',136,8,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@AS_VENOMDUST','attack',140,1,200,0,50000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@NPC_MAXPAIN','attack',716,5,300,1000,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@NPC_POISONATTACK','attack',188,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3208,'V_EREMES@NPC_VENOMIMPRESS','attack',738,1,200,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1645,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3208,'V_EREMES@NPC_STOP','attack',342,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@HP_ASSUMPTIO','idle',361,5,1000,1000,10000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@HP_ASSUMPTIO','idle',361,5,1000,1000,10000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_PNEUMA','idle',25,1,1000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@PR_LEXDIVINA','idle',76,10,1000,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3209,'V_MAGALETA@NPC_CHEAL','idle',729,5,300,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_HEAL','idle',28,9,300,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@HP_ASSUMPTIO','chase',361,5,1000,1000,10000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@HP_ASSUMPTIO','chase',361,5,1000,1000,10000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_DECAGI','chase',30,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@NPC_MAXPAIN','chase',716,5,1000,1000,60000,'no','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@PR_LEXDIVINA','chase',76,10,500,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@PR_LEXAETERNA','chase',78,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_HEAL','chase',28,9,300,0,2000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_HEAL','chase',28,9,300,0,2000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_PNEUMA','attack',25,1,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@AL_INCAGI','attack',29,10,200,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@MG_SAFETYWALL','attack',12,10,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@PR_LEXAETERNA','attack',78,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@NPC_HOLYATTACK','attack',189,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3209,'V_MAGALETA@NPC_CHEAL','attack',729,5,200,0,3000,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3209,'V_MAGALETA@NPC_CHEAL','attack',729,5,200,0,3000,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1640,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3209,'V_MAGALETA@NPC_LEX_AETERNA','attack',727,2,300,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3209,'V_MAGALETA@NPC_WIDESILENCE','attack',663,2,300,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@MG_SIGHT','idle',10,1,200,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_JUPITEL','chase',84,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_WATERBALL','chase',86,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@NPC_DARKSTRIKE','chase',340,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@MG_FIREWALL','chase',18,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@MG_FROSTDIVER','chase',15,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@HW_NAPALMVULCAN','chase',400,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_FROSTNOVA','chase',88,10,300,0,10000,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WL_EARTHSTRAIN','chase',2216,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_QUAGMIRE','chase',92,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@HW_GANBANTEIN','attack',483,1,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@NPC_DARKTHUNDER','attack',341,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_WATERBALL','attack',86,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@NPC_WIDEFREEZE','attack',664,3,300,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@NPC_DARKSTRIKE','attack',340,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WL_TETRAVORTEX','attack',2217,2,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_JUPITEL','attack',84,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WL_EARTHSTRAIN','attack',2216,3,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@WZ_SIGHTRASHER','attack',81,10,300,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1641,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3210,'V_KATRINN@MG_SAFETYWALL','attack',12,10,300,0,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_AGIUP','idle',350,1,1000,0,35000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@AC_DOUBLE','attack',46,10,200,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@AC_CHARGEARROW','attack',148,1,100,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_CRITICALWOUND','attack',673,1,100,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_AGIUP','attack',350,1,200,0,25000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@SN_SHARPSHOOTING','attack',382,5,300,500,1500,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_BLEEDING','attack',660,5,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_WIDECURSE','attack',677,3,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3211,'V_SHECIL@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1642,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3211,'V_SHECIL@NPC_ARROWSTORM','attack',728,3,300,500,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_EARTHQUAKE','chase',653,5,500,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@BS_ADRENALINE','attack',111,10,200,0,150000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@BS_MAXIMIZE','attack',114,1,100,0,60000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@MC_MAMMONITE','attack',42,9,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_HELMBRAKE','attack',345,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_POWERUP','attack',349,1,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_SPLASHATTACK','attack',174,1,500,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_ARMORBRAKE','attack',344,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1644,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3212,'V_HARWORD@NPC_WIDESTUN','attack',678,2,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3212,'V_HARWORD@NPC_MAGMA_ERUPTION','attack',733,3,500,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@SM_PROVOKE','chase',6,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@KN_SPEARBOOMERANG','chase',59,5,500,0,0,'yes','target','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@SM_ENDURE','chase',8,10,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@SM_BASH','attack',5,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@SM_MAGNUM','attack',7,9,100,0,0,'yes','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_POWERUP','attack',349,1,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@LK_SPIRALPIERCE','attack',397,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',1643,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_CRITICALWOUND','attack',673,2,200,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'5',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3213,'V_SEYREN@NPC_DRAGONBREATH','attack',731,8,400,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3213,'V_SEYREN@NPC_DRAGONBREATH','attack',731,3,400,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_MAXPAIN','attack',716,5,100,1000,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_EARTHQUAKE','attack',653,5,500,1000,6000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3213,'V_SEYREN@NPC_WIDEBLEEDING','attack',665,2,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@AS_VENOMDUST','idle',140,1,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@AS_GRIMTOOTH','chase',137,5,200,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@AS_GRIMTOOTH','chase',137,5,1000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@ASC_BREAKER','chase',379,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_INVISIBLE','chase',353,1,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_INVISIBLE','attack',353,1,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_AGIUP','attack',350,5,200,0,25000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@AS_SONICBLOW','attack',136,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@ASC_METEORASSAULT','attack',406,10,200,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_POISONATTACK','attack',188,10,100,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@AS_VENOMDUST','attack',140,1,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_MAXPAIN','attack',716,7,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_DRAGONFEAR','attack',659,1,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3214,'V_G_EREMES@NPC_VENOMFOG','attack',706,2,100,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3214,'V_G_EREMES@NPC_VENOMIMPRESS','attack',738,4,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@HP_ASSUMPTIO','idle',361,5,1000,1000,10000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@HP_ASSUMPTIO','idle',361,5,1000,1000,10000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_PNEUMA','idle',25,1,1000,0,0,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@PR_LEXDIVINA','idle',76,10,1000,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_CHEAL','idle',729,5,300,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_CHEAL','idle',729,5,300,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_LEX_AETERNA','chase',727,3,100,3000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@HP_ASSUMPTIO','chase',361,5,1000,1000,10000,'no','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@HP_ASSUMPTIO','chase',361,5,1000,1000,10000,'no','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_INCAGI','chase',29,10,200,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_DECAGI','chase',30,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@NPC_WIDESILENCE','chase',663,2,500,0,10000,'yes','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_PNEUMA','chase',25,1,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_HEAL','chase',28,10,300,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_HEAL','chase',28,10,300,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_INCAGI','attack',29,10,200,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@MG_SAFETYWALL','attack',12,10,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@AL_PNEUMA','attack',25,1,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@NPC_WIDESILENCE','attack',663,5,500,1000,6000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_LEX_AETERNA','attack',727,5,300,1000,6000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_CHEAL','attack',729,5,100,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3215,'V_G_MAGALETA@NPC_CHEAL','attack',729,5,100,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@NPC_MAXPAIN','attack',716,7,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@NPC_ALLHEAL','attack',687,1,100,3000,60000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3215,'V_G_MAGALETA@NPC_SLOWCAST','attack',672,5,100,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3216,'V_G_KATRINN@NPC_STORMGUST2','idle',723,3,100,3000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@MG_SIGHT','idle',10,1,50,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@MG_SIGHT','idle',10,1,1000,0,10000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@NPC_WIDEFREEZE','chase',664,5,300,3000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@HW_NAPALMVULCAN','chase',400,5,200,1000,2000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3216,'V_G_KATRINN@NPC_STORMGUST2','chase',723,3,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@WZ_VERMILION','chase',85,10,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@WZ_METEOR','chase',83,10,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@NPC_COMET','attack',708,5,300,1000,7000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3216,'V_G_KATRINN@NPC_STORMGUST2','attack',723,3,200,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@NPC_WIDESTONE','attack',666,1,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@HW_GANBANTEIN','attack',483,1,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@WL_EARTHSTRAIN','attack',2216,5,300,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@WZ_JUPITEL','attack',84,10,200,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@WL_TETRAVORTEX','attack',2217,5,300,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3216,'V_G_KATRINN@NPC_WIDEFREEZE','attack',664,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_AGIUP','idle',350,1,1000,0,600000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_EXPULSION','attack',674,1,10,3000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@AC_CHARGEARROW','attack',148,1,200,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@AC_DOUBLE','attack',46,10,200,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_AGIUP','attack',350,5,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_WIDECURSE','attack',677,5,100,1000,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_CRITICALWOUND','attack',673,1,100,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@SN_SHARPSHOOTING','attack',382,5,300,500,1500,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3217,'V_G_SHECIL@NPC_WIDECONFUSE','attack',667,5,1000,1000,20000,'no','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3217,'V_G_SHECIL@NPC_ARROWSTORM','attack',728,5,300,500,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_SELFDESTRUCTION','attack',173,1,200,2000,0,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@BS_ADRENALINE','attack',111,10,200,0,150000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@BS_MAXIMIZE','attack',114,1,100,0,60000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@MC_MAMMONITE','attack',42,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@BS_HAMMERFALL','attack',110,10,200,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_SPLASHATTACK','attack',174,1,100,0,0,'yes','target','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_HELMBRAKE','attack',345,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_EARTHQUAKE','attack',653,7,300,1000,60000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_WIDESTUN','attack',678,4,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_PULSESTRIKE','attack',661,5,100,1000,1000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3218,'V_G_HARWORD@NPC_MAXPAIN','attack',716,7,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3218,'V_G_HARWORD@NPC_MAGMA_ERUPTION','attack',733,5,500,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@KN_SPEARBOOMERANG','chase',59,5,1000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@SM_PROVOKE','chase',6,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@SM_ENDURE','chase',8,10,500,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3219,'V_G_SEYREN@NPC_DARKPIERCING','chase',715,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@KN_TWOHANDQUICKEN','attack',60,10,1000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@LK_SPIRALPIERCE','attack',397,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@NPC_POWERUP','attack',349,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3219,'V_G_SEYREN@NPC_DRAGONBREATH','attack',731,5,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3219,'V_G_SEYREN@NPC_DRAGONBREATH','attack',731,10,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@NPC_PULSESTRIKE','attack',661,5,200,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@NPC_MAXPAIN','attack',716,7,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@NPC_EARTHQUAKE','attack',653,10,500,0,6000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3219,'V_G_SEYREN@NPC_WIDEBLEEDING','attack',665,3,100,2000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_CALLSLAVE','idle',352,1,500,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3219,3218,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@AS_GRIMTOOTH','chase',137,5,200,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@ASC_BREAKER','chase',379,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@AS_GRIMTOOTH','chase',137,5,1000,0,0,'yes','target','skillused','18',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_INVISIBLE','chase',353,1,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_CALLSLAVE','attack',352,1,500,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3219,3218,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_INVISIBLE','attack',353,1,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_AGIUP','attack',350,5,200,0,25000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@AS_SONICBLOW','attack',136,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@ASC_METEORASSAULT','attack',406,10,200,2000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_POISONATTACK','attack',188,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_MAXPAIN','attack',716,9,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3220,'V_B_EREMES@NPC_VENOMIMPRESS','attack',738,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_PULSESTRIKE','attack',661,5,100,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_DRAGONFEAR','attack',659,5,100,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_VENOMFOG','attack',706,5,100,2000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3220,'V_B_EREMES@NPC_EARTHQUAKE','attack',653,10,1000,4000,60000000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3216,3217,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','idle',729,5,300,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','idle',729,5,300,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','chase',729,5,300,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','chase',729,5,300,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_WIDESILENCE','chase',663,5,500,1000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_LEX_AETERNA','chase',727,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'yes','self','myhpltmaxrate','30',3219,3214,3218,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@AL_INCAGI','attack',29,10,200,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@MG_SAFETYWALL','attack',12,10,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@AL_PNEUMA','attack',25,1,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','attack',729,5,100,0,1500,'yes','friend','friendhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_CHEAL','attack',729,5,100,0,1500,'yes','self','myhpltmaxrate','99',NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_MAXPAIN','attack',716,10,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_ALLHEAL','attack',687,1,100,3000,60000000,'no','self','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_SLOWCAST','attack',672,5,100,0,240000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_WIDESILENCE','attack',663,5,500,1000,6000,'no','target','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'18',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3221,'V_B_MAGALETA@NPC_LEX_AETERNA','attack',727,5,300,1000,6000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3221,'V_B_MAGALETA@NPC_WIDECONFUSE','attack',667,5,200,1000,60000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_AGIUP','idle',350,5,200,0,60000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3215,3216,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3215,3216,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3219,3214,3218,3215,3216,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3222,'V_B_SHECIL@NPC_ARROWSTORM','attack',728,5,200,0,3000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_POWERUP','attack',349,5,200,0,20000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_AGIUP','attack',350,5,200,0,20000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@AC_DOUBLE','attack',46,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_CRITICALWOUND','attack',673,5,50,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@SN_SHARPSHOOTING','attack',382,5,200,500,1500,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_WIDEBLEEDING','attack',665,5,1000,1000,10000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3222,'V_B_SHECIL@NPC_WIDECURSE','attack',677,5,1000,1000,10000,'no','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@BS_HAMMERFALL','chase',110,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'yes','self','myhpltmaxrate','30',3219,3214,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@BS_ADRENALINE','attack',111,10,200,0,150000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@BS_MAXIMIZE','attack',114,1,100,0,60000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@MC_MAMMONITE','attack',42,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'23',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@BS_HAMMERFALL','attack',110,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_MAXPAIN','attack',716,10,500,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_WATERATTACK','attack',184,10,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3223,'V_B_HARWORD@NPC_MAGMA_ERUPTION','attack',733,5,500,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_WIDESTUN','attack',678,5,200,2000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_PULSESTRIKE','attack',661,5,200,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3223,'V_B_HARWORD@NPC_EARTHQUAKE','attack',653,10,1000,4000,60000000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3215,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@MG_SIGHT','idle',10,1,1000,0,10000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3219,3214,3218,3215,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3219,3214,3218,3215,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_COMET','attack',708,5,300,1000,7000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3224,'V_B_KATRINN@NPC_STORMGUST2','attack',723,3,200,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_WIDESTONE','attack',666,1,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@HW_GANBANTEIN','attack',483,1,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@WL_EARTHSTRAIN','attack',2216,5,300,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@WZ_JUPITEL','attack',84,10,200,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@WL_TETRAVORTEX','attack',2217,5,300,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_WIDEFREEZE','attack',664,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3224,'V_B_KATRINN@NPC_WIDESILENCE','attack',663,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_CALLSLAVE','idle',352,1,500,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3214,3218,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@KN_SPEARBOOMERANG','chase',59,5,1000,0,0,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@SM_ENDURE','chase',8,10,500,0,30000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3214,3218,3215,3216,3217,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@KN_TWOHANDQUICKEN','attack',60,10,1000,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'2',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@LK_SPIRALPIERCE','attack',397,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3225,'V_B_SEYREN@NPC_DRAGONBREATH','attack',731,10,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3225,'V_B_SEYREN@NPC_DRAGONBREATH','attack',731,5,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_CRITICALSLASH','attack',170,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_COMBOATTACK','attack',171,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_MAXPAIN','attack',716,9,300,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_WIDEBLEEDING','attack',665,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_PULSESTRIKE','attack',661,5,200,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_CRITICALWOUND','attack',673,5,200,0,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3225,'V_B_SEYREN@NPC_EARTHQUAKE','attack',653,10,1000,4000,60000000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@PA_PRESSURE','attack',367,1,200,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@CR_SHIELDCHARGE','attack',250,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@CR_REFLECTSHIELD','attack',252,5,50,0,200000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@CR_REFLECTSHIELD','attack',252,5,1000,0,200000,'yes','self','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@NPC_DARKCROSS','attack',338,10,100,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@NPC_GRANDDARKNESS','attack',339,10,200,500,5000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@CR_GRANDCROSS','attack',254,10,200,500,5000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@NPC_DEFENDER','attack',205,5,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@NPC_POWERUP','attack',349,1,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3226,'V_RANDEL@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2232,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3226,'V_RANDEL@NPC_RAYOFGENESIS','attack',737,3,200,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','idle',231,4,1000,0,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','idle',231,4,1000,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_ACIDTERROR','chase',230,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','chase',231,5,500,0,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','chase',231,5,500,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@CR_ACIDDEMONSTRATION','attack',490,5,100,1000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_DEMONSTRATION','attack',229,1,200,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@NPC_AGIUP','attack',350,1,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','attack',231,5,500,0,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3227,'V_FLAMEL@NPC_MANDRAGORA','attack',735,3,500,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@AM_POTIONPITCHER','attack',231,5,500,0,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@NPC_WIDESLEEP','attack',668,3,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@NPC_MAXPAIN','attack',716,5,300,1000,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2230,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3227,'V_FLAMEL@NPC_WIDESTUN','attack',678,3,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@MG_SIGHT','idle',10,1,200,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@MG_SIGHT','idle',10,1,1000,0,10000,'yes','self','skillused','137',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3228,'V_CELIA@NPC_WIDEWEB','chase',721,1,500,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3228,'V_CELIA@NPC_PSYCHIC_WAVE','chase',736,1,500,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@SA_DISPELL','chase',289,5,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@MG_THUNDERSTORM','chase',21,10,300,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@NPC_WIDESOULDRAIN','chase',680,4,300,0,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@NPC_MENTALBREAKER','attack',159,3,50,800,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@SA_DISPELL','attack',289,5,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3228,'V_CELIA@NPC_PSYCHIC_WAVE','attack',736,4,500,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@MG_SAFETYWALL','attack',12,10,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2231,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3228,'V_CELIA@NPC_WIDEWEB','attack',721,1,300,0,7000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@NPC_WIDESTONE','attack',666,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3228,'V_CELIA@NPC_WIDESOULDRAIN','attack',680,2,100,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@MO_BODYRELOCATION','chase',264,1,200,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'28',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@AL_DECAGI','chase',30,10,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@MO_BALKYOUNG','attack',1016,1,10,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@MO_EXTREMITYFIST','attack',271,1,500,1000,6000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@NPC_CRITICALSLASH','attack',170,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@NPC_CHANGEUNDEAD','attack',348,3,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3229,'V_CHEN@NPC_CURSEDCIRCLE','attack',730,2,300,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'3',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2229,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3229,'V_CHEN@NPC_WIDEBLEEDING','attack',665,4,300,1000,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@AS_CLOAKING','attack',135,1,200,200,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@AS_CLOAKING','idle',135,1,200,200,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@AS_CLOAKING','chase',135,1,200,200,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@ST_FULLSTRIP','attack',476,5,500,0,60000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_PULSESTRIKE','attack',661,2,200,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_CRITICALSLASH','attack',170,1,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_WIDECONFUSE','attack',667,3,100,1000,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3230,'V_GERTIE@NPC_FATALMENACE','attack',732,3,300,0,7000,NULL,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'43,');
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_VENOMFOG','attack',706,1,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_AGIUP','attack',350,1,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2228,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3230,'V_GERTIE@RG_CLOSECONFINE','attack',1005,1,100,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3231,'V_ALPHOCCIO@NPC_DISSONANCE','attack',2540,3,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@NPC_PULSESTRIKE','attack',661,3,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@NPC_WIDEFREEZE','attack',664,3,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@NPC_SLOWCAST','attack',672,1,200,800,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3231,'V_ALPHOCCIO@NPC_REVERBERATION','attack',725,3,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3231,'V_ALPHOCCIO@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2234,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3231,'V_ALPHOCCIO@NPC_DANCINGBLADE','attack',713,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3232,'V_TRENTINI@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3232,'V_TRENTINI@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3232,'V_TRENTINI@NPC_UGLYDANCE','attack',2541,5,100,1000,100000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3232,'V_TRENTINI@NPC_AGIUP','attack',350,1,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3232,'V_TRENTINI@NPC_CRITICALSLASH','attack',170,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3232,'V_TRENTINI@NPC_REVERBERATION','attack',725,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3232,'V_TRENTINI@NPC_SUMMONSLAVE','attack',196,1,50,0,600000,'yes','self','slavele','0',2233,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3232,'V_TRENTINI@NPC_DISSONANCE','attack',2540,3,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_DEFENDER','attack',205,5,500,0,10000,'yes','self','longrangeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@PA_PRESSURE','attack',367,5,100,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@CR_SHIELDCHARGE','attack',250,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@CR_REFLECTSHIELD','attack',252,5,50,0,200000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@CR_REFLECTSHIELD','attack',252,5,1000,0,200000,'yes','self','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_DARKCROSS','attack',338,10,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_GRANDDARKNESS','attack',339,10,200,500,10000,'no','self','attackpcge','2',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_POWERUP','attack',349,5,500,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3233,'V_G_RANDEL@NPC_RAYOFGENESIS','attack',737,7,200,500,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_WIDESLEEP','attack',668,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3233,'V_G_RANDEL@NPC_WIDESILENCE','attack',663,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','idle',231,5,1000,500,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','idle',231,5,1000,500,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','chase',231,5,1000,500,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','chase',231,5,1000,500,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_DEMONSTRATION','chase',229,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_ACIDTERROR','chase',230,5,150,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'29',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_AGIUP','attack',350,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','attack',231,5,1000,500,10000,'yes','friend','friendhpltmaxrate','60',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_POTIONPITCHER','attack',231,5,1000,500,10000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,'5',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_MAXPAIN','attack',716,7,300,1000,60000,'no','self','myhpltmaxrate','3',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@CR_ACIDDEMONSTRATION','attack',490,3,100,1000,300000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@AM_DEMONSTRATION','attack',229,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_CRITICALWOUND','attack',673,3,100,0,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3234,'V_G_FLAMEL@NPC_MANDRAGORA','attack',735,3,500,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_WIDESLEEP','attack',668,5,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3234,'V_G_FLAMEL@NPC_WIDESTUN','attack',678,5,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@MG_FIREBOLT','chase',19,10,300,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@SA_DISPELL','chase',289,5,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_WIDESOULDRAIN','attack',680,5,200,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_MENTALBREAKER','attack',159,3,50,800,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3235,'V_G_CELIA@NPC_PSYCHIC_WAVE','attack',736,5,300,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@SA_DISPELL','attack',289,5,50,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@SA_LANDPROTECTOR','attack',288,5,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3235,'V_G_CELIA@NPC_WIDEWEB','attack',721,1,500,0,7000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_WIDESTONE','attack',666,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_WIDESOULDRAIN','attack',680,4,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3235,'V_G_CELIA@NPC_PSYCHIC_WAVE','attack',736,5,300,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_WIDECONFUSE','attack',667,3,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3235,'V_G_CELIA@NPC_MAXPAIN','attack',716,7,300,1000,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3235,'V_G_CELIA@NPC_PSYCHIC_WAVE','attack',736,7,300,3000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@NPC_WIDEBLEEDING','attack',665,5,300,1000,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@MO_BALKYOUNG','attack',1016,1,10,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'19',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@MO_EXTREMITYFIST','attack',271,5,500,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@NPC_CRITICALSLASH','attack',170,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@NPC_ARMORBRAKE','attack',344,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3236,'V_G_CHEN@NPC_PULSESTRIKE','attack',661,5,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3236,'V_G_CHEN@NPC_CURSEDCIRCLE','attack',730,4,100,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3236,'V_G_CHEN@NPC_DARKPIERCING','attack',715,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_PULSESTRIKE','attack',661,5,200,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_CRITICALSLASH','attack',170,1,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_AGIUP','attack',350,1,300,0,300000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@ST_FULLSTRIP','attack',476,5,50,0,30000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3237,'V_G_GERTIE@NPC_FATALMENACE','attack',732,5,300,0,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'34',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_WIDECONFUSE','attack',667,5,100,1000,30000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@RG_CLOSECONFINE','attack',1005,1,100,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_VENOMFOG','attack',706,3,200,0,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_WIDECURSE','attack',677,5,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3237,'V_G_GERTIE@NPC_INVISIBLE','attack',353,1,100,200,30000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3238,'V_G_ALPHOCCIO@NPC_DISSONANCE','attack',2540,3,100,1000,120000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3238,'V_G_ALPHOCCIO@NPC_PULSESTRIKE','attack',661,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3238,'V_G_ALPHOCCIO@NPC_REVERBERATION','attack',725,3,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3238,'V_G_ALPHOCCIO@NPC_WIDESLEEP','attack',668,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3238,'V_G_ALPHOCCIO@NPC_WIDEFREEZE','attack',664,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3238,'V_G_ALPHOCCIO@NPC_SLOWCAST','attack',672,3,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3238,'V_G_ALPHOCCIO@NPC_WIDESOULDRAIN','attack',680,6,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3239,'V_G_TRENTINI@NPC_UGLYDANCE','attack',2541,5,100,1000,120000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_PULSESTRIKE','attack',661,3,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_MAXPAIN','attack',716,5,300,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_WIDESILENCE','attack',663,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_WIDESTUN','attack',678,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_WIDEFREEZE','attack',664,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3239,'V_G_TRENTINI@NPC_WIDECURSE','attack',677,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3234,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3234,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3234,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@PA_PRESSURE','attack',367,5,200,0,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@CR_REFLECTSHIELD','attack',252,5,10,0,200000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@CR_REFLECTSHIELD','attack',252,5,1000,0,200000,'yes','self','casttargeted',NULL,NULL,NULL,NULL,NULL,NULL,'32',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@CR_GRANDCROSS','attack',254,10,300,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_GRANDDARKNESS','attack',339,10,200,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_POWERUP','attack',349,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3240,'V_B_RANDEL@NPC_RAYOFGENESIS','attack',737,10,200,500,7000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_HELLJUDGEMENT','attack',662,10,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_WIDESILENCE','attack',663,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3240,'V_B_RANDEL@NPC_WIDESLEEP','attack',668,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@AM_DEMONSTRATION','chase',229,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_MAXPAIN','attack',716,10,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3235,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@CR_ACIDDEMONSTRATION','attack',490,5,100,1000,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_PULSESTRIKE','attack',661,5,100,0,5000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_MAGICMIRROR','attack',671,10,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@AM_DEMONSTRATION','attack',229,5,200,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3241,'V_B_FLAMEL@NPC_MANDRAGORA','attack',735,5,500,500,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_WIDESLEEP','attack',668,5,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3241,'V_B_FLAMEL@NPC_WIDESTUN','attack',678,5,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_MENTALBREAKER','chase',159,5,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@SA_DISPELL','chase',289,5,500,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_DARKTHUNDER','chase',341,10,200,500,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@MG_COLDBOLT','chase',14,10,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@MG_FIREBOLT','chase',19,10,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@MG_LIGHTNINGBOLT','chase',20,10,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_WIDESTONE','chase',666,5,300,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3234,3236,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@SA_LANDPROTECTOR','attack',288,5,300,0,20000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_MENTALBREAKER','attack',159,5,50,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'7',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@SA_DISPELL','attack',289,5,1000,0,30000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_DARKTHUNDER','attack',341,10,300,1000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@MG_SAFETYWALL','attack',12,10,300,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3242,'V_B_CELIA@NPC_WIDEWEB','attack',721,1,500,0,7000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_WIDESTONE','attack',666,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3242,'V_B_CELIA@NPC_WIDESOULDRAIN','attack',680,5,100,2000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3242,'V_B_CELIA@NPC_PSYCHIC_WAVE','attack',736,7,500,500,5000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3242,'V_B_CELIA@NPC_PSYCHIC_WAVE','attack',736,10,300,3000,10000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'9',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3234,3235,3237,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@MO_EXTREMITYFIST','attack',271,5,300,1000,7000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@MO_EXTREMITYFIST','attack',271,5,100,1000,3000,'no','target','myhpltmaxrate','10',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_POWERUP','attack',349,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_AGIUP','attack',350,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_MAXPAIN','attack',716,10,300,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3243,'V_B_CHEN@NPC_DARKPIERCING','attack',715,1,100,0,10000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_PULSESTRIKE','attack',661,5,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_WIDEBLEEDING','attack',665,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3243,'V_B_CHEN@NPC_WIDESILENCE','attack',663,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3234,3235,3236,3238,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_AGIUP','attack',350,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_AGIUP','attack',350,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@RG_STRIPSHIELD','attack',216,5,50,0,2000,'yes','target','myhpltmaxrate','90',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@RG_STRIPARMOR','attack',217,5,50,0,2000,'yes','target','myhpltmaxrate','70',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@RG_STRIPWEAPON','attack',215,5,50,0,2000,'yes','target','myhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@ST_FULLSTRIP','attack',476,5,50,0,20000,'yes','target','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_PULSESTRIKE','attack',661,5,200,0,7000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_CRITICALSLASH','attack',170,1,300,0,5000,'yes','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3244,'V_B_GERTIE@NPC_FATALMENACE','attack',732,5,100,0,30000,NULL,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,'37,');
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_HELLJUDGEMENT','attack',662,10,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3244,'V_B_GERTIE@NPC_WIDESLEEP','attack',668,10,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3237,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_SLOWCAST','chase',672,5,200,300,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'36',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3237,'3239',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3234,3235,3236,3237,'3239',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3245,'V_B_ALPHOCCIO@NPC_ASSASSINCROSS','attack',2539,10,100,1000,150000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_POWERUP','attack',349,5,200,0,100000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_POWERUP','attack',349,5,200,0,25000,'yes','self','myhpltmaxrate','30',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_MAXPAIN','attack',716,9,300,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_WIDESLEEP','attack',668,5,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_WIDEFREEZE','attack',664,5,100,1000,30000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_PULSESTRIKE','attack',661,5,100,1000,10000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3245,'V_B_ALPHOCCIO@NPC_WIDESOULDRAIN','attack',680,10,100,1000,60000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@AL_TELEPORT','idle',26,1,1000,0,0,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_SUMMONSLAVE','idle',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3237,'3238',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_CALLSLAVE','idle',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_MAXPAIN','idle',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@AL_TELEPORT','walk',26,1,500,0,10000,'yes','self','rudeattacked',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_MAXPAIN','chase',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_MAXPAIN','attack',716,5,1000,0,10000,'yes','self','skillused','490',NULL,NULL,NULL,NULL,NULL,'8',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_CALLSLAVE','attack',352,1,1000,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_SUMMONSLAVE','attack',196,1,1000,700,10000,'no','self','slavele','3',3233,3234,3235,3236,3237,'3238',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_SUMMONMONSTER','attack',209,1,1000,700,60000000,'no','self','myhpltmaxrate','30',3233,3234,3235,3236,3237,'3238',NULL);
#REPLACE INTO `mob_skill_db_re` VALUES ( 3246,'V_B_TRENTINI@NPC_UGLYDANCE','attack',2541,5,100,1000,70000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_MAXPAIN','attack',716,9,300,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,'47',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_PULSESTRIKE','attack',661,3,100,0,10000,'yes','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDESILENCE','attack',663,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDESTUN','attack',678,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDEFREEZE','attack',664,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDECURSE','attack',677,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDEBLEEDING','attack',665,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDESTONE','attack',666,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDESLEEP','attack',668,5,100,1000,20000,'no','self','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3246,'V_B_TRENTINI@NPC_WIDECONFUSE','attack',667,5,100,1000,20000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

#=================================================================
# Phantasmagorika 15.2 Mob Skills Placeholder (Gathered from jRO)
#=================================================================

REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_BLOODDRAIN','angry',199,1,1000,0,3000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_STONESKIN','attack',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_STUNATTACK','attack',179,5,800,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_ARMORBRAKE','attack',344,10,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_STONESKIN','chase',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_BLOODDRAIN','attack',199,1,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@NPC_STONESKIN','idle',675,3,10000,0,30000,'yes','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3153,'Excavator Robot@AL_TELEPORT','idle',26,1,10000,0,0,'no','target','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3154,'Recon Robot@NPC_GUIDEDATTACK','attack',172,5,500,0,20000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3154,'Recon Robot@NPC_STUNATTACK','attack',179,5,800,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3155,'Repair Robot@NPC_STUNATTACK','any',179,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3155,'Repair Robot@NPC_GUIDEDATTACK','attack',172,5,1000,0,15000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3156,'Exploration Rover@NPC_SUMMONSLAVE','idle',196,5,10000,700,10000,'no','self','casttargeted','1',3157,3158,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3156,'Exploration Rover@NPC_GUIDEDATTACK','attack',172,5,1000,0,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3156,'Exploration Rover@NPC_SUMMONSLAVE','attack',196,5,10000,700,10000,'no','target','attackpcge','10',3157,3158,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3247,'Cenere G@NPC_PETRIFYATTACK','attack',180,3,500,500,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3247,'Cenere G@MG_THUNDERSTORM','chase',21,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3247,'Cenere G@NPC_POISONATTACK','any',188,1,5000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3247,'Cenere G@NPC_BLINDATTACK','attack',177,3,500,0,5000,'yes','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3247,'Cenere G@MG_THUNDERSTORM','attack',21,10,3000,1000,10000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@AL_PNEUMA','attack',25,1,2000,0,5000,'yes','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@AL_TELEPORT','idle',26,1,10000,0,0,'no','target','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@NPC_CRITICALSLASH','attack',170,1,2000,0,2000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@NPC_GUIDEDATTACK','attack',172,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','target','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@NPC_STUNATTACK','attack',179,5,500,1000,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3248,'Repair Robot T@AL_PNEUMA','chase',25,1,2000,0,5000,'no','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@KN_TWOHANDQUICKEN','attack',60,30,10000,0,120000,'yes','self','myhpltmaxrate','40',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@AL_PNEUMA','chase',25,1,2000,0,5000,'no','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@AL_TELEPORT','walk',26,1,10000,0,0,'no','target','rudeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@NPC_CRITICALWOUND','attack',673,2,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@NPC_GUIDEDATTACK','attack',172,5,1000,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3249,'Exploration Rover T@AL_PNEUMA','attack',25,1,2000,0,5000,'no','self','longrangeattacked','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3253,'SYS_MSG@AL_HEAL','attack',28,10,500,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3253,'SYS_MSG@AL_HEAL','chase',28,10,500,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3253,'SYS_MSG@AL_HEAL','idle',28,10,500,500,5000,'yes','friend','friendhpltmaxrate','50',NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_PIERCINGATT','attack',158,10,300,0,30000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_PIERCINGATT','chase',158,10,500,0,5000,'no','target','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_SILENCEATTACK','attack',178,5,750,250,10000,'no','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_SILENCEATTACK','chase',178,5,750,250,3000,'no','target','casttargeted','0',NULL,NULL,NULL,NULL,NULL,'6',NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_WIDESLEEP','attack',668,5,400,0,15000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_WIDESTONE','attack',666,5,600,0,30000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_WIDESOULDRAIN','attack',680,10,1000,0,60000,'no','self','always','0',NULL,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3254,'T_W_O@NPC_CRITICALWOUND','attack',673,5,150,500,45000,'no','target','always',NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

REPLACE INTO `mob_skill_db_re` VALUES (3505,'DR_BIG_EGGRING@NPC_SUMMONSLAVE','attack',196,4,10000,2000,10000,'no','self','slavele','3',3508,NULL,NULL,NULL,NULL,NULL,NULL);
REPLACE INTO `mob_skill_db_re` VALUES (3505,'DR_BIG_EGGRING@NPC_SUMMONSLAVE','idle',196,4,10000,2000,10000,'no','self','slavele','3',3508,NULL,NULL,NULL,NULL,NULL,NULL);


